package de.dfki.km.academicresearch.eswc2016challenge.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;

public class Path implements Comparable<Path> {

    private List<Resource> path;

    public Path(String line) throws Exception {
        path = new LinkedList<>();

        boolean resOrProp = true;

        line = line.substring(1, line.length() - 1);
        for (String entry : line.split(",")) {
            entry = entry.trim();
            entry = entry.substring(1, entry.length() - 1);

            Resource res;

            if (resOrProp) {
                res = ResourceFactory.createResource(entry);
            } else {
                res = ResourceFactory.createProperty(entry);
            }

            path.add(res);

            resOrProp = !resOrProp;
        }

        //check that the resouce count is odd!
        if ((path.size() & 1) == 0) {
            //even
            throw new Exception("Path resource count is even");
        }
    }

    public Path() {
        this.path = new ArrayList<>();
    }

    public Path(Resource... resources) {
        this.path = Arrays.asList(resources);
    }

    public Path(List<Resource> resources) {
        this.path = resources;
    }

    public void add(Resource r) {
        this.path.add(r);
    }

    public int length() {
        return path.size() / 2;//alsways rounded down , which is right
    }

    public int resourceCount() {
        return path.size();
    }
    
    
    public String toSourceCode(String dataSet){
    	
    	//Result: 
    	//expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	
    	StringBuilder sb = new StringBuilder();        
        sb.append("expected.add(new Path(");        
        
        for(int i=0; i < path.size() ; i++){
        	
        	sb.append(dataSet);
        	sb.append(".");
        	sb.append(path.get(i).getLocalName());
        	
        	if(i!= path.size() - 1){
        		//normal
        		sb.append(", ");
        	}
        }
        
        sb.append("));");
        return sb.toString();
    	
    	
    }
    
    
    

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        
        sb.append("(");
        sb.append(path.size());
        sb.append(")");
        
        sb.append(" [");

        List<String> strings = new LinkedList<>();
        for (Resource r : path) {
            strings.add("\"" + r.getURI() + "\"");
        }

        sb.append(StringUtils.join(strings, ", "));
        sb.append("]");
        return sb.toString();
    }

    public List<Resource> getPath() {
        return path;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((path == null) ? 0 : path.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Path other = (Path) obj;
        if (path == null) {
            if (other.path != null) {
                return false;
            }
        } else if (!path.equals(other.path)) {
            return false;
        }
        return true;
    }

    @Override
    public int compareTo(Path o) {
        return Integer.compare(this.resourceCount(), o.resourceCount());
    }

}
