package de.dfki.km.academicresearch.eswc2016challenge.pathfinder;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import org.apache.jena.graph.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class DefaultPathFinder extends PathFinder {

	private static final Logger logger = LoggerFactory.getLogger(DefaultPathFinder.class);
	
	@Override
	public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
		// TODO Auto-generated method stub
		return new LinkedList<>();
	}
	
	@Override
	public boolean hasIndex() {
		return true; //we dont want the streaming
	}
	
	@Override
	public void send(Triple item) {
		//logger.debug(item.toString());
	}

	@Override
	public void endStreaming() throws Exception {
		//use getIndexFile() to retrive the file for indces	
	}
	
	@Override
	public void loadIndex() throws Exception {
		
	}
	
	//what we need:
	//-k shortest path
	//-possible multiple edges between two nodes -> multigraph
	//-we have directed edges -> directed graph
	//-loopfree BUT only dont use same triple
	//-start or end with a spefic property (property pathes)
	
	//loopfree(no duplicate triples) top k shortest path in directed multigraphs with additional edge restrictions !!!!!!
	
	
	//we have:
	//- top k shortest trail (all edges are distinct)
	//https://en.wikipedia.org/wiki/Glossary_of_graph_theory#Walks
	//https://en.wikipedia.org/wiki/Path_(graph_theory)
	
    
    //IDEAS and papers:
	//K*: A heuristic search algorithm for finding the k shortest paths
	//http://www.sciencedirect.com/science/article/pii/S0004370211000865
    //---
    //Fast and Accurate Estimation of Shortest Paths in Large Graphs
    //https://people.mpi-inf.mpg.de/~sseufert/papers/aspsn-cikm.pdf
    //---
    //Fast top-k simple shortest paths discovery in graphs
    //slides : http://de.slideshare.net/redhatdb/topk-shortest-path
    //paper: http://dl.acm.org/citation.cfm?id=1871504
    //---
    //Efficient Top-k Shortest-Path Distance Queries on Large Networks by Pruned Landmark Labeling
    //https://www.aaai.org/ocs/index.php/AAAI/AAAI15/paper/viewFile/9320/9217
    //github: https://github.com/flowlight0/top-k-pruned-landmark-labeling
	//---
	//On the Difficulty of Some Shortest Path Problems
	//https://www.cs.ucsb.edu/~suri/psdir/stacs03.pdf
    //---
    //Holistic Top-k Simple Shortest Path Join in Graphs
    //http://ieeexplore.ieee.org/xpl/login.jsp?tp=&arnumber=5871619&url=http%3A%2F%2Fieeexplore.ieee.org%2Fiel5%2F69%2F4358933%2F05871619.pdf%3Farnumber%3D5871619
    //---
    //Efficiently Computing Top-K Shortest Path Join
    //https://openproceedings.org/2015/conf/edbt/paper-132.pdf
	//--
	//A lazy version of Eppstein's K shortest paths algorithm
	//http://dl.acm.org/citation.cfm?id=1764163
	//---
	//Finding the k Shortest Paths
	//http://citeseer.ist.psu.edu/viewdoc/download;jsessionid=7BB56B2ABC7C9113C121413A62AF3974?doi=10.1.1.30.3705&rep=rep1&type=pdf
	
	//http://www.mat.uc.pt/~eqvm/OPP/KSPP/KSPP.html
	//http://www.mat.uc.pt/~eqvm/cientificos/investigacao/r_papers.html#K
    
    //survey:
    //http://pages.cs.wisc.edu/~xfeng/sides/top_k_paths_survey.pdf
	
	//INFO:
	//https://www.w3.org/TR/sparql11-query/#propertypaths
	//https://www.w3.org/TR/sparql11-query/#propertypaths
	
	//http://letitcrash.com/post/30257014291/distributed-in-memory-graph-processing-with-akka
	
	
	//_____________
	
	
	//https://www.safaribooksonline.com/library/view/graph-databases/9781449356255/ch01.html
	//http://radar.oreilly.com/2013/05/improving-options-for-unlocking-your-graph-data.html
	//https://dzone.com/articles/google-pregel-graph-processing
	
	
	
	
	//GOOD: https://code.google.com/p/k-shortest-paths/
	//A NEW IMPLEMENTATION OF YEN'S RANKING LOOPLESS PATHS ALGORITHM
	//http://www.mat.uc.pt/~marta/Publicacoes/opt01.ps.gz
	
	//https://github.com/yan-qi/k-shortest-paths-java-version
    //https://code.google.com/p/k-shortest-paths/downloads/detail?name=KShortestPaths_Java_v2.3.zip
	
	
	
	
	//K shortest path :
	//with loop , then eppstein is good
	//without loop , then yen's algorithmn is good.
	//see https://en.wikipedia.org/wiki/K_shortest_path_routing
	
	//yen:
	//https://en.wikipedia.org/wiki/Yen%27s_algorithm
	
	//eppstein
	//https://www.ics.uci.edu/~eppstein/pubs/Epp-SJC-98.pdf
	//http://www.isi.edu/natural-language/people/epp-cs562.pdf
	//http://cstheory.stackexchange.com/questions/9366/finding-k-shortest-paths-with-eppsteins-algorithm
	//impl: http://www.ics.uci.edu/~eppstein/pubs/p-kpath.html
	//http://sourceforge.net/p/mulavito/git/ci/69088093932ef616ea88e0dd85b2a499b9961efa/tree/src/mulavito/algorithms/shortestpath/ksp/Eppstein.java#l8
	//Implementations: 
	//https://www.ics.uci.edu/~eppstein/pubs/p-kpath.html
	//http://www.mat.uc.pt/~eqvm/cientificos/fortran/codigos.html
	//
	//http://www.powershow.com/view/1806a1-NDcyN/k_Shortest_Paths_powerpoint_ppt_presentation
	//https://wiki.bordeaux.inria.fr/realopt/uploads/Project/Eppstein.ppt
	//https://github.com/nickdinges/k-shortest-path
	
	//together:
	//http://stackoverflow.com/questions/12870122/eppsteins-algorithm-and-yens-algorithm-for-k-shortest-paths
	
	//-------------------
	//giraph
	//http://buttercola.blogspot.de/2015/04/giraph-single-source-shortest-path-sssp.html
	//http://www2.cs.uh.edu/~gabriel/courses/cosc6339_s15/BDA_18_GraphAlgorithms_Giraph.pdf
	//-------
	//bidriecttional:
	//http://stackoverflow.com/questions/22233210/how-to-speed-up-breadth-first-search-in-a-undirected-graph
	//https://en.wikipedia.org/wiki/Bidirectional_search
	//http://courses.csail.mit.edu/6.006/fall11/lectures/lecture18.pdf
}
