package de.dfki.km.academicresearch.eswc2016challenge.pathfinder;

import java.util.LinkedList;
import java.util.List;

import org.apache.jena.graph.Triple;
import org.apache.jena.ontology.OntTools;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.rdf.model.impl.StatementImpl;
import org.apache.jena.sparql.algebra.Algebra;
import org.apache.jena.sparql.algebra.Op;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;

public class JenaPathFinder extends PathFinder{
	
	private static final Logger logger = LoggerFactory.getLogger(JenaPathFinder.class);
	
	//Jena also have path implementations!
	
	//org.apache.jena.sparql.path.Path pOne;		
	
	//Look at:
	//org.apache.jena.ontology.OntTools.Path pTwo = org.apache.jena.ontology.OntTools.findShortestPath(m, start, end, onPath);
			
	
	//Also important:
	//http://answers.semanticweb.com/questions/12994/can-jena-arq-show-property-path-details
	//http://stackoverflow.com/questions/26765885/jena-property-paths-when-building-queries-programatically
	//https://jena.apache.org/documentation/query/property_paths.html
	
	private Model m;
	
	public JenaPathFinder() {
		m = ModelFactory.createDefaultModel();
	}

	@Override
	public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
		//return firstApproach(srcURI, dstURI, k);
		return seconfdApproach(srcURI, dstURI, k);		
	}
	
	private List<Path> firstApproach(String srcURI, String dstURI, int k) throws Exception {
		String strQuery = String.format(
				"PREFIX : <http://doesnotexist.org>\r\n" + 
		        "SELECT *\r\n" + 
		        //"WHERE  {<%s> !:* <%s>.}",
		        "WHERE  {<%s> !:* ?dst.}",
		        srcURI, dstURI);
		
		Query query = QueryFactory.create(strQuery) ;
        Op op = Algebra.compile(query) ;
        logger.info("Parsed query: {}", op);
		
		
		QueryExecution qexec =QueryExecutionFactory.create(query, m);
    	
    	try{
    		ResultSet rs=qexec.execSelect();
    		ResultSetFormatter.out(System.out, rs) ;
    	}finally{
    		qexec.close();
    	}
		         
		
		return new LinkedList<>();
	
	}
	
	private List<Path> seconfdApproach(String srcURI, String dstURI, int k) throws Exception {
		
		org.apache.jena.ontology.OntTools.Path p = OntTools.findShortestPath(m, ResourceFactory.createResource(srcURI), ResourceFactory.createResource(dstURI), stmt -> true);
		
		Path ownPath = translate(p);
		
		List<Path> list = new LinkedList<>();
		list.add(ownPath);
		
		
		return list;
	
	}
	
	private de.dfki.km.academicresearch.eswc2016challenge.data.Path translate(org.apache.jena.ontology.OntTools.Path p){
		
		de.dfki.km.academicresearch.eswc2016challenge.data.Path ret = new Path();
		
		if(p.size()>0){
			ret.add(p.get(0).getSubject());
		}
		for(Statement s : p){
			ret.add(s.getPredicate());
			
			if(s.getObject().isResource() == false){
				logger.error("Path contains Literals");
				continue;
			}
			ret.add(s.getObject().asResource());
		}
		return ret;
	}

	
	
	//INDEX Specifica
	
	@Override
	public void send(Triple item) {
		
		//Resource c;
		//c.asNode()
		
		//Statement s = new StatementImpl(item.getSubject(), item.getPredicate(), item.getObject());
		//ModelGraphInterface
		//https://jena.apache.org/documentation/javadoc/jena/org/apache/jena/rdf/model/ModelGraphInterface.html
		//http://webcache.googleusercontent.com/search?q=cache:nM7IqgpQxT0J:answers.semanticweb.com/questions/14287/functions-to-convert-between-jena-nodetriple-and-rdfnodestatement+&cd=1&hl=de&ct=clnk&gl=de
		m.add(m.asStatement(item));
	}
	
	//Do not save or load any index and parses again and again
	
	@Override
	public void endStreaming() throws Exception {
		
	}
	
	@Override
	public void loadIndex() throws Exception {
		
	}
	
	
}
