package de.dfki.km.academicresearch.eswc2016challenge.pathfinder;

import java.util.List;

import org.apache.commons.collections15.Transformer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.URIPathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jung.Eppstein;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jung.JungEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jung.JungUtil;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jung.KShortestPathAlgorithm;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jung.Yen;
import edu.uci.ics.jung.graph.DirectedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;

public class JungOwnEdgePathFinder extends URIPathFinder {
	
	private static final Logger logger = LoggerFactory.getLogger(JungOwnEdgePathFinder.class);

	private Graph<String, JungEdge> g;
	private KShortestPathAlgorithm<String, JungEdge> pathlgo;
	@Override
	public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
		
		List<List<JungEdge>> path = pathlgo.getShortestPaths(srcURI, dstURI, k);
		List<Path> retPath = JungUtil.tranform(path);
		
		//logger.debug(retPath.toString());
		
		return retPath;
	}
	
	
	
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {		
		g.addEdge(new JungEdge(subject, object, predicate), subject, object);		
	}
	
	@Override
	public void startStreaming() throws Exception {
		g = new DirectedSparseMultigraph();
	}
	
	@Override
	public void endStreaming() throws Exception {
		
		//pathlgo = new Eppstein<String, JungEdge>(g, tranformToOne);	
		pathlgo = new Yen<String, JungEdge>(g, tranformToOne);
		
	}
	
	
	Transformer<JungEdge, Number> tranformToOne = new Transformer<JungEdge, Number>() {
		@Override
		public Number transform(JungEdge arg0) {
			return 1;
		}		 
	};
}
