package de.dfki.km.academicresearch.eswc2016challenge.pathfinder;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;
import java.util.List;

import org.apache.jena.graph.Triple;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.URIPathFinder;
import edu.asu.emit.algorithm.graph.Graph;
import edu.asu.emit.algorithm.graph.VariableGraph;
import edu.asu.emit.algorithm.graph.shortestpaths.YenTopKShortestPathsAlg;

public class YqiPathFinder extends URIPathFinder{
	
	
	private Graph graph;
	
	@Override
	public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
		
		int src = map.get(srcURI).intValue();
		int dst = map.get(dstURI).intValue();
		
		YenTopKShortestPathsAlg yenAlg = new YenTopKShortestPathsAlg(graph, graph.getVertex(src), graph.getVertex(dst));
		int i=0;
		while(yenAlg.hasNext())
		{
			edu.asu.emit.algorithm.graph.Path p = yenAlg.next();
						
			System.out.println("Path "+i++ +" : "+p);
		}
		return new LinkedList<>();
		
	}
	
	
	
	private BiMap<String, Long> map = HashBiMap.create();
	private long count = 0;
	private BufferedWriter w;
	
	private static final String sep = System.getProperty("line.separator");
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {
		super.sendOnlyURI(subject, predicate, object);
		
		Long subjectId = getId(subject);
		Long predicateId = getId(predicate);
		Long objectId = getId(object);
		
		StringBuilder sb = new StringBuilder();
		sb.append(subjectId.toString());
		sb.append(" ");
		sb.append(objectId.toString());
		sb.append(" 1");
		sb.append(sep);
		
		try {
			w.write(sb.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	@Override
	public void startStreaming() throws Exception {
		super.startStreaming();
		w= new BufferedWriter(new FileWriter(getIndexFile(0)));
	}
	
	@Override
	public void endStreaming() throws Exception {
		super.endStreaming();
		w.close();
		appendOneLineAtTop(getIndexFile(0),getIndexFile(1), String.format("%s %n", count));
		
		loadIndex();
	}
	private void appendOneLineAtTop(File oldFile, File newFile, String line) throws IOException{
		
		try (BufferedWriter writer = new BufferedWriter(new FileWriter(newFile))) {
			writer.write(line);
            try (BufferedReader br = new BufferedReader(new FileReader(oldFile))) {
                String linetmp;
                while ((linetmp = br.readLine()) != null) {
                    writer.write(linetmp.replace("^^<http://dbpedia.org/datatype/brake horsepower>", "") + "\n");
                }
            }
        }
	}
	
	private Long getId(String uri){
		Long uriid = map.get(uri);
		if(uriid == null){
			map.put(uri, count);
			uriid = count;
			count++;
		}
		return uriid;
	}
	
	@Override
	public void loadIndex() throws Exception {
		super.loadIndex();
		File f = getIndexFile(1);
		
		graph = new VariableGraph(f.getAbsolutePath());
		
	}

	
}
