package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.jena.rdf.model.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.DefaultPathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind.CheckDegree;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind.CheckMultigraph;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind.CheckMultigraph2;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind.DuplicateTriple;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind.Summary;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.URIPathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgrapht.JGraphTFinderBase;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;

public class AnalyseGraph extends JGraphTFinderBaseURI{

	private static final Logger logger = LoggerFactory.getLogger(AnalyseGraph.class);
	
	private List<IAnalyse> analyse;
	public AnalyseGraph() {
		analyse = new LinkedList<>();
		analyse.add(new CheckDegree());
		analyse.add(new CheckMultigraph());
		analyse.add(new CheckMultigraph2());
		analyse.add(new DuplicateTriple());		
	}
	
		
	@Override
	public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {		return new LinkedList<>(); }
		
	//graph metrics
	//https://reference.wolfram.com/language/guide/GraphMeasures.html
	//https://en.wikipedia.org/wiki/Distance_(graph_theory)
	//https://en.wikipedia.org/wiki/Dense_graph	
	//https://en.wikipedia.org/wiki/Centrality
	
	//Diameter in huge graphs -> great...
	//maybe sparql-> ?graph <diameter> ?x.????
	
	//+branching factor?? = out degree ->If this value is not uniform, an average branching factor can be calculated.
	//see also
	
	//FOR EXAMPLE:
	//From gephi
	//+ Average Degree
	//  Avg. weigthed Degree
	//+ Network Diameter
	//+ Graph Density
	//- HITS
	//- Modularity
	//- PageRank
	//- Connected Components
	
	//- Avg. Clustering Coefficient
	//- Eigenvector Centrality
	
	//+ avg. Path Length
	
	
	
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {
		super.sendOnlyURI(subject, predicate, object);
		analyse.forEach(a->a.sendOnlyURI(subject, predicate, object));
	}
	
	
	@Override
	public void endStreaming() throws Exception {
		super.endStreaming();
		logger.debug("Write file");
		
		//need data from this triple sink
		analyse.add(new Summary(subjectNotURI,predicateNotURI, objectNotURI, totalURITriples, tripleCount));
		
		int count = 1;
		
		for(IAnalyse a : analyse){
			logger.debug("Number {}", count);
			BufferedWriter w = null;
			try {
				w = new BufferedWriter(new FileWriter(getIndexFile(count)));
			
				w.write(a.getResult(this.g));
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			finally{
				try {
					w.close();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			count++;
		}
		
	}

	

}
