package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.Statement;
import org.jgrapht.graph.DirectedMultigraph;
import org.jgrapht.graph.DirectedWeightedPseudograph;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.IAnalyse;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;

public class CheckDegree implements IAnalyse{

	
	public CheckDegree() {
	}
	
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {
		
	}

	@Override
	public String getResult(DirectedWeightedPseudograph<String,URIEdge> g) throws Exception {
		
		
		DescriptiveStatistics indegree = new DescriptiveStatistics();
		DescriptiveStatistics outdegree = new DescriptiveStatistics();

		for(String s : g.vertexSet()){
			int out = g.outDegreeOf(s);
			int in = g.inDegreeOf(s);
			
			indegree.addValue(in);
			outdegree.addValue(out);
		}
		
		StringWriter w = new StringWriter();
		try{
			w.write(String.format("#Outdegree:%n"));
			printStatistics(outdegree, w);
			w.write(String.format("#Indegree:%n"));
			printStatistics(indegree, w);
		}finally{
			w.close();
		}
		return w.toString();
	}

	private void printStatistics(DescriptiveStatistics d, StringWriter w){
		w.write(String.format("#Important: %s%n", d.toString()));
	}
}
