package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.Statement;
import org.jgrapht.graph.DirectedMultigraph;
import org.jgrapht.graph.DirectedWeightedPseudograph;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.IAnalyse;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;


public class CheckMultigraph implements IAnalyse{

	private static final String separator = "<>";
	private Map<String, List<String>> map;
	private int count = 0;
	private int countReflexive = 0;
	private Map<String, String> reflexive;
	
	public CheckMultigraph() {
		map = new HashMap<String, List<String>>();
		reflexive = new HashMap<String, String>();
	}
	
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {
		String key = subject+separator+object;
		
		if(map.containsKey(key)){
			count++;
			map.get(key).add(predicate);
		}
		else{
			map.put(key, new LinkedList<>(Arrays.asList(predicate)));
		}
		
		if(subject.equals(object)){
			countReflexive++;
			reflexive.put(subject, predicate);
		}
		
	}

	@Override
	public String getResult(DirectedWeightedPseudograph<String,URIEdge> g) throws Exception {		
		StringWriter w = new StringWriter();
		try{
			w.write(String.format("#Count of multiedges: %d .%n", count));
			w.write(String.format("#Count of reflexive edges: %d .%n", countReflexive));
			
			w.write(String.format("#Reflexive: %n"));
			for(Entry<String, String> entry : reflexive.entrySet()){
				w.write(String.format("<%s> <%s> <%s>.%n", entry.getKey(),entry.getValue(), entry.getKey()));						
			}	
			
			
			w.write(String.format("%n%n%n#Multiedges: %n"));
			for(Entry<String, List<String>> entry : map.entrySet()){
				if(entry.getValue().size() > 1){
					String[] array = entry.getKey().split(separator);
					String subject = array[0];
					String object = array[1];
					for(String predicate : entry.getValue()){
						w.write(String.format("<%s> <%s> <%s>.%n", subject,predicate, object));
					}
					w.write(String.format("%n"));
				}				
			}			
		}finally{
			w.close();
		}
		return w.toString();
	}

}
