package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.kind;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.jgrapht.graph.DirectedWeightedPseudograph;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.analyse.IAnalyse;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;



public class DuplicateTriple implements IAnalyse{

	private Map<String, Integer> map;
	private static final String separator = "<>";
	
	public DuplicateTriple() {
		map = new HashMap<String, Integer>();
	}
	
	@Override
	public void sendOnlyURI(String subject, String predicate, String object) {
		String key = subject+separator+predicate+separator+object;
		
		Integer i = map.getOrDefault(key, 0);
		i++;
		map.put(key, i);
		
	}

	@Override
	public String getResult(DirectedWeightedPseudograph<String, URIEdge> g) throws Exception {
		int count = 0;
		int sumDuplicate = 0;
		Map<String, Integer> onlyMoreThanOne = new HashMap<>();
		Map<String, Integer> predicate = new HashMap<>();
		
		for(Entry<String, Integer> entry : map.entrySet()) {				
			if(entry.getValue() > 1){
				onlyMoreThanOne.put(entry.getKey(), entry.getValue());
				
				String[] array = entry.getKey().split(separator);
				Integer i = predicate.getOrDefault(array[1], 0);
				i+=entry.getValue();
				predicate.put(array[1], i);
				
				count++;
				sumDuplicate+=entry.getValue();
			}
		}
		map = null; //free memory
		
		//map.values().removeAll(Collections.singleton(1));//remove all which length is equal to one
				
		
		StringWriter w = null;
		try{
			w = new StringWriter();
			w.write(String.format("Duplicate Triple:%n"));
			w.write(String.format("#Together there are %d duplicate triples and %d of them are the same.%n", sumDuplicate, count));
			w.write(String.format("#Based on properties we have the following distribution:%n"));
						
			for(Entry<String, Integer> entry : sortByValue(predicate)) {				
				w.write(String.format("#Times: %d\tPredicate: <%s>%n", entry.getValue(), entry.getKey()));
			}
			w.write(String.format("############.%n"));
			
			for(Entry<String, Integer> entry : sortByValue(onlyMoreThanOne)) {
				String[] array = entry.getKey().split(separator);
				w.write(String.format("#%d times the following triple.%n", entry.getValue()));
				w.write(String.format("<%s> <%s> <%s>.%n", array[0],array[1], array[2]));
			}
			
		}finally{
			w.close();
		}
		return w.toString();
	}
	
	public <K,V extends Comparable<V>> List<Entry<K,V>> sortByValue(Map<K,V> unsortedMap) {
		List<Entry<K,V>> list = new LinkedList<Entry<K,V>>(unsortedMap.entrySet());
	 
		Collections.sort(list, new Comparator<Entry<K,V>>() {			
			@Override
			public int compare(Entry<K, V> o1, Entry<K, V> o2) {
				return o2.getValue().compareTo(o1.getValue());
			}
		});
		return list;
	}


}
