package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis;

import org.apache.jena.graph.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.dfki.km.academicresearch.eswc2016challenge.util.PropertyPath;

public abstract class URIPathFinder extends PathFinder {

	
	private static final Logger logger = LoggerFactory.getLogger(URIPathFinder.class);
	
	
	
	protected long subjectNotURI = 0;
	protected long predicateNotURI = 0;
	protected long objectNotURI = 0;
	protected long totalURITriples = 0;
	
	@Override
	public void send(Triple item) {
		super.send(item);
		
		boolean allAreURI = true;
		if(item.getSubject().isURI() == false) {
			subjectNotURI++;
			allAreURI = false;
        }
		if(item.getPredicate().isURI() == false) {
			predicateNotURI++;
			allAreURI = false;
        }
		if(item.getObject().isURI() == false) {
			objectNotURI++;
			allAreURI = false;
        }
		
		if(allAreURI)
			sendOnlyURI(item.getSubject().getURI(), item.getPredicate().getURI(),item.getObject().getURI());
		
	}
	
	public void sendOnlyURI(String subject, String predicate, String object) {
		totalURITriples++;
    }
	
	
	@Override
	public void endStreaming() throws Exception {
		logger.debug("saveIndex {} with {} URI Triples out of {} triples, subject not URI: {}, predicte not URI: {}, object not URI: {}", finderName, totalURITriples, tripleCount, subjectNotURI, predicateNotURI, objectNotURI);
	}

}
