package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgrapht;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.*;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.URIPathFinder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;
import org.apache.jena.rdf.model.Property;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.rdf.model.Statement;
import org.jgrapht.graph.DirectedMultigraph;
import org.nustaq.serialization.FSTObjectInput;
import org.nustaq.serialization.FSTObjectOutput;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class JGraphTFinderBase extends URIPathFinder {

    private static final Logger logger = LoggerFactory.getLogger(JGraphTFinderBase.class);

    protected DirectedMultigraph<Resource, Statement> g;

    public JGraphTFinderBase() {
        g = new DirectedMultigraph<>(Statement.class);
    }

    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        Resource src = ResourceFactory.createResource(srcURI);
        Resource dst = ResourceFactory.createResource(dstURI);
        return find(src, dst, k);
    }
    
    public abstract List<Path> find(Resource src, Resource dst, int k) throws Exception;
    
    @Override
    public void sendOnlyURI(String subject, String predicate, String object) {
        super.sendOnlyURI(subject, predicate, object);

        Resource s = ResourceFactory.createResource(subject);
        Resource o = ResourceFactory.createResource(object);
        g.addVertex(s);
        g.addVertex(o);

        Property p = ResourceFactory.createProperty(predicate);

        try {
            g.addEdge(s, o, ResourceFactory.createStatement(s, p, o));
        } catch(Exception e) {
            //loops are ok
        }
    }

    @Override
    public void startStreaming() throws Exception {
        //normal streaming needs 39.611ms
    }

    @Override
    public void endStreaming() throws Exception {
        super.endStreaming();
        File saveFile = getIndexFile();
        logger.debug("write to " + saveFile.getAbsolutePath());
        FSTObjectOutput out = null;
        try {
            out = new FSTObjectOutput(new FileOutputStream(saveFile));
            out.writeObject(g); //needs 11.578ms
        } finally {
            out.close();
        }
        logger.debug("done");
    }

    @Override
    public void loadIndex() throws Exception {
        super.loadIndex();

        File loadFile = getIndexFile();
        FSTObjectInput out = null;
        try {
            out = new FSTObjectInput(new FileInputStream(loadFile));
            logger.debug("start deserialize");
            g = (DirectedMultigraph<Resource, Statement>) out.readObject();   //needs 6.791ms -> GREAT!! much better than streaming 
            logger.debug("done");
        } finally {
            out.close();
        }
    }

}
