package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.BFSDistance;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.IShortestPathTree;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.ShortestPathTree;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import static java.util.stream.Collectors.toSet;
import org.jgrapht.util.FibonacciHeap;
import org.jgrapht.util.FibonacciHeapNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class EppsteinV1 extends JGraphTFinderBaseURI {

    private static final Logger logger = LoggerFactory.getLogger(EppsteinV1.class);
    
    private Map<URIEdge, Double> delta;
    private Set<URIEdge> edgesInGMinusT;
    
    private Map<String, FibonacciHeap<URIEdge>> v2H_out;
    private Map<String, FibonacciHeap<URIEdge>> v2H_T;
    
    
    public EppsteinV1() {
        delta = new HashMap<>();
        v2H_out = new HashMap<>();
        v2H_T = new HashMap<>();
    }
    
    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        
        //shortest path tree
        IShortestPathTree<String, URIEdge> sptCalculator = new BFSDistance<>();
        ShortestPathTree<String, URIEdge> spt = sptCalculator.computeShortestPathTree(g, dstURI);
        
        //all delta(e)
        calcDelta(spt);
        
        Set<URIEdge> edgesInT = spt.edgeSet();
        edgesInGMinusT = new HashSet<>(g.edgeSet());
        //G - T
        edgesInGMinusT.removeAll(edgesInT);
        
        
        //for each v
        for(String v : g.vertexSet()) {
            
            FibonacciHeap<URIEdge> H_out_v = H_out(v);
            v2H_out.put(v, H_out_v);
            
            FibonacciHeap<URIEdge> H_T = H_T(v, spt);
            v2H_T.put(v, H_T);
            
            FibonacciHeap<URIEdge> H_G = H_G(v);
            
            int a = 0;
        }
        
        
        
        print(edgesInGMinusT);
        
        return new LinkedList<>();
    }
    
    /**
     * For each edge e: delta(e) = l(e) + d(head(e), t) - d(tail(e), t).
     * @param spt 
     */
    private void calcDelta(ShortestPathTree<String, URIEdge> spt) {
        for(URIEdge e : g.edgeSet()) {
            
            //tail -> head
            String tail = e.getSource();
            String head = e.getTarget();
            
            //l(e) always 1
            double l_e = 1;
            double d_head_e_t = spt.getDistance(head);
            double d_tail_e_t = spt.getDistance(tail);
            
            //delta(e) = l(e) + d(head(e), t) - d(tail(e), t)
            delta.put(e, l_e + d_head_e_t - d_tail_e_t);
        }
    }
    
    /**
     * Let out(v) denotes the edges in G-T with tails in v.
     * @param v
     * @param edgesInGMinusT
     * @return 
     */
    private Set<URIEdge> out(String v) {
        return edgesInGMinusT.stream().filter(e -> e.getSource().equals(v)).collect(toSet());
    }
    
    /**
     * edge minimizing delta(e) in out(v).
     * @param v
     * @return 
     */
    private URIEdge outroot(String v) {
        Set<URIEdge> out_v = out(v);
        if(out_v.isEmpty())
            return null;
        return out(v).stream().min((e1, e2) -> Double.compare(delta.get(e1), delta.get(e2))).get();
    }
    
    /**
     * build a heap for each v of the edges in out(v).
     * @param v
     * @return 
     */
    private FibonacciHeap<URIEdge> H_out(String v) {
        FibonacciHeap<URIEdge> f = new FibonacciHeap<>();
        
        for(URIEdge e : out(v)) {
            f.insert(new FibonacciHeapNode<>(e), delta.get(e));
        }
        
        return f;       
    }
    
    /**
     * Containing only the roots outroot(w) of the heaps for each w on the path
     * from v to t.
     * @param v
     * @param spt
     * @return 
     */
    private FibonacciHeap<URIEdge> H_T(String v, ShortestPathTree<String, URIEdge> spt) {
        
        FibonacciHeap<URIEdge> f = new FibonacciHeap<>();
        
        String cur = v;
        
        while(true) {
            URIEdge outroot_v = outroot(cur);
            //if there is a minimal
            if(outroot_v != null)
                f.insert(new FibonacciHeapNode<>(outroot_v), delta.get(outroot_v));
            
            //next in T
            URIEdge e = spt.getPrevious(cur);
            if(e == null)
                break;
            cur = e.getTarget();
        }
        
        return f;
    }
    
    /**
     * By connecting each node outroot(w) in H_T(v) to an additional subtree 
     * beyond the two it points to in H_T(v), namely to the rest of heap H_out(w).
     * @param v
     * @return 
     */
    private FibonacciHeap<URIEdge> H_G(String v) {
        
        FibonacciHeap<URIEdge> H_T = v2H_T.get(v);
        
        //go from right to left
        //find another H_T were min().edge.target
        //find another H_T were root().edge.target
        
        for(FibonacciHeap<URIEdge> other : v2H_T.values()) {
            FibonacciHeapNode<URIEdge> n = other.min();
            
            if(n == null)
                continue;
            
            String target = n.getData().getTarget();
            
            if(target.equals(v)) {
                
                //make crossedge from other to H_T
                int a = 0;
                
            }
        }
        
        return null;
    }

    private void print(Collection<URIEdge> es) {
        for(URIEdge e : es) {
            logger.debug(e.toString());
        }
    }
    
    /*
    private class HeapNode {
        public URIEdge e;//the edge
        public double delta;//delta(e)
    }
    */
    
}
