package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.DijkstraDistance;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.ShortestPathTree;
import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.stream.Collectors;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.apache.jena.rdf.model.ResourceFactory;
import org.jgrapht.DirectedGraph;
import org.jgrapht.WeightedGraph;
import org.jgrapht.ext.DOTExporter;
import org.jgrapht.ext.GraphMLExporter;
import org.jgrapht.graph.DefaultDirectedWeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;
import org.jgrapht.graph.DirectedWeightedPseudograph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class EppsteinV2 extends JGraphTFinderBaseURI {

    private static final Logger logger = LoggerFactory.getLogger(EppsteinV2.class);

    private class State {

        public Map<URIEdge, Double> delta;

        public DirectedGraph<String, URIEdge> G;
        
        //public Set<URIEdge> edgesInGMinusT;
        public DirectedGraph<String, URIEdge> GminusT;

        /**
         * To get H_T(v) fast.
         */
        public Map<String, URIEdgeDelta> v2minSidetrack;

        public Map<String, BinaryHeap<URIEdgeDelta>> v2H_T;
        public Map<String, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge>> v2H_T_graph;
        @Deprecated
        public Map<String, URIEdgeDelta> v2H_T_root;

        public Map<String, BinaryHeap<URIEdgeDelta>> v2H_out;
        public Map<String, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge>> v2H_out_graph;
        @Deprecated
        public Map<String, URIEdgeDelta> v2H_out_root;

        public Map<String, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge>> v2merged_graph;

        public final URIEdgeDelta init = new URIEdgeDelta("", new URIEdge("init", "init", "TODO: set to srcURI"), 0);

        public Set<String> V;

        public ShortestPathTree<String, URIEdge> T;
        public ShortestPathTree<String, URIEdge> Tprime;

        public String srcURISpecial;
        public String srcURI;
        public String dstURI;
        public int k;
        public String P;
        
        public long begin;
        public long end;

        //public boolean PonlyInSidetracks;
        public int dublicatePathCounter;

        public State() {
            delta = new HashMap<>();

            v2H_T = new HashMap<>();
            v2H_T_graph = new HashMap<>();
            v2H_T_root = new HashMap<>();

            v2H_out = new HashMap<>();
            v2H_out_graph = new HashMap<>();
            v2H_out_root = new HashMap<>();

            v2merged_graph = new HashMap<>();

            v2minSidetrack = new HashMap<>();

            V = new HashSet<>();
        }
    }

    private Map<String, State> dst2state;

    //output for debugging
    private boolean alwaysNewState = true;

    private boolean autoWriteDisable = true;
    private boolean write = true;
    private boolean writeDot = write;
    @Deprecated
    private boolean writeGraphML = write;

    //output folder for dotfiles
    private File outFolder;

    @Deprecated
    private List<String> onlyTheseDots = Arrays.asList(
            "http://dbpedia.org/resource/Felipe_Massa",
            "http://dbpedia.org/resource/Robert_Kubica",
            "http://dbpedia.org/resource/2008_Canadian_Grand_Prix",
            "http://dbpedia.org/resource/Red_Bull"
    );
    @Deprecated
    private boolean useOnlyTheseDots = false;

    private CSVPrinter allcsv;
    
    public EppsteinV2() {
        dst2state = new HashMap<>();
        
    }

    private void initCSV() {
        try {
            File f = new File("all.csv");
            boolean fex = f.exists();
            FileWriter fw = new FileWriter(f, true);
            CSVFormat csvFileFormat = CSVFormat.DEFAULT.withRecordSeparator('\n');
            allcsv = new CSVPrinter(fw, csvFileFormat);
            
            if(!fex) {
                allcsv.printRecord(
                    "state.srcURI", 
                    "state.dstURI", 
                    "state.srcURISpecial", 
                    "state.k", 
                    "state.P", 

                    "state.G.vertexSet().size()", 
                    "state.G.edgeSet().size()",
                    
                    "m + n log n + k",

                    "state.GminusT.vertexSet().size()",
                    "state.GminusT.edgeSet().size()",

                    "state.T.vertexSet().size()",
                    "state.T.edgeSet().size()",

                    "state.delta.size()",
                    "state.v2minSidetrack.size()",

                    "iteration",
                    "pathes.size()",
                    "iteration - pathes.size()",
                    "overhead",
                    
                    "state.begin",
                    "state.end",
                    "state.end - state.begin",
                    
                    "nonConstructed",
                    "PrintHeapUsage.getUsedMB()",
                    "P_G.vertexSet().size()",
                    "P_G.edgeSet().size()",
                    "H.size()",

                    "type2count.get(InvalidType.None)",
                    "type2count.get(InvalidType.Sidetrack)",
                    "type2count.get(InvalidType.ShortestPath)",
                    "type2count.get(InvalidType.NeverReachDst)"
                );
            }
            
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(EppsteinV2.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }
    
    //==================== main functions ======================================
    
    @Override
    public List<Path> find(final String srcURI, String dstURI, int k, final String startOrEndProp) throws Exception {
        
        initCSV();
        
        logger.debug("G with {} vertices and {} edges", g.vertexSet().size(), g.edgeSet().size());
        
        //run eppstein twice
        if (startOrEndProp != null) {
            
            String P = startOrEndProp.trim();//because of task2 test3
            logger.debug("P is set to \"{}\"", URIEdge.cutURI(P));
            
            long begin = System.currentTimeMillis();
            
            List<Path> p1 = findWithPOneWay(srcURI, dstURI, k, P, g);
            
            
            DirectedGraph<String, URIEdge> gInvertedEdges = cloneReference(g, URIEdge.class, new HashSet<>(), true);
            List<Path> p2 = findWithPOneWay(dstURI, srcURI, k, P, gInvertedEdges);
            //cleanup
            for(Path pp2 : p2) {
                Collections.reverse(pp2.getPath());
            }
            
            List<Path> finalResult;
            
            //without zipping
            if(!p1.isEmpty() && p2.isEmpty()) {
                finalResult = p1;
            }
            else if(p1.isEmpty() && !p2.isEmpty()) {
                finalResult = p2;
            } else {
                //you have to zip
                finalResult = zip(p1, p2, k);
            }
            
            long end = System.currentTimeMillis();
            
            allcsv.printRecord(
                "both", 
                URIEdge.cutURI(srcURI) + "/" + URIEdge.cutURI(dstURI), 
                URIEdge.cutURI(dstURI) + "/" + URIEdge.cutURI(srcURI), 
                k, 
                P, 
                
                "full runtime:",
                (end - begin),
                
                "-",
                
                "-",
                "-",
                
                "-",
                "-",
                
                "-",
                "-",
                
                "-",
                "-",
                "-",
                "-",
                
                "-",
                "-",
                "-",
                
                "-",
                "-",
                "-",
                "-",
                "-",
                
                "-",
                "-",
                "-",
                "-"
            );
            
            allcsv.close();
            
            return finalResult;
            
            //cleanup step
            //for(Path pp1 : p1) {
                //overwrite neverExistingVertex with real src resource
            //    pp1.getPath().set(0, srcRes);
            //}
            
            
            //Set<URIEdge> notPtoDstURI = g.incomingEdgesOf(dstURI).stream().filter(e -> !e.getEdge().equals(P)).collect(toSet()); 
            //logger.debug("copy graph G and remove edges ? -notP-> dstURI, {} edges", notPtoDstURI.size());
            //DirectedGraph<String, URIEdge> g2 = cloneReference(g, URIEdge.class, notPtoDstURI);
            //g2.removeAllEdges(notPtoDstURI);
            //logger.debug("G2 with {} vertices and {} edges", g2.vertexSet().size(), g2.edgeSet().size());
            //List<Path> p2 = findPrivate(srcURI, dstURI, k, g2, null);
            //logger.debug("G2 has {} pathes found", p2.size());
            
            //for(Path pp2 : p2) {
            //    logger.debug(pp2.toString());
            //}
            
            //TODO
            //List<Path> result = new ArrayList<>(p1);
            //result.addAll(p2);
            //Collections.sort(result);
            
            //logger.debug("together we have {} pathes", result.size());
            
            
        }
        
        //simple eppstein with g
        List<Path> ret = findPrivate(null, srcURI, dstURI, k, g, null);
        
        allcsv.close();
        
        return ret;
    }
    
    private List<Path> zip(List<Path> p1, List<Path> p2, int k) {
        List<Path> result = new LinkedList<>();
            
        int i = 0;
        int j = 0;
        while(result.size() < k && (i < p1.size() || j < p2.size())) {

            int comp;
            Path left = null;
            Path right = null;

            //left is empty but right is full
            if(i >= p1.size() && j < p2.size()) {

                comp = +1;
                right = p2.get(j);
            }
            //left is full but right is empty
            else if(i < p1.size() && j >= p2.size()) {

                comp = -1;
                left = p1.get(i);

            } else {
                //both are full
                left  = p1.get(i);
                right = p2.get(j);
                comp = left.compareTo(right);

                if(left.equals(right)) {
                    comp = -1;
                    j++;
                } 
            }

            if(comp < 0) {
                if(!result.contains(left)) {
                    result.add(left);
                }
                i++;
            } else if(comp > 0) {
                if(!result.contains(right)) {
                    result.add(right);
                }
                j++;
            } else {

                if(!result.contains(left)) {
                    result.add(left);
                }
                i++;

                if(!result.contains(right)) {
                    result.add(right);
                }
                j++;
            }
        }
        return result;
    }
    
    private List<Path> findWithPOneWay(String srcURI, String dstURI, int k, String P, DirectedGraph<String, URIEdge> givenG) throws Exception {
        String neverExistingVertex = "neverExistingVertex";
        
        Set<URIEdge> srcURIoverP = givenG.outgoingEdgesOf(srcURI).stream().filter(e -> e.getEdge().equals(P)).collect(Collectors.toSet());

        //if this is empty we can never get valid pathes
        if(srcURIoverP.isEmpty()) {
            logger.warn("srcURIoverP is empty so there can never be a valid path");
            
            allcsv.printRecord(
                "neverExistingVertex", 
                dstURI, 
                srcURI, 
                k, 
                P, 
                
                "-",
                "-",
                
                "-",
                
                "-",
                "-",
                
                "-",
                "-",
                
                "-",
                "-",
                
                0,
                0,
                0,
                0,
                
                "-",
                "-",
                "-",
                
                0,
                0,
                0,
                0,
                0,
                
                0,
                0,
                0,
                0
            );
            
            return new LinkedList<>();
        }
        
        //clone g and modify g1
        DirectedGraph<String, URIEdge> g1 = cloneReference(givenG, URIEdge.class, new HashSet<>(), false);
        g1.addVertex(neverExistingVertex);
        for(URIEdge e : srcURIoverP) {
            g1.addEdge(neverExistingVertex, e.getTarget(), new URIEdge(e.getEdge(), neverExistingVertex, e.getTarget()));
        }

        //g1.removeAllEdges(srcURItoNotP);
        logger.debug("G1 with {} vertices and {} edges", g1.vertexSet().size(), g1.edgeSet().size());

        List<Path> p1 = findPrivate(srcURI, neverExistingVertex, dstURI, k, g1, P);
        logger.debug("G1 has {} pathes found", p1.size());
        
        return p1;
    }
    
    private List<Path> findPrivate(String srcURISpecial, String srcURI, String dstURI, int k, DirectedGraph<String, URIEdge> givenG, String startOrEndProp) throws Exception {

        logger.debug("");
        logger.debug("begin find {} {} {}", URIEdge.cutURI(srcURI), URIEdge.cutURI(dstURI), k);
        if (writeDot) {
            logger.warn("will write dot files");
        }
        
        logger.debug("G with {} vertices and {} edges", givenG.vertexSet().size(), givenG.edgeSet().size());
        
        createOutfolder(srcURI, dstURI, k);

        //we have to build it only once for one destination
        State state;
        if (alwaysNewState || !dst2state.containsKey(dstURI)) {

            //create a new one
            state = new State();
            
            //to point correctly with init edge
            state.begin = System.currentTimeMillis();
            state.G = givenG;
            state.init.getEdge().setTarget(srcURI);
            state.srcURI = srcURI;
            state.dstURI = dstURI;
            state.k = k;
            if(startOrEndProp != null)
                state.P = startOrEndProp.trim();
            state.srcURISpecial = srcURISpecial;

            //======================================================================
            // the graphs we need:
            // T is the shortest path tree
            // G is the original graph
            // G - T is the sidetrack graph
            //logger.debug("calc shortest path tree T...");
            state.T = new DijkstraDistance<String, URIEdge>().computeShortestPathTree(givenG, dstURI);
            logger.debug("T with {} vertices and {} edges", state.T.vertexSet().size(), state.T.edgeSet().size());
            
            /*
            if(state.P != null) {
                Set<URIEdge> disallow = givenG.outgoingEdgesOf(srcURI).stream().filter(e -> !e.getEdge().equals(state.P)).collect(Collectors.toSet());
                DirectedGraph<String, URIEdge> g1 = cloneReference(givenG, URIEdge.class, disallow, false);
                state.Tprime = new DijkstraDistance<String, URIEdge>().computeShortestPathTree(g1, dstURI);
                
                //consistency check
                URIEdge e = state.Tprime.getPrevious(srcURI);
                boolean PinT = e.getEdge().equals(state.P);
                assert PinT;
            }
            */
            
            

            //check if P is in T
            /*
            if(startOrEndProp != null) {
                URIEdge srcEdge = state.T.getPrevious(srcURI);
                URIEdge dstEdge = state.T.getDestinationEdge();
                
                boolean PinSrcEdge = srcEdge.getEdge().equals(startOrEndProp);
                boolean PinDstEdge = dstEdge.getEdge().equals(startOrEndProp);
                
                //if both are false only sidetracks make path
                //propertypath invalid
                state.PonlyInSidetracks = !PinSrcEdge && !PinDstEdge;
 
                logger.debug("P will only be in sidetracks");
            }
             */
            //write dot
            if (autoWriteDisable && state.T.vertexSet().size() > 50) {
                logger.warn("because T is so big we will not make dot and graphml files");
                writeDot = writeGraphML = false;
            }
            writeDotStringURIEdge("00_shortest_path_tree", state.T.toGraph());
            writeDotStringURIEdge("00_G", givenG);

            //we will only do something for the vertices in the shortest path tree
            //think about more components in a graph
            state.V = state.T.vertexSet();//maybe too optimized and we miss some
            //state.V = g.vertexSet();

            calcGminusT2(state);

            calcDeltaVdriven(state);

            //======================================================================
            //the atoms in our building the H_T(v)s with H_out(v) combined
            
            calcH_Ts(state);

            calcH_outs(state);

            //logger.debug("H_T.minSidetrackWasNull = {}", minSidetrackWasNull);
            //logger.debug("H_T.cur was {} times not in T", curWasNotInT);
            //logger.debug("Hout.targetContainsNotInTCount = {}", targetContainsNotInTCount);
            //logger.debug("Hout.empty Hout for {} vertices", emptyHoutFor.size());
            //we use combineLazy
            //combineH_TwithH_outV2(state);
            //recordIncrementalCostsForMerged(state);
            dst2state.put(dstURI, state);

        } else {
            logger.debug("we have calculated everything for {}", dstURI);

            state = dst2state.get(dstURI);

            //to point correctly with init edge
            state.G = givenG;
            state.init.getEdge().setTarget(srcURI);
            state.srcURI = srcURI;
            state.dstURI = dstURI;
            state.k = k;
            if(startOrEndProp != null)
                state.P = startOrEndProp.trim();
            state.srcURISpecial = srcURISpecial;
        }

        //Set<URIEdge> allowed = givenG.outgoingEdgesOf(srcURI).stream().filter(e -> e.getEdge().equals(state.P)).collect(Collectors.toSet());
        //for(URIEdge e : allowed) {
        //    state.T.setDistanceAndPrevious(e.getSource(), e, 0.0);
        //}
        
        //======================================================================
        //P(G) is the graph we need at the end to do bfs
        List<Path> pathes = constructiveBFSAll(state);

        logger.debug("done find {} {} {}", URIEdge.cutURI(srcURI), URIEdge.cutURI(dstURI), k);
        logger.debug("");
        
        
        
        return pathes;
    }

    //======================= help functions ===================================
    
    private void calcGminusT2(State state) throws IOException {
        Set<URIEdge> edgesInT = state.T.edgeSetPrevious();

        //sidetrack graph
        //logger.debug("build GminusT = G - T ...");
        state.GminusT = new DirectedWeightedPseudograph<>(URIEdge.class);
        for (URIEdge e : state.G.edgeSet()) {

            //only edges which are NOT in T
            if (edgesInT.contains(e)) {
                continue;
            }

            state.GminusT.addVertex(e.getSource());
            state.GminusT.addVertex(e.getTarget());
            state.GminusT.addEdge(e.getSource(), e.getTarget(), e);
        }
        logger.debug("G-T with {} vertices and {} edges", state.GminusT.vertexSet().size(), state.GminusT.edgeSet().size());
        //for 20.000er = G-T with 367772 vertices and 2085781 edges

        writeDotStringURIEdge("00_GminusT", state.GminusT);
    }

    private void calcDeltaVdriven(State state) {
        logger.debug("calc deltas for " + state.V.size() + " vertices...");
        for (String v : state.V) {

            //out(v) return only sidetracks (in G-T)
            for (URIEdge sidetrackEdge : out(v, state)) {

                URIEdge e = sidetrackEdge;

                //tail -> head
                String tail = e.getSource();
                String head = e.getTarget();

                //l(e) always 1 (in unweighted)
                double l_e = state.G.getEdgeWeight(e);
                Double d_head_e_t = state.T.containsInDistance(head) ? state.T.getDistance(head) : Double.POSITIVE_INFINITY;
                Double d_tail_e_t = state.T.containsInDistance(tail) ? state.T.getDistance(tail) : Double.POSITIVE_INFINITY;

                //delta(e) = l(e) + d(head(e), t) - d(tail(e), t)
                double delta_e = l_e + (d_head_e_t - d_tail_e_t);

                if(delta_e < 0) {
                    delta_e = 0;
                    //throw new RuntimeException("delta is negative!");
                }
                
                state.delta.put(e, delta_e);

                //if it is not already in the map: add it
                //or
                //if it is in the map and if the current value is smaller then override it
                if (!state.v2minSidetrack.containsKey(v) || delta_e < state.v2minSidetrack.get(v).getDelta()) {
                    state.v2minSidetrack.put(v, new URIEdgeDelta("", e, delta_e));
                }
            }
        }
        logger.debug("{} vertices have a minimal sidetrack", state.v2minSidetrack.size());
    }

    //============= H_T ========================================================
    private void calcH_Ts(State state) throws IOException {
        logger.debug("calc H_T for " + state.V.size() + " vertices");
        //for each v a H_T(v)
        for (String v : state.V) {

            BinaryHeap<URIEdgeDelta> H_T = H_T(v, state);
            state.v2H_T.put(v, H_T);

            //if(!H_T.isEmpty())
            //    state.v2H_T_root.put(v, H_T.element());
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_T_g = H_T.toGraph();
            state.v2H_T_graph.put(v, H_T_g);

            if (!useOnlyTheseDots || onlyTheseDots.contains(v)) {
                writeDot("01_H_T_" + URIEdge.cutURI(v), H_T_g);
            }
        }
    }

    private BinaryHeap<URIEdgeDelta> H_T(String v, State state) {
        return H_T(v, state, new HashSet<>());
    }

    /**
     * Containing only the roots outroot(w) of the heaps for each w on the path
     * from v to t.
     *
     * @param v
     * @param spt
     * @return
     */
    private BinaryHeap<URIEdgeDelta> H_T(String v, State state, Set<URIEdge> disallow) {
        BinaryHeap<URIEdgeDelta> h = new BinaryHeap<>();

        String cur = v;
        while (true) {

            URIEdgeDelta minSidetrack = state.v2minSidetrack.get(cur);
            //if there is a minimal sidetrack
            if (minSidetrack != null) {

                //special disallow
                if (!disallow.contains(minSidetrack.getEdge())) {
                    h.add(new URIEdgeDelta(v, minSidetrack.getEdge(), minSidetrack.getDelta()));
                }

            }

            //cur must be in T that we can move forward
            if (!state.T.vertexSet().contains(cur)) {
                break;//here we can not go more
            }

            //next in T
            URIEdge e = state.T.getPrevious(cur);
            if (e == null) {

                //consistency check
                //that this is the real end or do we have to go further?
                if (!cur.equals(state.dstURI)) {
                    throw new RuntimeException("cur is not dstURI");
                }

                break;
            }

            //consistency check
            //that the edge is correct
            if (!cur.equals(e.getSource())) {
                throw new RuntimeException("cur is not e.getSource()");
            }

            cur = e.getTarget();
        }

        return h;
    }

    //============= H_out ======================================================
    private void calcH_outs(State state) throws IOException {
        logger.debug("calc H_out for " + state.V.size() + " vertices");
        //for each v a H_out(v)
        for (String v : state.V) {
            //attention: the min sidetrack in Hout is already removed in 
            //private BinaryHeap<URIEdgeDelta> H_out(String v, State state)
            //by using v2minSidetrack and continue
            BinaryHeap<URIEdgeDelta> H_out = H_out(v, state);
            state.v2H_out.put(v, H_out);

            //root to the heap
            //if(!H_out.isEmpty())
            //    state.v2H_out_root.put(v, H_out.element());
            //to graph and change origin
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_out_g = H_out.toGraph();
            //change origin for debug
            for (URIEdgeDelta H_out_v_graph_v : H_out_g.vertexSet()) {
                //change origin to know that is was H_out
                H_out_v_graph_v.setOrigin("H_out");
            }

            state.v2H_out_graph.put(v, H_out_g);

            if (!useOnlyTheseDots || onlyTheseDots.contains(v)) {
                writeDot("02_H_out_" + URIEdge.cutURI(v), H_out_g);
            }
        }
    }

    /**
     * Let out(v) denotes the edges in G-T with tails in v.
     *
     * @param v
     * @param edgesInGMinusT
     * @return
     */
    private Set<URIEdge> out(String v, State state) {
        //no vertex in graph -> so no edges
        if (!state.GminusT.containsVertex(v)) {
            return new HashSet<>();
        }

        //return edgesInGMinusT.stream().filter(e -> e.getSource().equals(v)).collect(toSet());
        return state.GminusT.outgoingEdgesOf(v);
    }

    private BinaryHeap<URIEdgeDelta> H_out(String v, State state) {
        return H_out(v, state, new HashSet<>());
    }

    /**
     * build a heap for each v of the edges in out(v).
     *
     * @param v
     * @return
     */
    private BinaryHeap<URIEdgeDelta> H_out(String v, State state, Set<URIEdge> disallow) {
        BinaryHeap<URIEdgeDelta> h = new BinaryHeap<>();
        URIEdgeDelta minSidetrack = state.v2minSidetrack.get(v);
        for (URIEdge e : out(v, state)) {

            //special disallow
            if (disallow.contains(e)) {
                continue;
            }

            //the minimal sidetrack of this node v should not be in H_out(v)
            if (e.equals(minSidetrack.getEdge())) {
                continue; //however here we could miss one
            }
            //we only add to Hout if target is in T (so if we go the edge we will come to final target dstURI)
            boolean targetContainsInT = state.T.vertexSet().contains(e.getTarget());
            if (targetContainsInT) {
                h.add(new URIEdgeDelta(v, e, state.delta.get(e)));//always add changes nothing
            }
        }

        return h;
    }

    //============ H_T + H_out =================================================
    private DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combineLazy(String v, State state) throws IOException {
        return combineLazy(v, state, new HashSet<>());
    }

    private DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combineLazy(String v, State state, Set<URIEdge> disallow) throws IOException {

        //if we have it already computed: return it
        if (disallow.isEmpty() && state.v2merged_graph.containsKey(v)) {
            return state.v2merged_graph.get(v);
        }

        //if not we calculate it now
        DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge> merged = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_T_g;
        if (disallow.isEmpty()) {
            H_T_g = state.v2H_T_graph.get(v);
        } else { //never disallow in H_T because then we miss some good
            //H_T_g = H_T(v, state, disallow).toGraph();//new but disallow some
            H_T_g = state.v2H_T_graph.get(v);
            
            //BinaryHeap<URIEdgeDelta> bh = state.v2H_T.get(v);
            //URIEdgeDelta bh_head = bh.element();
            
            //boolean notP = !bh_head.getEdge().getEdge().equals(state.P);
            
            //if(notP) {
            //    bh.remove();
            //}
            
            //H_T_g = bh.toGraph();
            
        }
        if (H_T_g.vertexSet().isEmpty()) {
            return merged;
        }

        CloneResult H_T_cloneres = copyClone(H_T_g, merged);//need long because of clone, addEdge, containsEdge, hashcode
        //org.jgrapht.graph.AbstractBaseGraph.addEdge (Object, Object)

        //for each vertex, not only for root
        for (URIEdgeDelta H_T_g_v : H_T_cloneres.clone.vertexSet()) {

            String tail = H_T_g_v.getEdge().getSource();

            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_out_g;
            if (disallow.isEmpty()) {
                H_out_g = state.v2H_out_graph.get(tail);
            } else {
                H_out_g = H_out(tail, state, disallow).toGraph();//new but disallow some
            }

            if (H_out_g == null) {
                //tailHoutNotFound.add(tail);
                continue;
            }

            boolean emptyHout = H_out_g.vertexSet().isEmpty();
            if (emptyHout) {
                //emptyHouts.add(tail);
                continue;
            }

            CloneResult H_out_cloneres = copyClone(H_out_g, merged);
            //setOwner(H_out_cloneres.clone, H_T_g_v.getOwner());
            setOwner(H_out_cloneres.clone, v);//always correct owner!

            //H_T root must point to H_out
            merged.addEdge(H_T_g_v, H_out_cloneres.root);//need long because of contains edge check hash code
        }

        recordIncrementalCosts(merged);

        //store it for retrieval in the feature
        //if we want to construct a special combined with disallowed we
        //don't want to store it
        //so, if we disallow nothing -> normal case -> store
        if (disallow.isEmpty()) {
            state.v2merged_graph.put(v, merged);

            if (!useOnlyTheseDots || onlyTheseDots.contains(v)) {
                writeDot("03_H_T_with_H_out_" + URIEdge.cutURI(v), merged);
            }
        }

        return merged;
    }

    private void recordIncrementalCosts(DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge> givenG) throws IOException {
        //Record incremental costs
        //logger.debug("record incremental costs...");
        for (DefaultWeightedEdge e : givenG.edgeSet()) {
            URIEdgeDelta source = givenG.getEdgeSource(e);
            URIEdgeDelta target = givenG.getEdgeTarget(e);

            double incrementalCost = target.getDelta() - source.getDelta();

            ((WeightedGraph) givenG).setEdgeWeight(e, incrementalCost);
        }
        //writeDot("record-incremental-costs", g);
    }

    //======================= P_G and final result =============================
    private List<Path> constructiveBFSAll(State state) throws IOException {

        logger.debug("constructiveBFSAll");

        //all found pathes
        List<Path> pathes = new LinkedList<>();

        //P(G) however only a part of it
        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //the heap for bfs
        Queue<URIEdgeDelta> H = new PriorityQueue<>();

        //maps from one node to a valid built path
        //Map<URIEdgeDelta, Path> v2validPath = new HashMap<>();

        //initial shortest
        H.add(state.init);
        P_G.addVertex(state.init);
        //BuildPathResult initBPR = buildPath(new Sidetracks(Arrays.asList()), state);
        //state.init.setInvalidType(initBPR.type);
        //if (initBPR.valid) {
        //    v2validPath.put(state.init, initBPR.path);
        //}

        //to know the iteration
        int iteration = 0;

        //for unique owner rename
        int uniqueOwner = 1;

        //for logging
        final int everyIterationLog = Integer.MAX_VALUE;

        //for statistics
        Map<InvalidType, Integer> type2count = new HashMap<>();
        for (InvalidType t : InvalidType.values()) {
            type2count.put(t, 0);
        }

        //pruned because of duplicate sidetrack
        //int pruned = 0;
        
        //count for non construct a cross edge (saved useless nodes)
        int nonConstructed = 0;

        while (!H.isEmpty() /*&& iteration < max*/ && pathes.size() < state.k) {

            //direct harvest ==================================================
            URIEdgeDelta r_c1 = H.remove();//k=0 init-FM
          
            //only color relevant nodes
            BuildPathResult bpres = colorInvalid(r_c1, state);
            
            //for statistic
            type2count.put(bpres.type, type2count.get(bpres.type) + 1);
            
            //only if valid add to found
            if(bpres.valid) {
                pathes.add(bpres.path);
            }
            
            if(bpres.type == InvalidType.Sidetrack) {
                //logger.debug(r_c1.getSidetracks().toString());
            }
            
            //or we have to overjump it
            //direct harvest ==================================================

            //condition for further construction (cross edge adding)
            boolean construct = (
                            r_c1.getInvalidType() == InvalidType.None ||                         //after a valid path there can come many other valid pathes  
                            r_c1.getInvalidType() == InvalidType.ShortestPath ||                 //shortest path walked twice or more however a sidetrack can help to walk aroud it
                            r_c1.getInvalidType() == InvalidType.PropertyPathBcOfShortestPath    //the shortest path makes property path invalid, however a sidetrack could help
                        );
            //if we overjump construct we save
            //adding useless nodes which at the end always makes pathes invalid
            //copyClone where addVertex addEdge is expensive
            
            //construction or prune ============================================
            if (construct) {
                
                //we have to add for r_c1 to can do P_G.outgoingEdgesOf(r_c1)
                //were the cur points to
                String head = r_c1.getEdge().getTarget();
                //the atoms we need for P_G
                DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined = combineLazy(head, state); //state.v2merged_graph.get(head);
                
                //special case for P
                //if(state.P != null && r_c1 == state.init) {
                //    Set<URIEdge> disallow = g.outgoingEdgesOf(head).stream().filter(e -> !e.getEdge().equals(state.P)).collect(Collectors.toSet());
                //   combined = combineLazy(head, state, disallow);
                //}
                
                //if combined isnot empty there is a cross edge from r_c1
                if (!combined.vertexSet().isEmpty()) {

                    //color all nodes invalid
                    //List<BuildPathResult> buildPathResults = colorInvalid(r_c1, combined, state);

                    //statistic of invalid types by counting them
                    //for (BuildPathResult bpr : buildPathResults) {
                    //    type2count.put(bpr.type, type2count.get(bpr.type) + 1);
                    //}

                    /*
                    long invalidCount = buildPathResults.stream().filter(bpr -> !bpr.valid).count();
                    if(invalidCount > 0) {
                        writeDot(String.format("colored_invalid_%03d_%dvs%d_%s-%s_%dvalid",
                                iteration, 
                                invalidCount, 
                                combined.vertexSet().size(), 
                                URIEdge.cutURI(cur.getEdge().getSource()), 
                                URIEdge.cutURI(cur.getEdge().getTarget()),
                                P_GvalidCount
                        ), combined);
                    }
                         */
                        //when we do this we stop at
                        //iteration: 57 / found pathes: 19 / extra: 38 / k: 20 / pruned: 0
                        //for example test 3
                        /*
                    boolean allSidetrack = buildPathResults.stream().allMatch(bpr -> bpr.type == InvalidType.Sidetrack);
                    boolean prune = buildPathResults.stream().allMatch(
                            bpr -> 
                            bpr.type == InvalidType.Sidetrack ||
                            bpr.type == InvalidType.NeverReachDst ||
                            bpr.type == InvalidType.PropertyPath);
                         */
                    //the more we can prune
                    //the faster we are
                    //because we have not to do that many extra checkings
                    //iteration: 6000 / found pathes: 5428 / extra: 572 / k: 5483 / pruned: 238 / used MB: 706 / P(G).vertices 163373 / H 11373 / invalid type statistic: {NeverReachDst=9, PropertyPath=1653, ShortestPath=38577, None=117216, Sidetrack=12678}
                    //if one is None or ShortestPath we can not prune
                    //because there could be come a sidetrack
                    //which will turn around the invalid ShortestPath to valid
                    //any match should be faster in positive case
                    //boolean noPrune = buildPathResults.stream().anyMatch(
                    //        bpr
                    //        -> bpr.type == InvalidType.None
                    //        || bpr.type == InvalidType.ShortestPath);

                    //do next or prune
                    //if (noPrune) {
                        //clone and add to P(G)

                        //clone combined to P_G
                        CloneResult g_cloneres = copyClone(combined, P_G);

                        //unique owner
                        setOwner(g_cloneres.clone, head + ":" + (uniqueOwner++));

                        //make the cross edge and set weight
                        DefaultWeightedEdge e = P_G.addEdge(r_c1, g_cloneres.root);
                        ((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) P_G).setEdgeWeight(e, g_cloneres.root.getDelta());

                        //already built
                        //for (BuildPathResult bpr : buildPathResults) {
                            //only valid builds
                        //    if (bpr.valid) {
                                //use mapper to come from old (combined) in new (clone)
                        //        URIEdgeDelta v = g_cloneres.mapper.get(bpr.uriEdgeDelta);
                        //        v2validPath.put(v, bpr.path);
                        //    }
                        //}

                    //} else {
                    //    pruned++;
                    //}
                }
            } else {
                nonConstructed++;
            }
            //construction =====================================================

            //bfs ==============================================================
            //for j = each edge from r -> s with cost c2
            Set<DefaultWeightedEdge> es = P_G.outgoingEdgesOf(r_c1);
            for (DefaultWeightedEdge e : es) {

                //the nodes to outgoing edges
                URIEdgeDelta s = P_G.getEdgeTarget(e);

                //(c1+c2)
                s.setDelta(r_c1.getDelta() + P_G.getEdgeWeight(e));

                s.setParent(r_c1);

                //if we prune in contruction we dont need this
                //works in example test 3 k=20
                //boolean prune = s.getInvalidType() == InvalidType.Sidetrack &&
                //!s.getOwner().equals(r_c1.getOwner()) &&
                //r_c1.getInvalidType() != InvalidType.ShortestPath &&
                //r_c1.getInvalidType() != InvalidType.None;
                //if(!prune) {
                H.add(s);
                //} else {
                //    pruned++;
                //}
            }
            //clean up P_G because we will never need it
            //P_G.removeVertex(r_c1);
            //bfs ==============================================================

            //validCount = P_G.vertexSet().stream().filter(e -> e.isValid()).count();
            writeDot(String.format("P_G_%03d_%dfound", iteration, pathes.size()), P_G);

            //logger.warn("P(G) has {} vertices and {} edges", P_G.vertexSet().size(), P_G.edgeSet().size());
            iteration++;

            if (iteration != 0 && iteration % everyIterationLog == 0) {
                logger.debug("iteration: {} / found pathes: {} / extra: {} / k: {} / nonConstructed: {} / used MB: {} / P(G).vertices {} / H {} / invalid type statistic: {}",
                        iteration,
                        pathes.size(),
                        iteration - pathes.size(),
                        state.k,
                        nonConstructed,
                        PrintHeapUsage.getUsedMB(),
                        P_G.vertexSet().size(),
                        H.size(),
                        type2count
                );
            }
        }

        logger.debug("after {} iterations we received {} valid pathes and H contains {} entries", iteration, pathes.size(), H.size());

        logger.debug("special cases: {}", specialcase);
        
        state.end = System.currentTimeMillis();
        
        allcsv.printRecord(
                state.srcURI, 
                state.dstURI, 
                state.srcURISpecial, 
                state.k, 
                state.P, 
                
                state.G.vertexSet().size(), //n
                state.G.edgeSet().size(), //m
                
                state.G.edgeSet().size() + state.G.vertexSet().size() * Math.log(state.G.vertexSet().size()) + state.k, //O(m +n log n +k)
                
                state.GminusT.vertexSet().size(),
                state.GminusT.edgeSet().size(),
                
                state.T.vertexSet().size(),
                state.T.edgeSetPrevious().size(),
                
                state.delta.size(),
                state.v2minSidetrack.size(),
                
                iteration,
                pathes.size(),
                iteration - pathes.size(),
                (iteration - pathes.size()) / (double) pathes.size(),
                
                state.begin,
                state.end,
                state.end - state.begin,
                
                nonConstructed,
                PrintHeapUsage.getUsedMB(),
                P_G.vertexSet().size(),
                P_G.edgeSet().size(),
                H.size(),
                
                type2count.get(InvalidType.None),
                type2count.get(InvalidType.Sidetrack),
                type2count.get(InvalidType.ShortestPath),
                type2count.get(InvalidType.NeverReachDst)
        );
        
        return pathes;
    }

    
    private BuildPathResult colorInvalid(URIEdgeDelta fromH, State state) {
        
        List<URIEdge> sidetracks = fromH.getSidetracks();
        sidetracks.remove(state.init.getEdge());
        Sidetracks sts = new Sidetracks(sidetracks);

        BuildPathResult bpres = buildPath(sts, state);
        bpres.uriEdgeDelta = fromH;

        fromH.setInvalidType(bpres.type);
        
        return bpres;
    }
    
    private URIEdgeDelta getRoot(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined) {
        if (combined.vertexSet().isEmpty()) {
            return null;
        }

        for (URIEdgeDelta v : combined.vertexSet()) {
            if (combined.incomingEdgesOf(v).isEmpty()) {
                return v;
            }
        }

        return null;
    }

    int specialcase = 0;
    
    private BuildPathResult buildPath(Sidetracks sidetracks, State state) {

        
        URIEdge additional = null;
        if(state.P != null) {
            if(sidetracks.getEdges().isEmpty()) {
                URIEdge Tedge = state.T.getPrevious(state.srcURI);

                //if never existing vertex has no edge in T
                //if(Tedge == null) {
                //    return new BuildPathResult(null, InvalidType.ShortestPath);
                //}

                additional = new URIEdge(Tedge.getEdge(), state.srcURISpecial, Tedge.getTarget());
            } else {
                //sidetrack is first or use shortest path
                if(sidetracks.getEdges().get(0).getSource().equals(state.srcURI)) {
                    specialcase++;

                    URIEdge e = sidetracks.getEdges().get(0);

                    additional = new URIEdge(e.getEdge(), state.srcURISpecial, e.getTarget());
                } else {
                    URIEdge Tedge = state.T.getPrevious(state.srcURI);
                    additional = new URIEdge(Tedge.getEdge(), state.srcURISpecial, Tedge.getTarget());
                }
            }
        }
        
        //array list for fast access
        List<URIEdge> nextSidetracks = new ArrayList<>(sidetracks.getEdges());

        //we can faster decide if InvalidType.PropertyPath
        //check without building path
        
        //if(state.P != null) {
        /*
        if (false) {

            URIEdge first;
            URIEdge last;

            boolean shortestPathTreeUsed = false;
            
            //no sidetrack so we have to use the ones in T
            if (nextSidetracks.isEmpty()) {
                first = state.T.getPrevious(state.srcURI);
                last = state.T.getDestinationEdge();
                shortestPathTreeUsed = true;
            } else {
                first = nextSidetracks.get(0);
                last = nextSidetracks.get(nextSidetracks.size() - 1);
            }

            //if not sidetrack comes from src it is in T
            if (!nextSidetracks.isEmpty() && !first.getSource().equals(state.srcURI)) {
                first = state.T.getPrevious(state.srcURI);
                shortestPathTreeUsed = true;
            }

            //if not sidetrack goes to dst it is in T
            if (!nextSidetracks.isEmpty() && !last.getTarget().equals(state.dstURI)) {
                last = state.T.getDestinationEdge();
                shortestPathTreeUsed = true;
            }

            if (!(first.getEdge().equals(state.P) || last.getEdge().equals(state.P))) {
                
                return new BuildPathResult(null, 
                        shortestPathTreeUsed ? 
                                InvalidType.PropertyPathBcOfShortestPath :
                                InvalidType.PropertyPathBcOfSidetrack);
                
            }
        }
        */
        
        //we detect that a sidetrack is activated twice or more
        if (sidetracks.hasDuplicates(additional)) {
            return new BuildPathResult(null, InvalidType.Sidetrack);
        }

        //Queue<URIEdge> nextSidetracks = new ArrayBlockingQueue<>(
        //        sidetracks.getEdges().size(),
        //        false,
        //        sidetracks.getEdges());
        int nextSidetrackIndex = 0;

        InvalidType t;

        //createResource
        //createProperty needs time!
        //for building path
        List<String> uris = new ArrayList<>(1000);
        String cur = state.srcURI;
        //p.add(ResourceFactory.createResource(cur));//s
        uris.add(cur);

        //for duplication check
        Set<URIEdge> usedTracks = new HashSet<>();
        Set<Integer> usedTracksHashes = new TreeSet<>();
        
        int modulo = 1000000;
        boolean[] bitmap = new boolean[modulo];
        
        //additional start
        if(additional != null) {
            //bitmap[Math.abs(additional.hashCode()) % modulo] = true;
            usedTracks.add(additional);
        }

        int iteration = 0;
        
        while (!(/*nextSidetracks.isEmpty()*/nextSidetrackIndex >= nextSidetracks.size() && cur.equals(state.dstURI))) {

            URIEdge track;

            //we go the sidetrack
            if (/*!nextSidetracks.isEmpty()*/nextSidetrackIndex < nextSidetracks.size() && /*nextSidetracks.element()*/ nextSidetracks.get(nextSidetrackIndex).getSource().equals(cur)) {
                //track = nextSidetracks.poll();
                track = nextSidetracks.get(nextSidetrackIndex++);
                t = InvalidType.Sidetrack;
            } else {
                //we go spt
                
                //old:
                //for P we use changed Tprime
                //if(state.P != null && iteration == 0) {
                    //track = state.Tprime.getPrevious(cur);
                //} else {
                    
                //}
                
                track = state.T.getPrevious(cur);
                t = InvalidType.ShortestPath;

                //path ends
                //a path which will never end in dstURI because cur can not reach dstURI
                if (track == null) {
                    return new BuildPathResult(null, InvalidType.NeverReachDst);
                }
            }

            //returns true if this set did not already contain the specified element
            //duplicate detection
            if(!usedTracks.add(track)) { 
                return new BuildPathResult(null, t);
            }
            

            //duplicate check with bitmap
            //int key = Math.abs(track.hashCode()) % modulo;
            //if (!bitmap[key]) {
            //    bitmap[key] = true;
            //} else {
            //    return new BuildPathResult(null, t);
            //}
           
           
            //if(!usedTracksHashes.add(track.hashCode())) { //the add needs some time
            //    return new BuildPathResult(null, t);
            //}

            //building
            //p.add(ResourceFactory.createProperty(track.getEdge()));//p
            //p.add(ResourceFactory.createResource(track.getTarget()));//o
            uris.add(track.getEdge());
            uris.add(track.getTarget());

            //next
            cur = track.getTarget();
            iteration++;
        }

        //if we have this contraint
        //we check it at the beginning
        /*
        if(P != null) {
            //Property p1 = (Property) p.getPath().get(1);
            //Property pn = (Property) p.getPath().get(p.resourceCount() - 2);
            String p1 = uris.get(1);
            String pn = uris.get(uris.size() - 2);
            
            //if not (p1 = P or pn = P)
            //if(!(p1.hasURI(P) || pn.hasURI(P))) {
            if(!(p1.equals(P) || pn.equals(P))) {
                return new BuildPathResult(null, InvalidType.PropertyPath);
            }
        }
         */
        
        //cleanup
        if(state.P != null) {
            uris.set(0, state.srcURISpecial);
        }
        
        //logger.debug("\t" + p.toString());
        //ResourceFactory.create needs some time so do it
        //only if valid path
        Path p = new Path();
        p.add(ResourceFactory.createResource(uris.get(0)));
        for (int i = 1; i < uris.size(); i += 2) {
            p.add(ResourceFactory.createProperty(uris.get(i)));
            p.add(ResourceFactory.createResource(uris.get(i + 1)));
        }

        //special case
        if(p.resourceCount() == 1) {
            return new BuildPathResult(null, InvalidType.ShortestPath);
        }
        
        //everything ok
        return new BuildPathResult(p, InvalidType.None);
    }

    private boolean validPath(Sidetracks sidetracks, State state) {
        return buildPath(sidetracks, state).valid;
    }

    //================= util ===================================================
    
    private CloneResult copyClone(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> src, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> dst) {

        if (src.vertexSet().isEmpty()) {
            throw new RuntimeException("it is not allowed to copy an empty source graph");
        }

        Map<URIEdgeDelta, URIEdgeDelta> mapper = new HashMap<>();

        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> clone = clone(src, mapper);

        if (clone.vertexSet().size() != src.vertexSet().size()) {
            throw new RuntimeException("clone error vertices");
        }
        if (clone.edgeSet().size() != src.edgeSet().size()) {
            throw new RuntimeException("clone error edges");
        }

        URIEdgeDelta root = null;

        for (URIEdgeDelta v : clone.vertexSet()) {

            if (clone.incomingEdgesOf(v).isEmpty()) {
                if (root == null) {
                    root = v;
                } else {
                    throw new RuntimeException("more then one root in src");
                }
            }

            dst.addVertex(v);
        }

        if (root == null) {
            throw new RuntimeException("root not found");
        }

        for (DefaultWeightedEdge e : clone.edgeSet()) {
            dst.addEdge(clone.getEdgeSource(e), clone.getEdgeTarget(e), e);
        }

        return new CloneResult(clone, root, mapper);
    }

    private DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> clone(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> givenG, Map<URIEdgeDelta, URIEdgeDelta> mapper) {
        DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge> copy = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //Map<URIEdgeDelta, URIEdgeDelta> mapper = new HashMap<>();
        for (URIEdgeDelta v : givenG.vertexSet()) {
            URIEdgeDelta v_copy = new URIEdgeDelta(v);
            copy.addVertex(v_copy);
            mapper.put(v, v_copy);
        }

        for (DefaultWeightedEdge e : givenG.edgeSet()) {
            DefaultWeightedEdge e_copy = copy.addEdge(mapper.get(givenG.getEdgeSource(e)), mapper.get(givenG.getEdgeTarget(e)));
            //copy weight
            copy.setEdgeWeight(e_copy, givenG.getEdgeWeight(e));
        }

        return copy;
    }

    private DirectedGraph<String, URIEdge> cloneReference(DirectedGraph<String, URIEdge> givenG, Class<? extends URIEdge> edgeClass, Set<URIEdge> disallow, boolean invert) {
        
        DirectedWeightedPseudograph<String, URIEdge> copy = new DirectedWeightedPseudograph<>(edgeClass);
        
        //Map<URIEdgeDelta, URIEdgeDelta> mapper = new HashMap<>();
        for (String v : givenG.vertexSet()) {
            copy.addVertex(v);
        }

        for (URIEdge e : givenG.edgeSet()) {
            if(!disallow.contains(e)) {
                if(invert) 
                    copy.addEdge(givenG.getEdgeTarget(e), givenG.getEdgeSource(e), new URIEdge(e.getEdge(), givenG.getEdgeTarget(e), givenG.getEdgeSource(e)));
                else
                    copy.addEdge(givenG.getEdgeSource(e), givenG.getEdgeTarget(e), e);
            }
        }

        return copy;
    }

    private void setOwner(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> g, String owner) {
        for (URIEdgeDelta v : g.vertexSet()) {
            v.setOwner(owner);
        }
    }

    private void writeDot(String name, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> g) throws IOException {
        if (!writeDot) {
            return;
        }

        //for visualization
        DOTExporter<URIEdgeDelta, DefaultWeightedEdge> de = new DOTExporter<>(
                (URIEdgeDelta v) -> toID(v), //id
                (URIEdgeDelta v) -> toLabel(v), //label

                (DefaultWeightedEdge e) -> String.valueOf(g.getEdgeWeight(e)) //label
        );

        FileWriter fw = new FileWriter(new File(outFolder, name + ".dot"));
        de.export(fw, g);
        fw.close();
    }

    private void writeDotStringURIEdge(String name, DirectedGraph<String, URIEdge> g) throws IOException {
        if (!writeDot) {
            return;
        }

        //for visualization
        DOTExporter<String, URIEdge> de = new DOTExporter<>(
                (String v) -> "\"" + v + "\"", //id
                (String v) -> v, //label
                (URIEdge e) -> e.toString()
        );

        FileWriter fw = new FileWriter(new File(outFolder, name + ".dot"));
        de.export(fw, g);
        fw.close();
    }

    private String toID(URIEdgeDelta v) {
        return "\"" + "ID" + String.valueOf(v.hashCode()) + "\"";
    }

    private String toLabel(URIEdgeDelta v) {
        return String.format("%s %s (%s-%s) %s", v.getEdge().toString(), v.getDelta(), URIEdge.cutURI(v.getOwner()), v.getOrigin(), v.getInvalidType());
    }

    private void emptyFolder(File folder, boolean isRoot) {
        File[] files = folder.listFiles();
        if (files != null) {
            for (File f : files) {
                if (f.isDirectory()) {
                    emptyFolder(f, false);
                } else {
                    f.delete();
                }
            }
        }
        if (!isRoot) {
            folder.delete();
        }
    }

    private void createOutfolder(String srcURI, String dstURI, int k) {
        outFolder = new File("dot_and_graphml", "" + k);
        if (!outFolder.exists()) {
            outFolder.mkdirs();
        } else {
            emptyFolder(outFolder, true);
        }
    }

    private void logHeap(Queue<URIEdgeDelta> H) {
        StringBuilder sb = new StringBuilder();

        if (H.isEmpty()) {
            sb.append("empty");
        }

        for (URIEdgeDelta e : H) {
            sb.append(toLabel(e));
            sb.append(" < ");
        }

        logger.debug("Heap H is {}", sb.toString());
    }

    //================= private classes ========================================
    private class Sidetracks {

        public List<URIEdge> edges;

        public Sidetracks(List<URIEdge> edges) {
            this.edges = edges;
        }

        public List<URIEdge> getEdges() {
            return edges;
        }

        public boolean hasDuplicates(URIEdge additional) {
            return hasDuplicatesWithHashSet(additional);
        }

        public boolean hasDuplicatesWithHashSet(URIEdge additional) {
            return new HashSet<>(edges).size() < edges.size();
        }

        public boolean hasDuplicatesWithBitmap(URIEdge additional) {
            //is a problem for task1_q1_1.068 for modulo = 10000
            int modulo = 1000000;
            boolean[] bitmap = new boolean[modulo];
            
            //more
            if(additional != null) {
                bitmap[Math.abs(additional.hashCode()) % modulo] = true;
            }
            
            for (URIEdge e : edges) {
                int key = Math.abs(e.hashCode()) % modulo;
                if (!bitmap[key]) {
                    bitmap[key] = true;
                } else {
                    return true;
                }
            }
            return false;
        }
    }

    private class CloneResult {

        public DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> clone;
        public URIEdgeDelta root;
        public Map<URIEdgeDelta, URIEdgeDelta> mapper;

        public CloneResult(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> clone, URIEdgeDelta root, Map<URIEdgeDelta, URIEdgeDelta> mapper) {
            this.clone = clone;
            this.root = root;
            this.mapper = mapper;
        }

    }

    private class BuildPathResult {

        public Path path;
        public InvalidType type;
        public boolean valid;
        public URIEdgeDelta uriEdgeDelta;

        public BuildPathResult(Path path, InvalidType type) {
            this.path = path;
            this.type = type;
            this.valid = type == InvalidType.None;
        }

    }

    private class URIEdgeDeltaComparator implements Comparator<URIEdgeDelta> {

        @Override
        public int compare(URIEdgeDelta a, URIEdgeDelta b) {
            int compareDelta = a.compareTo(b);
            if (compareDelta != 0) {
                return compareDelta;
            }
            return a.getEdge().getEdge().compareTo(b.getEdge().getEdge());
        }

    }

    //========== deprecated ====================================================
    
    @Deprecated
    private List<BuildPathResult> colorInvalid(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined, State state) throws IOException {

        List<BuildPathResult> buildPathResults = new LinkedList<>();

        for (URIEdgeDelta v : combined.vertexSet()) {
            v.setParent(cur);

            List<URIEdge> sidetracks = v.getSidetracks();
            sidetracks.remove(state.init.getEdge());
            Sidetracks sts = new Sidetracks(sidetracks);

            BuildPathResult bpres = buildPath(sts, state);
            bpres.uriEdgeDelta = v;
            buildPathResults.add(bpres);

            v.setInvalidType(bpres.type);
        }

        return buildPathResults;
    }
    
    @Deprecated
    private void writeGraphML(String name, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> g) throws Exception {
        if (!writeGraphML) {
            return;
        }

        //for visualization
        GraphMLExporter<URIEdgeDelta, DefaultWeightedEdge> de = new org.jgrapht.ext.GraphMLExporter<>(
                (URIEdgeDelta v) -> toID(v), //id
                (URIEdgeDelta v) -> toLabel(v), //label

                (DefaultWeightedEdge e) -> String.valueOf(e.hashCode()), //id
                (DefaultWeightedEdge e) -> String.valueOf(g.getEdgeWeight(e)) //label
        );

        FileWriter fw = new FileWriter(new File(outFolder, name + ".graphml"));
        de.export(fw, g);
        fw.close();
    }

    @Deprecated
    private DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> checkAndCleanup(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined, State state) throws IOException {

        //to do the normal stuff
        //return combined;
        //after 20152 / 41304 iterations we received 20152 valid pathes
        //20171 buildPath.null-tracks / 0 constructiveBFSAll.duplicate pathes / 0 constructiveBFSAll.twice constructed
        //k + trackDuplicate + twice constucted = 20152
        //check cur to all v in combined
        URIEdgeDelta root = getRoot(combined);
        Set<URIEdge> disallow = new HashSet<>();

        //bfs
        Queue<URIEdgeDelta> q = new LinkedList<>();
        q.add(root);
        while (!q.isEmpty()) {
            URIEdgeDelta c = q.poll();

            c.setParent(cur);

            List<URIEdge> sidetracks = c.getSidetracks();
            sidetracks.remove(state.init.getEdge());
            Sidetracks sts = new Sidetracks(sidetracks);
            boolean valid = validPath(sts, state);
            //c.setValid(valid);

            //if not valid then disallow the next time
            if (!valid) {
                disallow.add(c.getEdge());
            }

            //bfs
            for (DefaultWeightedEdge e : combined.outgoingEdgesOf(c)) {
                q.add(combined.getEdgeTarget(e));
            }
        }

        if (!disallow.isEmpty()) {

            //if we remove a disallowed we maybe will miss a crossedge from this
            //invalid node
            //check this assumption?
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> cleaned = combineLazy(root.getOwner(), state, disallow);

            //writeDot("invalid", combined);
            //writeDot("cleaned", cleaned);
            return cleaned;
        }

        return null;
    }

    @Deprecated
    private List<Sidetracks> constructiveBFSValidHeap(State state) throws IOException {

        List<Sidetracks> sidetrackResult = new LinkedList<>();

        //P(G) however only a part of it
        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //try with a comparator
        //URIEdgeDeltaComparator c = new URIEdgeDeltaComparator();
        //the heap for bfs
        Queue<URIEdgeDelta> H = new PriorityQueue<>();//c);

        //initial shortest
        H.add(state.init);
        P_G.addVertex(state.init);

        int uniqueOwner = 1;

        for (int k = 0; k < state.k && !H.isEmpty(); k++) {

            //direct harvest ==================================================
            //logger.debug("H.remove() from {} sized heap...", H.size());
            URIEdgeDelta r_c1 = H.remove();//k=0 init-FM

            //logHeap(H);
            //harvest
            List<URIEdge> sidetracks = r_c1.getSidetracks();
            sidetracks.remove(state.init.getEdge());//we need the init edge not for building shortest path
            Sidetracks sts = new Sidetracks(sidetracks);
            //logger.debug("\tsidetracks: " + sidetracks);

            sidetrackResult.add(sts);
            //direct harvest ==================================================

            //construction =====================================================
            //we have to add for r_c1 to can do P_G.outgoingEdgesOf(r_c1)
            //were the cur points to
            String head = r_c1.getEdge().getTarget();
            //the atoms we need for P_G
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined = combineLazy(head, state); //state.v2merged_graph.get(head);
            //if combined isnot empty there is a cross edge from r_c1
            if (!combined.vertexSet().isEmpty()) {
                //clone combined to P_G
                CloneResult g_cloneres = copyClone(combined, P_G);

                //unique owner
                setOwner(g_cloneres.clone, head + (uniqueOwner++));

                //make the cross edge and set weight
                DefaultWeightedEdge e = P_G.addEdge(r_c1, g_cloneres.root);
                ((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) P_G).setEdgeWeight(e, g_cloneres.root.getDelta());
            }
            //logger.warn("cross edge added for {}", URIEdge.cutURI(head));
            //construction =====================================================

            //bfs ==============================================================
            //for j = each edge from r -> s with cost c2
            Set<DefaultWeightedEdge> es = P_G.outgoingEdgesOf(r_c1);
            for (DefaultWeightedEdge e : es) {

                //the nodes to outgoing edges
                URIEdgeDelta s = P_G.getEdgeTarget(e);

                //(c1+c2)?
                s.setDelta(r_c1.getDelta() + P_G.getEdgeWeight(e));

                //if(s.getParent() == null) {
                s.setParent(r_c1);
                //} else {
                //we don't want to have loops!
                //logger.warn("waring: s has already parent {}, but should set parent {}", s.getParent(), r_c1);
                //throw new IOException("s has already parent");
                //}

                //we decide here if this is a possible path
                List<URIEdge> ssts = s.getSidetracks();
                ssts.remove(state.init.getEdge());//we need the init edge not for building shortest path
                Sidetracks sSidetracks = new Sidetracks(ssts);
                //s.setValid(validPath(sSidetracks, state));//s will in the future be a valid path

                if (s.isValid()) {
                    //so H will consist of possible valid pathes
                    H.add(s);
                }
            }
            //bfs ==============================================================

            //logHeap(H);
            //clean up P_G because we will never need it
            P_G.removeVertex(r_c1);

            //logger.warn("P(G) has {} vertices and {} edges", P_G.vertexSet().size(), P_G.edgeSet().size());
        }

        //logger.debug("we had {} dublicate triple pathes", state.dublicatePathCounter);
        return sidetrackResult;
    }

    //TODO: besprechen
    private void removeUnnecessaryV(String srcURI, State state) {
        //check if an incoming sidetrack edge exists for v
        logger.debug("check if an incoming sidetrack edge exists for v...");
        //in the presentation we want to remove c and e
        Set<String> removedV = new HashSet<>();
        for (String v : state.V.toArray(new String[0])) {
            if (!v.equals(srcURI) && state.GminusT.containsVertex(v) && state.GminusT.incomingEdgesOf(v).isEmpty()) {
                state.V.remove(v);
                removedV.add(v);
            }
        }
        logger.debug("we look at {} vertices, removed: {}", state.V.size(), removedV.size());
    }

    @Deprecated
    private void recordIncrementalCostsForMerged(State state) throws IOException {
        logger.debug("record incremental costs for " + state.V.size() + " vertices");
        for (String v : state.V) {
            recordIncrementalCosts((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) state.v2merged_graph.get(v));
            writeDot("03_H_T_with_H_out_inc_cost_" + URIEdge.cutURI(v), state.v2merged_graph.get(v));
        }
    }

    @Deprecated
    private void combineH_TwithH_outV2(State state) throws IOException {
        logger.debug("combine H_T with H_out for " + state.V.size() + " vertices");
        for (String v : state.V) {

            DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge> merged = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_T_g = state.v2H_T_graph.get(v);
            CloneResult H_T_cloneres = copyClone(H_T_g, merged);//need long because of clone, addEdge, containsEdge, hashcode
            //org.jgrapht.graph.AbstractBaseGraph.addEdge (Object, Object)

            //for each vertex, not only for root
            for (URIEdgeDelta H_T_g_v : H_T_cloneres.clone.vertexSet()) {

                String tail = H_T_g_v.getEdge().getSource();

                URIEdgeDelta H_out_root = state.v2H_out_root.get(tail);
                //if no h_out is there
                if (H_out_root == null) {
                    continue;
                }

                CloneResult H_out_cloneres = copyClone(state.v2H_out_graph.get(tail), merged);
                setOwner(H_out_cloneres.clone, H_T_g_v.getOwner());
                URIEdgeDelta other_merged_root = H_out_cloneres.root;

                //H_T root must point to H_out
                merged.addEdge(H_T_g_v, other_merged_root);//need long because of contains edge check hash code
            }

            state.v2merged_graph.put(v, merged);

            //writeDot("H_T_with_H_out_" + URIEdge.cutURI(v), merged);
        }
    }

    @Deprecated
    private DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> iterativeConstruction(State state, int I) throws IOException {
        logger.debug("iterativeConstruction with I = {}", I);
        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //queue and start with initial edge
        Queue<URIEdgeDelta> q = new LinkedList<>();
        q.add(state.init);
        P_G.addVertex(state.init);

        writeDot("04_000_P_G", P_G);

        int snapshot = 1;
        while (!q.isEmpty() && snapshot < I) {

            URIEdgeDelta cur = q.poll();

            //try to find a cross edges for cur
            //were the cur points to
            String head = cur.getEdge().getTarget();

            //the atoms we need for P_G
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined = combineLazy(head, state); //state.v2merged_graph.get(head);
            if (combined.vertexSet().isEmpty()) {
                //no cross edge possible
                continue;
            }

            //==================================================================
            //from here on you can do a mistake by copy everything to P_G
            //because we have the duplicate triple constraint
            duplicateTripleConstraintV1(cur, P_G, q, head, combined, state);

            //==================================================================
            writeDot(String.format("04_%03d_P_G", snapshot), P_G);
            snapshot++;
        }

        return P_G;
    }

    @Deprecated
    private void duplicateTripleConstraintV1(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G, Queue<URIEdgeDelta> q, String head, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined, State state) {
        //we have no checking

        //clone the graph
        CloneResult g_cloneres = copyClone(combined, P_G);

        //make the cross edge and set weight
        DefaultWeightedEdge e = P_G.addEdge(cur, g_cloneres.root);
        ((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) P_G).setEdgeWeight(e, g_cloneres.root.getDelta());

        //add copied graph to queue so that we will
        //make the cross edges too
        for (URIEdgeDelta v : g_cloneres.clone.vertexSet()) {
            q.add(v);
        }
    }

    @Deprecated
    private void duplicateTripleConstraintV2(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G, Queue<URIEdgeDelta> q, String head, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined, State state) {
        //we copy however delete again

        //clone the graph
        CloneResult g_cloneres = copyClone(combined, P_G);

        //make the cross edge and set weight
        DefaultWeightedEdge e = P_G.addEdge(cur, g_cloneres.root);
        ((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) P_G).setEdgeWeight(e, g_cloneres.root.getDelta());

        //check if the new graph will generate duplicate triples
        Set<URIEdgeDelta> duplicateTriplesV = new HashSet<>();
        Set<URIEdgeDelta> normalV = new HashSet<>();
        for (URIEdgeDelta v : g_cloneres.clone.vertexSet()) {
            //special case
            if (checkDuplicateTriple(v, P_G)) {
                duplicateTriplesV.add(v);
                logger.warn("duplicated triple detected: {}", v);
                logger.warn("we will delete it and all subgraphs...");
                Set<URIEdgeDelta> removed = removeAll(v, P_G);
                logger.warn("{} removed: {}", removed.size(), removed);
            } else {
                normalV.add(v);
            }
        }
        //if(!duplicateTriplesV.isEmpty()) {
        //    P_G.removeAllVertices(duplicateTriplesV);
        //}

        //add copied graph to queue so that we will
        //make the cross edges too
        for (URIEdgeDelta v : normalV) {
            q.add(v);
        }
    }

    @Deprecated
    private void duplicateTripleConstraintV3(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G, Queue<URIEdgeDelta> q, String head, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined, State state) {
        //we do not copy
        //we check for duplicate sidetrack triples
        //and
        //we check for duplicate shortest path triples by walking cycles

    }

    @Deprecated
    private boolean checkDuplicateTriple(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G) {

        List<URIEdgeDelta> path = new LinkedList();
        path.add(cur);

        Set<DefaultWeightedEdge> prevEdges;
        while (!(prevEdges = P_G.incomingEdgesOf(cur)).isEmpty()) {
            URIEdgeDelta prev = P_G.getEdgeSource(prevEdges.iterator().next());

            //only look at node where owner differs
            if (!prev.getOwner().equals(cur.getOwner())) {

                //here we have the exception
                for (URIEdgeDelta p : path) {

                    //we have seen the edge before
                    if (p.getEdge().equals(prev.getEdge())) {
                        //so return true for duplicate
                        return true;
                    }
                }

                //add to search further
                path.add(prev);
            }

            //next
            cur = prev;
        }

        return false;
    }

    @Deprecated
    private Set<URIEdgeDelta> removeAll(URIEdgeDelta cur, DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G) {

        Set<URIEdgeDelta> removed = new HashSet<>();

        Queue<URIEdgeDelta> q = new LinkedList<>();
        q.add(cur);

        while (!q.isEmpty()) {
            URIEdgeDelta c = q.poll();

            for (DefaultWeightedEdge e : P_G.outgoingEdgesOf(c)) {
                q.add(P_G.getEdgeTarget(e));
            }

            P_G.removeVertex(c);
            removed.add(c);
        }

        return removed;
    }

    @Deprecated
    private List<Sidetracks> bfs(DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G, State state) throws IOException {

        //Algorithm (Breadth-First Search):
        //push root.0 onto heap H
        //for i = 1 to k
        //pop node r.c1 from top of H
        //print cost c1 + 55
        //for j = each edge from r -> s with cost c2
        //push node s.(c1+c2) onto H
        List<Sidetracks> result = new LinkedList<>();

        //BinaryHeap<URIEdgeDelta> H = new BinaryHeap<>();
        Queue<URIEdgeDelta> H = new PriorityQueue<>();

        //push root.0 onto heap H
        //initial shortest
        H.add(state.init);

        //writeDot("heap_k000", H.toGraph());
        //if a path is invalid we have to search more
        int searchMore = 0;

        //we say k = 1 because we don't have the extra edge to root
        for (int k = 0; k < (state.k + searchMore) && !H.isEmpty(); k++) {

            logger.debug("H.remove() from {} sized heap...", H.size());
            //pop node r.c1 from top of H
            URIEdgeDelta r_c1 = H.remove();//k=0 init-FM

            //writeDot("heap_k" + k + "_1", H.toGraph());
            //print cost c1 + 55
            //logger.debug(r_c1 + ": " + (55 + r_c1.getDelta()));
            logger.debug("r_c1.getSidetracks()...");//was infinity because of parent problem
            List<URIEdge> sidetracks = r_c1.getSidetracks();
            sidetracks.remove(state.init.getEdge());//we need the init edge not for building shortest path
            Sidetracks sts = new Sidetracks(sidetracks);
            logger.debug("\tsidetracks: " + sidetracks);

            //special triple duplication constraint ============================
            if (buildPath(sts, state) != null) {
                result.add(sts);
            } else {
                logger.warn("duplicate detected");
                searchMore++;
                //continue;
            }

            //for j = each edge from r -> s with cost c2
            Set<DefaultWeightedEdge> es = P_G.outgoingEdgesOf(r_c1);
            for (DefaultWeightedEdge e : es) {
                //push node s.(c1+c2) onto H

                //I think this is always fresh
                URIEdgeDelta s = P_G.getEdgeTarget(e);

                //(c1+c2)?
                s.setDelta(r_c1.getDelta() + P_G.getEdgeWeight(e));
                //s.getAdditional().add(e);

                if (s.getParent() == null) {
                    s.setParent(r_c1);
                } else {
                    //if this happens then node s has at least two parents
                    //is this ok?
                    //then we want to go to the root and are in a loop
                    logger.warn("waring: s has already parent {}, but should set parent {}", s.getParent(), r_c1);
                    throw new IOException("s has already parent");
                }

                logger.debug("H.add(s)...");
                H.add(s);
            }

            //writeDot("heap_k" + k + "_2", H.toGraph());
        }

        if (H.isEmpty()) {
            logger.warn("bfs heap H is empty and {} sidetrack-combis collected", result.size());
        }

        logger.debug("bfs done");

        return result;
    }

    @Deprecated
    private List<Sidetracks> constructiveBFS(State state) throws IOException {
        /*
        Problem: es fehlen (11)er Pfade
        
        Gründe:
        1. durch das invalid path filtern gehen gute pfade verloren
        2. combineLazy kombiniert HT und Hout nicht vollständig
        
        
        beim hinzufügen zu H:
        we had 86 dublicate triple pathes
         */

        List<Sidetracks> sidetrackResult = new LinkedList<>();
        //List<Path> result = new LinkedList<>();

        //P(G) however only a part of it
        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //try with a comparator
        URIEdgeDeltaComparator c = new URIEdgeDeltaComparator();
        //the heap for bfs
        Queue<URIEdgeDelta> H = new PriorityQueue<>(c);

        //initial shortest
        H.add(state.init);
        P_G.addVertex(state.init);

        //writeDot(String.format("04_%03d_%03d_P_G", 0, 0), P_G);
        //if a path is invalid we have to search more
        //int searchMore = 0;
        state.dublicatePathCounter = 0;

        //we say k = 1 because we don't have the extra edge to root
        for (int k = 0; k < state.k && !H.isEmpty(); k++) {

            //direct harvest ==================================================
            URIEdgeDelta r_c1 = null;
            //while(!H.isEmpty()) {

            logger.debug("H.remove() from {} sized heap...", H.size());
            //pop node r.c1 from top of H
            r_c1 = H.remove();//k=0 init-FM

            logHeap(H);

            //harvest
            List<URIEdge> sidetracks = r_c1.getSidetracks();
            sidetracks.remove(state.init.getEdge());//we need the init edge not for building shortest path
            Sidetracks sts = new Sidetracks(sidetracks);
            logger.debug("\tsidetracks: " + sidetracks);

            //special triple duplication constraint ========================
            //Path path = buildPath(state.srcURI, state.dstURI, sts, state.T);
            //if(path != null) {
            sidetrackResult.add(sts);
            //    result.add(path);
            //    logger.debug("\tvalid path: {}", path.toString());
            //    break;
            //} else {
            //    logger.warn("\tduplicate detected, draw again from H");
            //    state.dublicatePathCounter++;
            //}
            //}

            //we draw until H was empty so we break
            //if(r_c1 == null) {
            //    logger.warn("H is empty, thus break");
            //    break;
            //}
            //direct harvest ==================================================
            //construction =====================================================
            //we have to add for r_c1 to can do P_G.outgoingEdgesOf(r_c1)
            //were the cur points to
            String head = r_c1.getEdge().getTarget();
            //the atoms we need for P_G
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> combined = combineLazy(head, state); //state.v2merged_graph.get(head);
            //if combined isnot empty there is a cross edge from r_c1
            if (!combined.vertexSet().isEmpty()) {
                //cross edge possible

                //clone combined to P_G
                CloneResult g_cloneres = copyClone(combined, P_G);
                //make the cross edge and set weight
                DefaultWeightedEdge e = P_G.addEdge(r_c1, g_cloneres.root);
                ((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) P_G).setEdgeWeight(e, g_cloneres.root.getDelta());
            }
            logger.warn("cross edge added for {}", URIEdge.cutURI(head));
            //construction =====================================================

            //bfs ==============================================================
            //for j = each edge from r -> s with cost c2
            Set<DefaultWeightedEdge> es = P_G.outgoingEdgesOf(r_c1);
            for (DefaultWeightedEdge e : es) {

                //the nodes to outgoing edges
                URIEdgeDelta s = P_G.getEdgeTarget(e);

                //(c1+c2)?
                s.setDelta(r_c1.getDelta() + P_G.getEdgeWeight(e));

                if (s.getParent() == null) {
                    s.setParent(r_c1);
                } else {
                    //we don't want to have loops!
                    logger.warn("waring: s has already parent {}, but should set parent {}", s.getParent(), r_c1);
                    throw new IOException("s has already parent");
                }

                //we decide here if this is a possible path
                List<URIEdge> ssts = s.getSidetracks();
                ssts.remove(state.init.getEdge());//we need the init edge not for building shortest path
                Sidetracks sSidetracks = new Sidetracks(ssts);
                //s.setValid(validPath(sSidetracks, state));//s will in the future be a valid path

                if (s.isValid()) {
                    //so H will consist of possible valid pathes
                    H.add(s);
                }
            }
            //bfs ==============================================================

            logHeap(H);

            //clean up P_G because we will never need it
            P_G.removeVertex(r_c1);

            logger.warn("P(G) has {} vertices and {} edges", P_G.vertexSet().size(), P_G.edgeSet().size());
        }

        //logger.debug("we had {} dublicate triple pathes", state.dublicatePathCounter);
        return sidetrackResult;
    }

    @Deprecated
    private List<Sidetracks> constructiveBFSValidHeapAllLazy(State state) throws IOException {

        List<Sidetracks> sidetrackResult = new LinkedList<>();

        //P(G) however only a part of it
        DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> P_G = new DefaultDirectedWeightedGraph<>(DefaultWeightedEdge.class);

        //try with a comparator
        URIEdgeDeltaComparator c = new URIEdgeDeltaComparator();
        //the heap for bfs
        Queue<URIEdgeDelta> H = new PriorityQueue<>(c);

        //initial shortest
        H.add(state.init);
        P_G.addVertex(state.init);

        for (int k = 0; k < state.k && !H.isEmpty(); k++) {

            //direct harvest ==================================================
            logger.debug("H.remove() from {} sized heap...", H.size());
            URIEdgeDelta r_c1 = H.remove();//k=0 init-FM
            logHeap(H);
            //harvest
            List<URIEdge> sidetracks = r_c1.getSidetracks();
            sidetracks.remove(state.init.getEdge());//we need the init edge not for building shortest path
            Sidetracks sts = new Sidetracks(sidetracks);
            sidetrackResult.add(sts);
            logger.debug("\tsidetracks: " + sidetracks);
            //direct harvest ==================================================

            //construction =====================================================
            String tail = r_c1.getEdge().getSource();
            String head = r_c1.getEdge().getTarget();

            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_out_g = state.v2H_out_graph.get(tail);
            DirectedGraph<URIEdgeDelta, DefaultWeightedEdge> H_T_g = state.v2H_T_graph.get(head);

            if (!r_c1.getOrigin().equals("H_out") && H_out_g != null && !H_out_g.vertexSet().isEmpty()) {
                CloneResult H_out_cloneres = copyClone(H_out_g, P_G);
                setOwner(H_out_cloneres.clone, tail);
                P_G.addEdge(r_c1, H_out_cloneres.root);
                recordIncrementalCosts((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) H_out_cloneres.clone);
            }
            if (H_T_g != null && !H_T_g.vertexSet().isEmpty()) {
                CloneResult H_T_cloneres = copyClone(H_T_g, P_G);
                recordIncrementalCosts((DefaultDirectedWeightedGraph<URIEdgeDelta, DefaultWeightedEdge>) H_T_cloneres.clone);
                //cross edge
                P_G.addEdge(r_c1, H_T_cloneres.root);
            }
            writeDot(String.format("P_G_%03d", k), P_G);
            //construction =====================================================

            //bfs ==============================================================
            //for j = each edge from r -> s with cost c2
            Set<DefaultWeightedEdge> es = P_G.outgoingEdgesOf(r_c1);
            for (DefaultWeightedEdge e : es) {

                //the nodes to outgoing edges
                URIEdgeDelta s = P_G.getEdgeTarget(e);

                //(c1+c2)?
                s.setDelta(r_c1.getDelta() + P_G.getEdgeWeight(e));

                if (s.getParent() == null) {
                    s.setParent(r_c1);
                } else {
                    //we don't want to have loops!
                    logger.warn("waring: s has already parent {}, but should set parent {}", s.getParent(), r_c1);
                    throw new IOException("s has already parent");
                }

                //we decide here if this is a possible path
                List<URIEdge> ssts = s.getSidetracks();
                ssts.remove(state.init.getEdge());//we need the init edge not for building shortest path
                Sidetracks sSidetracks = new Sidetracks(ssts);
                //s.setValid(validPath(sSidetracks, state));//s will in the future be a valid path

                if (s.isValid()) {
                    //so H will consist of possible valid pathes
                    H.add(s);
                }
            }
            //bfs ==============================================================

            logHeap(H);

            //clean up P_G because we will never need it
            P_G.removeVertex(r_c1);

            logger.warn("P(G) has {} vertices and {} edges", P_G.vertexSet().size(), P_G.edgeSet().size());
        }

        //logger.debug("we had {} dublicate triple pathes", state.dublicatePathCounter);
        return sidetrackResult;
    }

    @Deprecated
    private void calcDelta(State state) {
        logger.debug("calc deltas for " + state.T.edgeSet().size() + " edges...");
        for (URIEdge e : state.T.edgeSet()) {

            //tail -> head
            String tail = e.getSource();
            String head = e.getTarget();

            //l(e) always 1 (in unweighted)
            double l_e = state.G.getEdgeWeight(e);
            Double d_head_e_t = state.T.containsInDistance(head) ? state.T.getDistance(head) : Double.POSITIVE_INFINITY;
            Double d_tail_e_t = state.T.containsInDistance(tail) ? state.T.getDistance(tail) : Double.POSITIVE_INFINITY;

            //delta(e) = l(e) + d(head(e), t) - d(tail(e), t)
            double delta_e = l_e + (d_head_e_t - d_tail_e_t);

            state.delta.put(e, delta_e);

            //if is a sidetrack
            if (state.GminusT.containsEdge(e)) {
                //if it is not already in the map: add it
                //or
                //if it is in the map and if the current value is smaller then override it
                if (!state.v2minSidetrack.containsKey(e.getSource()) || delta_e < state.v2minSidetrack.get(e.getSource()).getDelta()) {
                    state.v2minSidetrack.put(e.getSource(), new URIEdgeDelta("", e, delta_e));
                }
            }
        }

        //consistency check (works - OK)
        /*
        Set<String> vs = new HashSet<>();
        for(URIEdge e : state.T.edgeSet()) {
            vs.add(e.getSource());
            vs.add(e.getTarget());
        }
        boolean eq = vs.equals(state.T.vertexSet());
         */
    }

    @Deprecated
    private void calcGminusT(State state) throws IOException {
        //logger.debug("calc G - T edges...");
        Set<URIEdge> edgesInT = state.T.edgeSetPrevious();
        Set<URIEdge> edgesInGMinusT = new HashSet<>(state.G.edgeSet());
        //G - T
        edgesInGMinusT.removeAll(edgesInT);

        //sidetrack graph
        //logger.debug("build GminusT = G - T ...");
        state.GminusT = new DirectedWeightedPseudograph<>(URIEdge.class);
        for (URIEdge e : edgesInGMinusT) {
            state.GminusT.addVertex(e.getSource());
            state.GminusT.addVertex(e.getTarget());
            state.GminusT.addEdge(e.getSource(), e.getTarget(), e);
        }
        logger.debug("G-T with {} vertices and {} edges", state.GminusT.vertexSet().size(), state.GminusT.edgeSet().size());

        writeDotStringURIEdge("00_GminusT", state.GminusT);
    }

}
