package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Queue;
import java.util.Set;
import static java.util.stream.Collectors.toList;
import static java.util.stream.Collectors.toSet;
import org.apache.jena.rdf.model.ResourceFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class ExtendedEppstein extends JGraphTFinderBaseURI {

    private static final Logger logger = LoggerFactory.getLogger(ExtendedEppstein.class);
    
    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {

        DistanceData dd = distances(dstURI);

        return bfs(srcURI, dstURI, k, dd);
    }

    //eppstein bfs
    private List<Path> bfs(String srcURI, String dstURI, int k, DistanceData dd) {
        Queue<VertexDistance> next = new LinkedList<>();
        next.add(new VertexDistance(srcURI, 0));

        List<Path> found = new LinkedList<>();

        Set<String> visited = new HashSet<>();
        
        int iter = 0;

        //BFS
        while (!next.isEmpty() && found.size() < k) {

            long begin = System.currentTimeMillis();

            //do the stuff here ================================================
            //BFS(src) :
            //collect next vertices in queue (with distance)
            //end if we found k shortest paths
            //sidetracks are a set of all possible sidetracks (got by calc SPT)
            //List<URIEdge> possibleSidetracks = new LinkedList<>();
            //take the sidetracks st where src(st) in next
            for (URIEdge sidetrack : dd.sidetracks) { //very big: improvement Map<Src of Edge, List<Edge>>
//                boolean sidetrackFound = next.stream().map(vd -> vd.v).anyMatch(v -> v.equals(sidetrack.getSource()));
//                
//                if(sidetrackFound)
//                    possibleSidetracks.add(sidetrack);

                for (VertexDistance vd : next) {
                    if (vd.v.equals(sidetrack.getSource())) {

                        //s = src(st)
                        //o = trg(st)
                        //calc for each st: d(SDG(s)) + 1 + d(SPT(o))
                        int fulldist = vd.d + 1 + dd.src2shortestEdge.get(sidetrack.getTarget()).distance;
                        
                        logger.debug("{} {}", fulldist, sidetrack);
                        
                        Path p = buildPath(sidetrack, dd);
                            
                        found.add(p);
                    }
                }

            }

            //add path to found sorted by full distance
            
            
            //do the stuff here end ============================================
            //now we do a next iteration
            VertexDistance cur = next.poll();

            if(visited.contains(cur.v))
                continue;
            
            visited.add(cur.v);
            
            //now i should go sidetracks
            //however first the sidetracks to a node where the distance to the target is low
            //Set<URIEdge> sidetracks = g.outgoingEdgesOf(cur.v).stream().filter(e -> dd.sidetracks.contains(e)).collect(toSet());
            for (URIEdge e : g.outgoingEdgesOf(cur.v)) {
                //int index = dd.sortedSidetrackEdges.indexOf(e);

                //VertexDistance to = new VertexDistance(e.getTarget(), cur.d + 1);
                //int fulldist = to.d + dd.map.get(to.v).distance;
                //TODO add full path to output
                //logger.debug("to: {}", to);
                next.add(new VertexDistance(e.getTarget(), cur.d + 1));
            }

            long end = System.currentTimeMillis();
            long duration = end - begin;

            //w.write(String.format("%d, %d, %d, %d, %d, %d, %s%n", iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), next.size(), curDist.distance, cur));
            //Collections.sort((List<VertexDistance>) next);
            //logger.debug(next.toString());

            iter++;
        }

        for(Path p : found)
            logger.debug(p.toString());
        
        return found;
    }

    private Path buildPath(URIEdge sidetrack, DistanceData dd) {
        
        List<URIEdge> es = new LinkedList<>();
        es.add(sidetrack);
        
        String cur = sidetrack.getTarget();
        while(true) {
            URIEdgeDistance ued = dd.src2shortestEdge.get(cur);
            if(ued.edge == null)
                break;
            es.add(ued.edge);
            cur = ued.edge.getTarget();
        }
        
        cur = sidetrack.getSource();
        while(true) {
            URIEdgeDistance ued = dd.dst2shortestEdge.get(cur);
            if(ued == null)
                break;
            es.add(0, ued.edge);
            cur = ued.edge.getSource();
        }
        
        return translatePath(es);
    }
    
    protected Path translatePath(List<URIEdge> edges) {

        Path p = new Path();

        if(!edges.isEmpty())
            p.add(ResourceFactory.createResource(edges.get(0).getSource()));
        
        for (URIEdge e : edges) {
            p.add(ResourceFactory.createProperty(e.getEdge()));
            p.add(ResourceFactory.createResource(e.getTarget()));
        }

        return p;
    }

    
    private DistanceData distances(String dstURI) throws IOException {
        BufferedWriter w = new BufferedWriter(new FileWriter(getIndexFile(1)));

        //header
        //iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), next.size(), cur, curDist
        w.write(String.format("iteration, duration (ms), used memory (mb), incoming degree, next size, current distance, current node%n"));

        Queue<String> next = new LinkedList<>();
        next.add(dstURI);

        Map<String, URIEdgeDistance> src2shortestedge = new HashMap<>();
        src2shortestedge.put(dstURI, new URIEdgeDistance(null, 0));

        Map<String, URIEdgeDistance> dst2shortestedge = new HashMap<>();
        
        int iter = 0;

        //bfs from dst
        while (!next.isEmpty()) {

            long begin = System.currentTimeMillis();

            String cur = next.poll();
            URIEdgeDistance curDist = src2shortestedge.get(cur);

            Set<URIEdge> es = g.incomingEdgesOf(cur);

            for (URIEdge e : es) {

                if (!src2shortestedge.containsKey(e.getSource())) {
                    //not seen, so we add
                    src2shortestedge.put(e.getSource(), new URIEdgeDistance(e, curDist.distance + 1));

                    //many edges problem
                    //dst2shortestedge.put(e.getTarget(), new URIEdgeDistance(e, curDist.distance + 1));
                    
                    //next to search
                    next.add(e.getSource());
                }
            }

            long end = System.currentTimeMillis();
            long duration = end - begin;

            w.write(String.format("%d, %d, %d, %d, %d, %d, %s%n", iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), next.size(), curDist.distance, cur));
        }

        //Set<String> optimalEdges = best.values().stream().map(e -> e.getEdge()).collect(toSet());
        Set<URIEdge> bestTriples = src2shortestedge.values().stream().map(e -> e.edge).collect(toSet());
        Set<URIEdge> sidetracks = new HashSet<>(g.edgeSet());
        sidetracks.removeAll(bestTriples);

        List<URIEdgeDistance> sidetrackDists = new LinkedList<>();
        for (URIEdge sidetrack : sidetracks) {
            sidetrackDists.add(new URIEdgeDistance(sidetrack, src2shortestedge.get(sidetrack.getTarget()).distance));
        }

        List<URIEdgeDistance> sortedSidetrackDists = sidetrackDists.stream().sorted(new Comparator<URIEdgeDistance>() {
            @Override
            public int compare(URIEdgeDistance a, URIEdgeDistance b) {
                return Integer.compare(a.distance, b.distance);
            }
        }).collect(toList());

        List<URIEdge> sortedSidetrackEdges = sidetrackDists.stream().sorted(new Comparator<URIEdgeDistance>() {
            @Override
            public int compare(URIEdgeDistance a, URIEdgeDistance b) {
                return Integer.compare(a.distance, b.distance);
            }
        }).map(sd -> sd.edge).collect(toList());

        for (URIEdgeDistance a : sortedSidetrackDists) {
            logger.debug(a.toString());
        }

        logger.debug("count best unique triple: {}", bestTriples.size());
        logger.debug("count sidetrack triple: {}", sidetracks.size());
        logger.debug("count all unique triple: {}", g.edgeSet().size());
        logger.debug("ratio: {}", bestTriples.size() / (float) g.edgeSet().size());

        w.close();

        DistanceData dd = new DistanceData();
        dd.src2shortestEdge = src2shortestedge;
        dd.dst2shortestEdge = dst2shortestedge;
        dd.bestTriples = bestTriples;
        dd.sidetracks = sidetracks;
        dd.sortedSidetrackDists = sortedSidetrackDists;
        dd.sortedSidetrackEdges = sortedSidetrackEdges;

        return dd;
    }

    private class URIEdgeDistance {

        public URIEdge edge;
        public int distance;

        public URIEdgeDistance(URIEdge edge, int distance) {
            this.edge = edge;
            this.distance = distance;
        }

        @Override
        public String toString() {
            if (edge == null) {
                return "0";
            }

            return distance + ": " + edge;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 37 * hash + Objects.hashCode(this.edge);
            return hash;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final URIEdgeDistance other = (URIEdgeDistance) obj;
            if (!Objects.equals(this.edge, other.edge)) {
                return false;
            }
            return true;
        }

    }

    private class DistanceData {

        public Set<URIEdge> bestTriples;
        public Set<URIEdge> sidetracks;
        public Map<String, URIEdgeDistance> src2shortestEdge;
        public Map<String, URIEdgeDistance> dst2shortestEdge;
        public List<URIEdgeDistance> sortedSidetrackDists;
        public List<URIEdge> sortedSidetrackEdges;
    }

    private class VertexDistance implements Comparable<VertexDistance> {

        public String v;
        /**
         * Walked distance.
         */
        public int d;

        public VertexDistance(String v, int d) {
            this.v = v;
            this.d = d;
        }

        @Override
        public int compareTo(VertexDistance o) {
            return Integer.compare(this.d, o.d);
        }

        @Override
        public String toString() {
            return "VertexDistance{" + "v=" + v + ", d=" + d + '}';
        }

    }

}
