package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;
import java.util.Queue;
import java.util.Set;
import org.apache.jena.rdf.model.Property;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.rdf.model.Statement;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class MarkusV2 extends JGraphTFinderBaseURI {

    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        return translatePath(findKShortestPath(srcURI, dstURI, k, startOrEndProp));
    }

    protected List<StmtPath> findKShortestPath(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        return findKShortestPath(srcURI, dstURI, k, startOrEndProp, new HashSet<>());
    }
    
    protected List<StmtPath> findKShortestPath(String srcURI, String dstURI, int k, String startOrEndProp, Set<Statement> disallow) throws Exception {
        Resource dst = ResourceFactory.createResource(dstURI);
        
        //data storage
        Queue<StmtPath> next = new LinkedList<>();
        List<StmtPath> found = new LinkedList<>();

        //fist to get the initial pathes
        //initial
        Set<URIEdge> es = g.outgoingEdgesOf(srcURI);
        for (URIEdge e : es) {
            Statement stmt = toStatement(e);
            
            //special case for yen
            if(disallow.contains(stmt))
                continue;
            
            StmtPath sp = new StmtPath(stmt);
            next.add(sp);
        }
        
        //k = 1, O(V + E)

        //int iter = 0;
        //BFS
        //for each path we go
        while (!next.isEmpty() && found.size() < k) {

            //the path we go currently
            StmtPath cur = next.poll();

            //the end of the path
            //object of last triple
            Resource last = cur.getLast();

            //if the path ends with the end vertex
            //we are finished
            if (last.equals(dst)) {
                //we add to found
                //and don't search further
                found.add(cur);
                //continue; //search further
            }
            
            //we look at the branches
            Set<URIEdge> outs = g.outgoingEdgesOf(last.toString());
            for (URIEdge e : outs) {

                Statement stmt = toStatement(e);
                
                //special case for yen
                if(disallow.contains(stmt))
                    continue;
                    
                //no duplicate triples in our current path
                //if we have a duplicate we can not walk this triple
                if (!cur.valid(stmt)) {
                    //if for all is continue then the path is dead
                    continue;
                }

                //concat the current with the new statement
                StmtPath newPath = new StmtPath(cur, stmt);

                next.add(newPath);
            }

            //sort that smaller ones go to top
            //Collections.sort(next);
            //print to see correct result in console
            //print
            //System.out.println("Iteration: " + iter++);
            //for (StmtPath sp : found) {
            //    System.out.println(sp.toString());
            //}
            //System.out.println("----------------------------------------------");
        }

        //sort result
        Collections.sort(found);//StmtPath is Compareable
        //TODO: why sort?

        //print
        //System.out.println("After sort");
        //for (StmtPath sp : found) {
        //    System.out.println(sp.toString());
        //}
        //System.out.println("----------------------------------------------");

        return found;
    }
    
    
    protected List<Path> translatePath(List<StmtPath> stmtpathes) {

        List<Path> ret = new LinkedList<>();

        for (StmtPath path : stmtpathes) {
            Path newPath = new Path();
            List<Statement> p = path.path;

            if (p.size() > 0) {
                newPath.add(p.get(0).getSubject());
            }

            for (Statement e : p) {
                newPath.add(e.getPredicate());
                newPath.add(e.getObject().asResource());
            }
            ret.add(newPath);
        }

        return ret;
    }

    protected Statement toStatement(URIEdge ue) {
        Resource s = ResourceFactory.createResource(ue.getSource());
        Resource o = ResourceFactory.createResource(ue.getTarget());
        Property p = ResourceFactory.createProperty(ue.getEdge());
        return ResourceFactory.createStatement(s, p, o);
    }
    
    protected URIEdge toURIEdge(Statement stmt) {
        return new URIEdge(stmt.getSubject().getURI(), stmt.getPredicate().getURI(), stmt.getObject().asResource().getURI());
    }

    protected class StmtPath implements Comparable<StmtPath> {

        public Set<Statement> set;
        public List<Statement> path;

        
        
        public StmtPath() {
            this.set = new HashSet<>();
            this.path = new LinkedList<>();
        }

        public StmtPath(Statement stmt) {
            this.set = new HashSet<>();
            this.path = new LinkedList<>();
            set.add(stmt);
            path.add(stmt);
        }

        public StmtPath(StmtPath path, Statement newStmt) {
            this.set = new HashSet<>(path.set);
            this.path = new LinkedList<>(path.path);
            this.set.add(newStmt);
            this.path.add(newStmt);
        }

        public boolean valid(Statement newStmt) {
            return !set.contains(newStmt);
        }

        public Resource getLast() {
            if (path.isEmpty()) {
                return null;
            }
            //if (path.size() == 1) {
            //    return path.get(0).getSubject();
            //}
            return path.get(path.size() - 1).getObject().asResource();
        }

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();

            sb.append("(");
            sb.append(path.size());
            sb.append(") ");

            for (Statement stmt : path) {
                sb.append(stmt.toString());
            }

            return sb.toString();
        }

        @Override
        public int compareTo(StmtPath other) {
            if (this.path.size() > other.path.size()) {
                return 1;
            }
            if (this.path.size() < other.path.size()) {
                return -1;
            }

            //first comes: P
            String firstPred1 = this.path.get(0).getPredicate().toString();
            String firstPred2 = other.path.get(0).getPredicate().toString();
            return firstPred1.compareTo(firstPred2);
        }

        public Resource getNode(int index) {
            if(index == path.size()) {
                return getLast();
            }
            return path.get(index).getSubject();
        }
        
        public StmtPath subPath(int begin, int end) {
            StmtPath sp = new StmtPath();
            List<Statement> l = this.path.subList(begin, end);
            sp.path.addAll(l);
            sp.set.addAll(l);
            return sp;
        }
        
        public List<Resource> toNodeList() {
            List<Resource> res = new LinkedList<>();
            
            if (path.size() > 0) {
                res.add(path.get(0).getSubject());
            }
            
            for(Statement stmt : path) {
                res.add(stmt.getObject().asResource());
            }
            return res;
        }

        @Override
        public int hashCode() {
            int hash = 3;
            hash = 19 * hash + Objects.hashCode(this.path);
            return hash;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final StmtPath other = (StmtPath) obj;
            if (!Objects.equals(this.path, other.path)) {
                return false;
            }
            return true;
        }
        
        
    }

}
