package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.Statement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class MarkusV3 extends MarkusV2 {
    
    private static final Logger logger = LoggerFactory.getLogger(MarkusV3.class);
    
    @Override
    public List<Path> find(String srcURI, String dstURI, int K, String startOrEndProp) throws Exception {
        
        //https://en.wikipedia.org/wiki/Yen's_algorithm
        
        //A[0] = Dijkstra(Graph, source, sink);
        StmtPath[] A = new StmtPath[K];
        A[0] = findKShortestPath(srcURI, dstURI, 1, startOrEndProp).get(0);//TODO
        //shortest path found
        
        //B = [];
        List<StmtPath> B = new LinkedList<>();
        
        //to exclude edges
        Set<Statement> disallow = new HashSet<>();
        
        //for k from 1 to K:
        for(int k = 1; k < K; k++) {
            logger.debug("k: {}", k);
            logger.debug("K: {}", K);
            
            //The spur node ranges from the first node to the next to last node in the previous k-shortest path.
                //(a,b,c) (c,d,e) = [a, c, e] -> size([a, c, e]) = 3 -> i = 0,1,2
            // for i from 0 to size(A[k − 1]) − 1:
            int I = A[k - 1].toNodeList().size() + 1; //+1 because of i < I. 2 pathes = 3 node.
            for(int i = 0; i < I; i++) {
            
                logger.debug("i: {}", i);
                logger.debug("I: {}", I);
                
                //spurNode = A[k-1].node(i);
                Resource spurNode = A[k-1].getNode(i);
                logger.debug("spurNode: {}", spurNode);
                //(a,b,c) (c,d,e) = [a, c, e]
                
                //rootPath = A[k-1].nodes(0, i);
                //The sequence of nodes from the source to the spur node of the previous k-shortest path.
                StmtPath rootPath = A[k-1].subPath(0, i + 1);
                //List<Resource> rootPath = A[k-1].toNodeList().subList(0, i + 1);
                logger.debug("rootPath: {}", rootPath);
                
                //for each path p in A:
                for(StmtPath p : A) {
                    
                    //because we have an array
                    if(p == null)
                        break;//rest will be null
                    
                    //Remove the links that are part of the previous shortest paths which share the same root path.
                    
                    StmtPath x = p.subPath(0, i + 1);
                    logger.debug("p.subPath(0, i + 1): {}", x);
                    
                    //if rootPath == p.nodes(0, i):
                    if(rootPath.equals(x)) {
                        //remove p.edge(i, i + 1) from Graph;
                        disallow.add(p.subPath(i, i + 1).path.get(0)); //(i,i) is always empty
                    }
                }
                logger.debug("disallow: {}", disallow);
                
                
                //for each node rootPathNode in rootPath except spurNode:
                //for(Resource r : rootPathNode)
                    //remove rootPathNode from Graph;
                    
                logger.debug("find {} -> {}", spurNode.getURI(), dstURI);
                    
                //spurPath = Dijkstra(Graph, spurNode, sink);
                List<StmtPath> spurPaths = findKShortestPath(spurNode.getURI(), dstURI, 1, startOrEndProp, disallow);
                if(spurPaths.isEmpty()) {
                    logger.debug("spurPaths.isEmpty()");
                    break;
                }
                StmtPath spurPath = spurPaths.get(0);
                logger.debug("spurPath: {}", spurPath);
                
                //totalPath = rootPath + spurPath;
                StmtPath totalPath = new StmtPath();
                //totalPath.path.addAll(rootPath.path); //needs concat for statements
                totalPath.path.addAll(spurPath.path);
                totalPath.set.addAll(totalPath.path);
                logger.debug("totalPath: {}", totalPath);
                
                //B.append(totalPath);
                B.add(totalPath);
                
                //restore edges to Graph;
                //restore nodes in rootPath to Graph;
                disallow.clear();
                
                logger.debug(" ");
                
            }//for path size
            logger.debug("end i");
            
            //if B is empty:
            if(B.isEmpty()) {
                logger.debug("B is empty break");
                //break;
                break;
            }
            
            //B.sort();
            Collections.sort(B);
            
            //A[k] = B[0];
            A[k] = B.get(0);
            logger.debug("A: {}", Arrays.toString(A));
            
            //B.pop();
            B.remove(0);
            
            logger.debug("B: {}", B);
            
            logger.debug(" ");
        }//for k
        
        logger.debug("end k");
        
        return translatePath(Arrays.asList(A));
    }
    
}
