package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class URIEdgeDelta implements Comparable<URIEdgeDelta> {
    
    private String owner;
    private URIEdge edge;
    private double delta;
    private URIEdgeDelta parent;
    private String origin;
    private boolean valid = true;
    private InvalidType invalidType = InvalidType.Unkown;
    
    
    //filled by setParent
    private List<URIEdge> sidetracks;
    private boolean pruneable = false;
    
    public URIEdgeDelta(String owner, URIEdge edge, double delta) {
        this.edge = edge;
        this.delta = delta;
        this.owner = owner;
        this.origin = "H_T";//comes default from H_T(v)
        this.sidetracks = new LinkedList<>();
        //this.myID = id++;
    }
    
    /**
     * Copy.
     * @param d 
     */
    public URIEdgeDelta(URIEdgeDelta d) {
        this.edge = d.edge;
        this.delta = d.delta;
        this.owner = d.owner;
        this.origin = d.origin;
        this.valid = d.valid;
        this.sidetracks = d.sidetracks;
        this.invalidType = d.invalidType;
        if(d.parent != null)
            this.parent = new URIEdgeDelta(d.parent);
    }

    //==========================================================================
    //to get the sidetracks on the way to root
    
    public URIEdgeDelta getParent() {
        return parent;
    }

    public void setParent(URIEdgeDelta parent) {
        this.parent = parent;
        
        /* if we do it every parent set time
        if(!this.owner.equals(parent.owner)) {
            sidetracks.add(parent.getEdge());
            
            Set<URIEdge> s = new HashSet<>(sidetracks);
            pruneable = s.size() < sidetracks.size();//bc then duplicate sidetrack
        }*/
    }
    
    public List<URIEdgeDelta> getParents() {
        List<URIEdgeDelta> parents = new LinkedList<>();
        URIEdgeDelta cur = this;
        URIEdgeDelta parent;
        while((parent = cur.getParent()) != null) {
            parents.add(parent);
            cur = parent;
        }
        Collections.reverse(parents);
        return parents;
    }
    
    @Deprecated
    public List<URIEdgeDelta> getParentsOtherOwner() {
        List<URIEdgeDelta> parents = new LinkedList<>();
        URIEdgeDelta cur = this;
        URIEdgeDelta parent;
        while((parent = cur.getParent()) != null) {
            
            //means: cross edge
            if(!cur.owner.equals(parent.owner))
                parents.add(parent);
            
            cur = parent;
        }
        Collections.reverse(parents);
        return parents;
    }

    @Deprecated
    public List<URIEdgeDelta> getSidetracksDelta() {
        List<URIEdgeDelta> st = new LinkedList<>();
        for(URIEdgeDelta e : getParentsOtherOwner()) {
            st.add(e);
        }
        st.add(this);
        return st;
    }
    
    /* too slow
    public List<URIEdge> getSidetracks() {
        List<URIEdge> st = new LinkedList<>();
        for(URIEdgeDelta e : getParentsOtherOwner()) {
            st.add(e.getEdge());
        }
        st.add(this.getEdge());
        return st;
    }
    */
    
    public List<URIEdge> getSidetracksByParents() {
        LinkedList<URIEdge> parents = new LinkedList<>();
        parents.add(this.getEdge());
        URIEdgeDelta cur = this;
        URIEdgeDelta parent;
        while((parent = cur.getParent()) != null) {
            
            //means: cross edge
            if(!cur.owner.equals(parent.owner))
                parents.addFirst(parent.getEdge());
            
            cur = parent;
        }
        //Collections.reverse(parents);
        return parents;
    }
    
    public List<URIEdge> getSidetracks() {
        //List<URIEdge> sts = new LinkedList<>(sidetracks);
        //sts.add(this.getEdge());
        //return sts;
        return getSidetracksByParents();
    }

    public boolean isPruneable() {
        return pruneable;
    }
    
    /* too slow
    public List<URIEdge> getSidetracks() {
        //skips init edge
        return getSidetracksDelta().stream().map(ed -> ed.edge).collect(Collectors.toList());
    }
    */
    
    //==========================================================================
    //getter setter
    
    public void setDelta(double delta) {
        this.delta = delta;
    }
    
    public double getDelta() {
        return delta;
    }
    
    public URIEdge getEdge() {
        return edge;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public String getOrigin() {
        return origin;
    }

    public void setOrigin(String origin) {
        this.origin = origin;
    }

    public boolean isValid() {
        return getInvalidType() == InvalidType.None;
    }

    public InvalidType getInvalidType() {
        return invalidType;
    }

    public void setInvalidType(InvalidType invalidType) {
        this.invalidType = invalidType;
    }
    
    
    
    //==========================================================================
    //compare by delta
    //to string
    //equals on owner, origin, edge
    
    @Override
    public int compareTo(URIEdgeDelta other) {
        return Double.compare(this.delta, other.delta);
    }

    public String toStringOld() {
        try {
            return "URIEdgeDelta{" + "owner=" + owner + ", edge=" + edge + ", delta=" + delta + ", parent=" + parent + '}';
        } catch(StackOverflowError e) {
            return "URIEdgeDelta{" + "owner=" + owner + ", edge=" + edge + ", delta=" + delta + "}";
        }
    }

    @Override
    public String toString() {
        try {
            return "URIEdgeDelta{" + "owner=" + URIEdge.cutURI(owner) + ", edge=" + edge + ", delta=" + delta + ", invalid=" + invalidType + ", parent=" + parent + ", origin=" + origin + '}';
        } catch(StackOverflowError e) {
            return "URIEdgeDelta{" + "owner=" + URIEdge.cutURI(owner) + ", edge=" + edge + ", delta=" + delta + ", origin=" + origin + ", invalid=" + invalidType + "}";
        }
    }

    public boolean isEqualTo(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final URIEdgeDelta other = (URIEdgeDelta) obj;
        if (!Objects.equals(this.owner, other.owner)) {
            return false;
        }
        if (!Objects.equals(this.origin, other.origin)) {
            return false;
        }
        if (!Objects.equals(this.edge, other.edge)) {
            return false;
        }
        return true;
    }


}
