package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base;

import java.io.Serializable;
import org.jgrapht.graph.DefaultWeightedEdge;

public class URIEdge extends DefaultWeightedEdge implements Serializable  {

    /**
     *
     */
    private static final long serialVersionUID = -7403871154095205358L;

    private String source;
    private String edge;
    private String target;

    public URIEdge(String edge, String source, String target) {
        super();
        this.edge = edge;
        this.source = source;
        this.target = target;
    }

    public String getEdge() {
        return edge;
    }

    public void setEdge(String edge) {
        this.edge = edge;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getTarget() {
        return target;
    }

    public void setTarget(String target) {
        this.target = target;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((edge == null) ? 0 : edge.hashCode());
        result = prime * result + ((source == null) ? 0 : source.hashCode());
        result = prime * result + ((target == null) ? 0 : target.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        URIEdge other = (URIEdge) obj;
        if (edge == null) {
            if (other.edge != null) {
                return false;
            }
        } else if (!edge.equals(other.edge)) {
            return false;
        }
        if (source == null) {
            if (other.source != null) {
                return false;
            }
        } else if (!source.equals(other.source)) {
            return false;
        }
        if (target == null) {
            if (other.target != null) {
                return false;
            }
        } else if (!target.equals(other.target)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return cutURI(source) + "|" + cutURI2(edge) + "|" + cutURI(target);
    }

    public static String cutURI(String uri) {
        if(uri.contains("/")) {
            return uri.substring(uri.lastIndexOf("/") + 1, uri.length());
        }
        return uri;
    }
    
    public static String cutURI2(String uri) {
        int i1 = uri.lastIndexOf('/');
        if(i1 == -1)
            return uri;
        
        int i2 = uri.lastIndexOf('/', i1 - 1);
        if(i2 == -1)
            return uri;
        
        String a = uri.substring(i2 + 1, i1);
        String b = uri.substring(i1 + 1, uri.length());
        
        if(a.isEmpty())
            return b;
            
        return a.substring(0, 1) + "/" + b;
    }

    public String getIdentificationString(){    	
    	StringBuilder sb = new StringBuilder();
    	sb.append(source);
    	sb.append("--");
    	sb.append(edge);
    	sb.append("--");
    	sb.append(target);
    	return sb.toString();
    }
    
    public String getEdgeLocalName(){    	
    	return cutURI(edge);
    }
    
}
