package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.eppstein3;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.BinaryHeap;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.URIEdgeDelta;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.JGraphTFinderBaseURI;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.DijkstraDistance;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.IShortestPathTree;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree.ShortestPathTree;
import de.dfki.km.academicresearch.eswc2016challenge.util.EppsteinDataset2;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Set;
import java.util.Stack;

import static java.util.stream.Collectors.toList;
import org.apache.jena.rdf.model.ResourceFactory;
import org.jgrapht.DirectedGraph;
import org.jgrapht.WeightedGraph;
import org.jgrapht.ext.DOTExporter;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultDirectedWeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class EppsteinV3 extends JGraphTFinderBaseURI {

    private static final Logger logger = LoggerFactory.getLogger(EppsteinV3.class);
    
    
    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        
        logger.debug("calc shortest path tree");
        IShortestPathTree<String, URIEdge> sptCalculator = new DijkstraDistance<>();
        ShortestPathTree<String, URIEdge> spt = sptCalculator.computeShortestPathTree(g, dstURI);
        logger.debug("calc shortest path tree END");
        
        //buildDeviationHeaps(spt, dstURI);
        //buildOutHeap(spt, srcURI);
    	
        buildOutHeap(spt, EppsteinDataset2.s.getURI()); //t //r
        
    	return null;
        
    }
    
    
    
    private void buildDeviationHeaps(ShortestPathTree<String, URIEdge> spt, String dst){
    	
    	Map<String, BinaryHeap<URIEdgeDelta>> heaps = new HashMap<>();
    	
    	BinaryHeap<URIEdgeDelta> dstheap = buildOutHeap(spt, dst); 
    	heaps.put(dst, dstheap);
    	
    	Stack<String> stack = new Stack<String>();
    	
    	for(String node : this.g.vertexSet()){
    		if(heaps.containsKey(node))
    			continue;
    		
    		//Go through the shortest path and push all node until the heaps are already built
    		for(String s = node; heaps.containsKey(s) == false && spt.getPrevious(s) != null; s = spt.getPrevious(s).getTarget())
    			stack.push(s);    		
    		
    		if(stack.empty() == false){
    			
    			String s = stack.pop();
    			BinaryHeap<URIEdgeDelta> cur = buildOutHeap(spt, s);
    			
    			
    			String nextInT = spt.getPrevious(s).getTarget();
    			if(nextInT != null)
    				heaps.put(s, mergeHeap(cur, heaps.get(nextInT)));//s und nextInT
    			
    			while(!stack.isEmpty()) {
    			      String n = stack.pop();
    			      BinaryHeap<URIEdgeDelta> next = buildOutHeap(spt, n);
    			      heaps.put(n, mergeHeap(next, cur));//s und nextInT
    			      cur = next;
    			}
    		}
    	}
    }
    
    /**
     * Builds a 2-heap of the edges in G-T with tails at the node, with the restriction that the root of the heap only has one child.
     * The weight of an edge e is delta(e).
     * @param spt
     * @param node
     * @return
     */
    private BinaryHeap<URIEdgeDelta> buildOutHeap(ShortestPathTree<String, URIEdge> spt, String node){

    	
    	if(spt.getDistance(node) == Double.POSITIVE_INFINITY)
    		return null;
    	
    	BinaryHeap<URIEdgeDelta> h_hout = new BinaryHeap<>();
    	
    	URIEdge bestedge = spt.getPrevious(node);
    	double nodedistance = spt.getDistance(node);
    	for(URIEdge e : this.g.outgoingEdgesOf(node)){   		
    		
    		if(bestedge != null && bestedge.equals(e) == false){// a sidetrack ! //here is something missing
    			double delta = g.getEdgeWeight(e) + spt.getDistance(e.getTarget()) - nodedistance;
    			h_hout.add(new URIEdgeDelta("", e, delta));
    		}
    	}
    	
    	return h_hout;
    }
    
    
    private BinaryHeap<URIEdgeDelta> mergeHeap(BinaryHeap<URIEdgeDelta> one, BinaryHeap<URIEdgeDelta> two){
    	return null;
    }
    
    
}
