package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree;

import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.BufferedWriter;
import java.io.Writer;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import org.jgrapht.DirectedGraph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class BFSDistance<V, E> extends IShortestPathTree<V, E> {

    private static final Logger logger = LoggerFactory.getLogger(BFSDistance.class);

    @Override
    public ShortestPathTree<V, E> computeShortestPathTree(DirectedGraph<V, E> g, V destination, Writer writer) throws Exception {

        BufferedWriter w = new BufferedWriter(writer);

        //header
        //iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), next.size(), cur, curDist
        w.write(String.format("iteration, duration (ms), used memory (mb), incoming degree, next size, current node, current distance%n"));
        //https://en.wikipedia.org/wiki/Dijkstra%27s_algorithm

        Queue<V> q = new LinkedList<>();
        q.add(destination);

        ShortestPathTree<V, E> spt = new ShortestPathTree<V, E>();

        spt.setDistanceAndPrevious(destination, null, 0.0);
        int iter = 0;

        //bfs from dst
        while (!q.isEmpty()) {

            long begin = System.currentTimeMillis();

            V cur = q.poll();
            double curDist = spt.getDistance(cur);

            //for multi edge
            Map<V, Set<E>> clusters = new HashMap<>();
            
            Set<E> es = g.incomingEdgesOf(cur);
            for (E e : es) {

                V tmp = g.getEdgeSource(e);
                
                if(!clusters.containsKey(tmp)) {
                    clusters.put(tmp, new HashSet<>(Arrays.asList(e)));
                } else {
                    clusters.get(tmp).add(e);
                }
            }

            //because multi edge
            //per vertex the incoming edges
            //one entry has the multi edges to the target
            for(Map.Entry<V, Set<E>> cluster : clusters.entrySet()) {
                
                //look at one vertex
                V tmp = cluster.getKey();
                
                //check if visited
                if (!spt.containsInDistance(tmp)) {
                    
                    //not seen, so we add
                    //use first edge (it doesn't matter for multi edge)
                    spt.setDistance(tmp, curDist + g.getEdgeWeight((E) cluster.getValue().toArray()[0]));
                    
                    //now we can do multi edge
                    for(E e : cluster.getValue())
                        spt.setPrevious(tmp, e);//set is internal an add

                    //next to search
                    q.add(tmp);
                }
            }
            
            long end = System.currentTimeMillis();
            long duration = end - begin;

            w.write(String.format("%d, %d, %d, %d, %d, %s, %f%n", iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), q.size(), cur, curDist));
        }

        //Set<String> optimalEdges = best.values().stream().map(e -> e.getEdge()).collect(toSet());
        Set<E> bestTriples = new HashSet<>(spt.getPrevious().values());

        logger.debug("count best unique triple: {}", bestTriples.size());
        logger.debug("count all unique triple: {}", g.edgeSet().size());
        logger.debug("ratio: {}", bestTriples.size() / (float) g.edgeSet().size());

        w.close();
        return spt;
    }

}
