package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree;

import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.BufferedWriter;
import java.io.Writer;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;
import org.jgrapht.DirectedGraph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class DijkstraDistance<V, E> extends IShortestPathTree<V, E> {

    private static final Logger logger = LoggerFactory.getLogger(DijkstraDistance.class);

    @Override
    public ShortestPathTree<V, E> computeShortestPathTree(DirectedGraph<V, E> g, V destination, Writer writer) throws Exception {

        BufferedWriter w = new BufferedWriter(writer);

        //header
        //iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), next.size(), cur, curDist
        w.write(String.format("iteration, duration (ms), used memory (mb), incoming degree, next size, current node, current distance%n"));
        //https://en.wikipedia.org/wiki/Dijkstra%27s_algorithm

        Queue<V> q = new LinkedList<>();
        q.add(destination);

        ShortestPathTree<V, E> spt = new ShortestPathTree<V, E>();

        spt.setDistanceAndPrevious(destination, null, 0.0);
        int iter = 0;

        //https://en.wikipedia.org/wiki/Dijkstra%27s_algorithm
        while (!q.isEmpty()) {

            long begin = System.currentTimeMillis();

            V cur = q.poll(); //we dont need to use a priority queue , because we have to visit all vertices
            double curDist = spt.getDistance(cur);

            Set<E> es = g.incomingEdgesOf(cur);
            for (E e : es) {
            	spt.addEdge(e);
                V neighbour = g.getEdgeSource(e);
                double alt = curDist + g.getEdgeWeight(e);

                if (!spt.containsInDistance(neighbour)) {
                    spt.setDistanceAndPrevious(neighbour, e, alt);
                    if(cur == destination) {
                        spt.setDestinationEdge(e);
                    }
                    //logger.debug("Set {} to {}", neighbour, alt);
                } else if (alt < spt.getDistance(neighbour)) {
                    spt.setDistanceAndPrevious(neighbour, e, alt);
                    if(cur == destination) {
                        spt.setDestinationEdge(e);
                    }
                    //logger.debug("Reset {} to {}", neighbour, alt);
                } else {
                    continue;//because of loop?
                }

                //next to search
                q.add(neighbour);
            }

            long end = System.currentTimeMillis();
            long duration = end - begin;

            w.write(String.format("%d, %d, %d, %d, %d, %s, %f%n", iter++, duration, PrintHeapUsage.getUsedMB(), es.size(), q.size(), cur, curDist));
        }

        //Set<String> optimalEdges = best.values().stream().map(e -> e.getEdge()).collect(toSet());
        //Set<E> bestTriples = new HashSet<>(spt.getPrevious().values());

        //logger.debug("count best unique triple: {}", bestTriples.size());
        //logger.debug("count all unique triple: {}", g.edgeSet().size());
        //logger.debug("ratio: {}", bestTriples.size() / (float) g.edgeSet().size());

        w.close();
        return spt;
    }

}
