package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.shortestpathtree;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.base.URIEdge;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.jgrapht.graph.DefaultDirectedWeightedGraph;

public class ShortestPathTree<V, E> {

    //TODO: could be improved with int array
    private Map<V, Double> distance;
    private Map<V, E> previousEdge;
    private Set<E> allEdges;

    private Map<V, Set<E>> previousEdges;
    
    private E destinationEdge;

    public ShortestPathTree() {

        this.distance = new HashMap<>();
        this.previousEdge = new HashMap<>();
        this.allEdges = new HashSet<>();
        this.previousEdges = new HashMap<>();
    }

    public void setDistance(V key, Double value) {
        distance.put(key, value);
    }

    //test
    public int multiCount = 0;
    
    public void addEdge(E value) {
    	allEdges.add(value);
    }
    
    public void setPrevious(V key, E value) {
        previousEdge.put(key, value);

        /*
        if (!previousEdges.containsKey(key)) {
            previousEdges.put(key, new HashSet<>(Arrays.asList(value)));
        } else {
            previousEdges.get(key).add(value);
            multiCount++;
        }
        */
    }

    public void setDistanceAndPrevious(V key, E prev, Double value) {
        setDistance(key, value);
        setPrevious(key, prev);
    }

    public E getDestinationEdge() {
        return destinationEdge;
    }

    public void setDestinationEdge(E destinationEdge) {
        this.destinationEdge = destinationEdge;
    }

    public Double getDistance(V key) {
    	return distance.getOrDefault(key, Double.POSITIVE_INFINITY);
        //return distance.get(key);
    }

    public E getPrevious(V key) {
        return previousEdge.get(key);
    }

    public Set<E> getPreviousMulti(V key) {
        return previousEdges.get(key);
    }
    
    public Map<V, Double> getDistance() {
        return distance;
    }

    public Map<V, E> getPrevious() {
        return previousEdge;
    }

    public boolean containsInDistance(V key) {
        return this.distance.containsKey(key);
    }

    public Set<E> edgeSetPrevious() {
        return new HashSet<>(previousEdge.values());
    }
    
    public Set<E> edgeSet() {
        return allEdges;
    }
    
    public Set<V> vertexSet() {
        return distance.keySet();
    }
    
    public DefaultDirectedWeightedGraph<V, URIEdge> toGraph() {
        DefaultDirectedWeightedGraph<V, URIEdge> g = new DefaultDirectedWeightedGraph<>(URIEdge.class);
        
        for(V v : vertexSet()) {
            g.addVertex(v);
        }
        for(E e : edgeSetPrevious()) {
            if(e == null)
                continue;
            
            URIEdge urie = (URIEdge) e;
            g.addEdge((V)urie.getSource(), (V)urie.getTarget(), (URIEdge) e);
        }
        
        return g;
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((distance == null) ? 0 : distance.hashCode());
        result = prime * result + ((previousEdge == null) ? 0 : previousEdge.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        ShortestPathTree other = (ShortestPathTree) obj;
        if (distance == null) {
            if (other.distance != null) {
                return false;
            }
        } else if (!distance.equals(other.distance)) {
            return false;
        }
        if (previousEdge == null) {
            if (other.previousEdge != null) {
                return false;
            }
        } else if (!previousEdge.equals(other.previousEdge)) {
            return false;
        }
        return true;
    }

}
