package de.dfki.km.academicresearch.eswc2016challenge.pathfinder.subsetExtractor;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import scala.collection.mutable.HashMap;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import org.apache.jena.graph.Triple;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.RDFFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SubsetExtractorAll extends PathFinder {

    private static final Logger logger = LoggerFactory.getLogger(SubsetExtractorAll.class);

    private Map<String, Set<String>> mapImportantResources;
    private Map<String, Model> mapModel;

    public SubsetExtractorAll(Collection<Object[]> tasks, String querynumber) {
    	
    	this.mapImportantResources = new java.util.HashMap<>();
    	this.mapModel = new java.util.HashMap<>();
    	
    	for(Object[] array : tasks){
    		int query = (int)array[0];
    		int k = (int)array[1];
    		
    		String task = String.format("task%s_q%s_%s", querynumber, query, k);
    		
    		File f = new File("./challenge_training_result_sets/"+ task +".txt");    		
    		List<Path> listExpected = null;
    		try {
                listExpected = PathList.toPathList(f);
            } catch (Exception e) {
                logger.error("PAthList error", e);
            }

    		Set<String> tmp = new HashSet<>();
            for (Path p : listExpected) {
                for (Resource r : p.getPath()) {
                    if (r.isURIResource()) {                    	
                    	tmp.add(r.getURI());
                    }
                }
            }
            
            this.mapImportantResources.put(task, tmp);
    		this.mapModel.put(task, ModelFactory.createDefaultModel());
    		
    	}
    }

    @Override
    public List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception {
        return null;
    }


    @Override
    public void send(Triple item) {
    	
    	
    	String subject = null;
    	String object = null;
    	
    	if (item.getSubject().isURI())
        	subject = item.getSubject().getURI();
    	
    	if (item.getObject().isURI())
    		object = item.getObject().getURI();
    	
    	for(Entry<String, Set<String>> entry : mapImportantResources.entrySet()){    		
    		Set<String> importantURIs = entry.getValue();
    		if ((subject != null && importantURIs.contains(subject)) || (object!=null && importantURIs.contains(object))) {    			
    			Model m = mapModel.get(entry.getKey());
    			m.add(m.asStatement(item));
            }        		
    	}
    	
    	
    	/*
        if (item.getSubject().isURI()) {
        	
        	for(Entry<String, Set<String>> entry : mapImportantResources.entrySet()){
        		if (entry.getValue().contains(item.getSubject().getURI())) {        			
        			Model m = mapModel.get(entry.getKey());
                    m.add(m.asStatement(item));
                }        		
        	}
        	
        }
        
        
        if (item.getObject().isURI()) {
        	
        	for(Entry<String, Set<String>> entry : mapImportantResources.entrySet()){
        		if (entry.getValue().contains(item.getObject().getURI())) {        			
        			Model m = mapModel.get(entry.getKey());
                    m.add(m.asStatement(item));
                }        		
        	}
        	
        }
        */
    }

    @Override
    public void endStreaming() throws Exception {        
        for(Entry<String, Model> entry : mapModel.entrySet()){        	
        	RDFDataMgr.write(new FileOutputStream("./subset/"+entry.getKey() + ".nt"), entry.getValue(), RDFFormat.NTRIPLES_UTF8);        	
    	}
    }

}
