package de.dfki.km.academicresearch.eswc2016challenge.sink;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.SPOEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.SPOVertex;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.GraphIndexJade;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Map;
import org.apache.jena.atlas.lib.Sink;
import org.apache.jena.graph.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class BulkJadeSink implements Sink<Triple> {

    private static final Logger logger = LoggerFactory.getLogger(BulkJadeSink.class);
    
    private GraphIndexJade jade;
    
    private ArrayList<SPOVertex> vs;
    private ArrayList<SPOEdge> es;
    
    private File saveFile;
    
    public BulkJadeSink(GraphIndexJade jade) {
        this.jade = jade;
        
        vs = new ArrayList<>(367772);
        es = new ArrayList<>(2397994);
        
        //finished: 177248 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.JadeSink  - flush GraphIndexJade: 367772 vertices, 2397994 edges
    }
    
    private int i = 0;
    private int literalCount = 0;
    
    @Override
    public void send(Triple t) {
        
        if(t.getObject().isLiteral()) {
            literalCount++;
            return;
        }
        
        SPOVertexLookup sl = getVertex(t.getSubject().getURI(), jade.getId2Vert());
        SPOVertexLookup ol = getVertex(t.getObject().getURI(), jade.getId2Vert());
        
        if(!sl.found)
            vs.add(sl.vertex);
        
        if(!ol.found)
            vs.add(ol.vertex);
        
        SPOEdge e = new SPOEdge(sl.vertex, t.getPredicate().getURI(), ol.vertex);
        es.add(e);
        
        if(i > 0 && i % 100000 == 0) {
            logger.debug("processed: " + i);
        }
        i++;
    }

    @Override
    public void flush() {
        jade.load(vs, es);
        logger.debug("flush " + jade.toString() + ", " + literalCount + " literals skipped");
        
        if(saveFile != null) {
        
            logger.debug("write to " + saveFile.getAbsolutePath());
            ObjectOutputStream out = null;
            try {
                out = new ObjectOutputStream(new FileOutputStream(saveFile));
                out.writeObject(vs);
                out.writeObject(es);
                out.close();
            } catch (IOException ex) {
                logger.error(null, ex);
            } finally {
                try {
                    out.close();
                } catch (IOException ex) {
                    logger.error(null, ex);
                }
            }
            logger.debug("done");
        
        }
        
        //27048 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.JadeSink2  - write to vs_es.bin
        //45227 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.JadeSink2  - done
        //183 MB file
    }

    @Override
    public void close() {
        logger.debug("close " + jade.toString());
    }

    public File getSaveFile() {
        return saveFile;
    }

    public void setSaveFile(File saveFile) {
        this.saveFile = saveFile;
    }
    
    
    
    private final int NEW = -1;
    
    private SPOVertexLookup getVertex(String id, Map<String, SPOVertex> id2vert) {
        if(id2vert.containsKey(id)) {
            return new SPOVertexLookup(id2vert.get(id), true);
        } else {
            SPOVertex v = new SPOVertex(id);
            v.setIndex(NEW);//to mark it as new
            id2vert.put(v.getId(), v);
            return new SPOVertexLookup(v, false);
        }
    }
    
    private class SPOVertexLookup {
        public SPOVertex vertex;
        public boolean found;
        
        public SPOVertexLookup(SPOVertex vertex, boolean found) {
            this.vertex = vertex;
            this.found = found;
        }
    }
    
}
