package de.dfki.km.academicresearch.eswc2016challenge.sink;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.apache.commons.io.IOUtils;
import org.apache.jena.atlas.lib.Sink;
import org.apache.jena.graph.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class SQLiteSink implements Sink<Triple> {

    private Connection c;

    private static final Logger logger = LoggerFactory.getLogger(SQLiteSink.class);

    public SQLiteSink(String filename) {
        try {
            Class.forName("org.sqlite.JDBC");
            c = DriverManager.getConnection("jdbc:sqlite:" + filename);
        } catch (Exception e) {
            logger.error(null, e);
        }
        
        try {
            fromResourceExecute(c, "/de/dfki/km/academicresearch/eswc2016challenge/sql/create_triples.sql");
        } catch (Exception ex) {
            logger.error(null, ex);
        }
    }

    int i = 0;
    
    @Override
    public void send(Triple t) {
        
        if(t.getObject().isLiteral())
            return;
        
        try {
            fromResourceExecute(c,  "/de/dfki/km/academicresearch/eswc2016challenge/sql/insert_into_triples.sql",
                    //http://stackoverflow.com/questions/12615113/how-to-escape-special-characters-like-in-sqlite-in-android
                    t.getSubject().getURI().replace("'", "''"),
                    t.getPredicate().getURI().replace("'", "''"),
                    t.getObject().getURI().replace("'", "''")
            );
        } catch (Exception ex) {
            logger.error(t.toString(), ex);
        }
        
        if(i > 0 && i % 100 == 0) {
            logger.debug("" + i);
        }
        i++;
        
        //2339 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.SQLiteSink  - 100
        //4224 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.SQLiteSink  - 200
        
        //2 sec for 100 triples
        
        //20000 sec for 10.000.000 triples
        //5,56 h 
    }

    @Override
    public void flush() {
    }

    @Override
    public void close() {
        try {
            c.close();
        } catch (SQLException ex) {
            logger.error(null, ex);
        }
    }

    
    private String fromResource(String classpath, Object... args) throws IOException {
        String sql = IOUtils.toString(SQLiteSink.class.getResourceAsStream(classpath));
        
        if(args.length > 0)
            sql = String.format(sql, args);
        
        return sql;
    }
    
    private ResultSet fromResourceQuery(Connection c, String classpath, Object... args) throws SQLException, IOException {
        String sql = fromResource(classpath, args);
        Statement stmt = c.createStatement();
        return stmt.executeQuery(sql);
    }
    
    private boolean fromResourceExecute(Connection c, String classpath, Object... args) throws SQLException, IOException {
        String sql = fromResource(classpath, args);
        Statement stmt = c.createStatement();
        return stmt.execute(sql);
    }
    
}
