package de.dfki.km.academicresearch.eswc2016challenge.sink;

import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.GraphIndexJade;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.SPOEdge;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jade.SPOVertex;
import java.util.Map;
import org.apache.jena.atlas.lib.Sink;
import org.apache.jena.graph.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class StreamJadeSink implements Sink<Triple> {

    private static final Logger logger = LoggerFactory.getLogger(StreamJadeSink.class);
    
    private GraphIndexJade jade;
    
    public StreamJadeSink(GraphIndexJade jade) {
        this.jade = jade;
        
        //dataset has 10.000.000 triple
        jade.getGraph().grow(0, 3000000);
        
        
        //finished: 177248 [main] DEBUG de.dfki.km.academicresearch.eswc2016challenge.Test.JadeSink  - flush GraphIndexJade: 367772 vertices, 2397994 edges
    }
    
    private int i = 0;
    
    @Override
    public void send(Triple t) {
        
        if(t.getObject().isLiteral())
            return;
        
        SPOVertexLookup sl = getVertex(t.getSubject().getURI(), jade.getId2Vert());
        SPOVertexLookup ol = getVertex(t.getObject().getURI(), jade.getId2Vert());
        
        if(!sl.found)
            jade.getGraph().addVertex(sl.vertex);
        
        if(!ol.found)
            jade.getGraph().addVertex(ol.vertex);
        
        SPOEdge e = new SPOEdge(sl.vertex, t.getPredicate().getURI(), ol.vertex);
        
        jade.getGraph().addEdge(e);
        
        if(i > 0 && i % 100000 == 0) {
            logger.debug(jade.toString());
        }
        i++;
    }

    @Override
    public void flush() {
        logger.debug("flush " + jade.toString());
    }

    @Override
    public void close() {
        logger.debug("close " + jade.toString());
    }
    
    
    private final int NEW = -1;
    
    private SPOVertexLookup getVertex(String id, Map<String, SPOVertex> id2vert) {
        if(id2vert.containsKey(id)) {
            return new SPOVertexLookup(id2vert.get(id), true);
        } else {
            SPOVertex v = new SPOVertex(id);
            v.setIndex(NEW);//to mark it as new
            id2vert.put(v.getId(), v);
            return new SPOVertexLookup(v, false);
        }
    }
    
    private class SPOVertexLookup {
        public SPOVertex vertex;
        public boolean found;
        
        public SPOVertexLookup(SPOVertex vertex, boolean found) {
            this.vertex = vertex;
            this.found = found;
        }
    }
    
}
