package de.dfki.km.academicresearch.eswc2016challenge.sparqlext;

import java.util.List;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.DatasetFactory;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.sparql.algebra.Algebra;
import org.apache.jena.sparql.algebra.Op;
import org.apache.jena.sparql.engine.QueryEngineFactory;
import org.apache.jena.sparql.engine.QueryEngineRegistry;
import org.apache.jena.sparql.engine.QueryExecutionBase;
import org.apache.jena.sparql.engine.main.QueryEngineMain;
import org.apache.jena.sparql.engine.ref.QueryEngineRef;
import org.apache.jena.tdb.TDBFactory;
import org.apache.jena.tdb.TDBLoader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.dfki.km.academicresearch.eswc2016challenge.Main;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import tdb.tdbloader;

public class JenaTest {
	
	private static final Logger logger = LoggerFactory.getLogger(JenaTest.class);
	
	public static void main(String[] args) 
	{
		
		
		//index: https://jena.apache.org/documentation/larq/
		
		 /*
         * PREFIX dbc: <http://dbpedia.org/resource/Category:>
         * PREFIX dbo: <http://dbpedia.org/ontology/>
         * PREFIX dbp: <http://dbpedia.org/property/>
         * PREFIX dbr: <http://dbpedia.org/resource/>
         */
        /*
         String strQuery = "PREFIX dbr: <http://dbpedia.org/resource/>\r\n" + 
         "PREFIX : <http://doesnotexist.org>\r\n" + 
         "SELECT *\r\n" + 
         "WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull }";
	*/
		//One possibility:
		/*
		String strQuery = 
				 "PREFIX dbr: <http://dbpedia.org/resource/>\r\n" + 
		         "PREFIX : <http://doesnotexist.org>\r\n" + 
		         "DESCRIBE *\r\n" + 
		         "WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull }";
		*/
		//another
		//String strQuery = 
		//		   "PREFIX dbr: <http://dbpedia.org/resource/>\r\n" + 
		//         "PREFIX : <http://doesnotexist.org>\r\n" + 
		//         "SELECT *\r\n" + 
		//         "WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull. }\r\n" + 
		//         "LIMIT 10";
		
		
//		   "PREFIX dbr: <http://dbpedia.org/resource/>\r\n" + 
		//         "PREFIX : <http://doesnotexist.org>\r\n" + 
		//         "SELECT *\r\n" + 
		//         "WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull. }\r\n" + 
		//         "LIMIT 10";
		
		
		// ?g1 { s p o.  ......   }
		// 
		// ?g1  = <http://test.de/test>
		
		//second query: 
		//SELECT *
		//FROM <<http://test.de/test>
		//Where{ ....   }
		
		//dbr:Felipe_Massa .....  r1 :test dbr:Red_Bull.
		//                        r1 rdfs:label ?r2  dbr:Red_Bull.
		
		
		//dbr:Felipe_Massa !:* ?r1.
		//r1 rdfs:label ?r2  dbr:Red_Bull.
		
		
		//String strQuery = 
		//         "SELECT ?s ?p ?o\r\n" + 
		//         "WHERE  { ?s ?p ?o. }";
		
		String strQuery = 
		         "SELECT ?p ?o\r\n" + 
		         "WHERE  { <http://km.dfki.de/eswc2016challenge/A> ?p ?o. }";
		
		
		
		//use special variables for path representation....
		//and then make a result set with variables ?length ?s ?p1 ?r1 ?p2 ?r2 ?p3 ?d   (?s for source and ?d for destination)
		//and then make a result set with variables ?length ?p1 ?r1 ?p2 ?r2 ?p3    (maybe only path)
		//print these variables only if * or these variables are encountered in select statement
		//so the following would also working  SELECT ?x ?y ?z ?length ?p1 ?p2
		//The LIMIT and OFFSET  will work as expected and reduce the set of pathes... (also ORDER will work?)
		//but what happens if we have multiple property path experssions? (should we use ?p1length ?p1prop1 ?p1r1 etc.?  and ?p2length ?p2prop1 ?p2r1 etc.?  
		//how to do the connection? because of reordering the triples? 
		//use a quad? WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull ?path}
		
		//maybe then it is also possible to use the variable again and again....
		//for example find a path where the first resource in that path should have a label "test"
		//SELECT *
        //WHERE  { 
		//	br:Felipe_Massa !:* dbr:Red_Bull.
		//  ?r1 rdfs:label "test".
		//}
        //LIMIT 10
		//
		//
		
		
		
		
		//IMPORTANT:
		//https://jena.apache.org/documentation/query/extension.html
		//Extending query evaluation for querying different storage and inference systems
		//     |
		//     > https://jena.apache.org/documentation/query/arq-query-eval.html
		
		//https://jena.apache.org/documentation/query/
		
         Query query = QueryFactory.create(strQuery) ;
         Op op = Algebra.compile(query) ;
         logger.info("Parsed query: {}", op);
         
         Op optimized = Algebra.optimize(op);
         logger.info("Optimised query: {}", optimized);
         
        
        //createDatasetTraining();

		//System.out.println(query.serialize()) ;		
        //IndentedWriter w = new IndentedWriter(System.out);		
        //op.output(w);
		//createDatasetD1();
        //createDatasetTraining();
        //createDatasetTraining();
         makeTDB();
        
         //HiconEngine.register();
         QueryEngineRef.register();
		
         List<QueryEngineFactory> possibleFactories = QueryEngineRegistry.get().factories();
		
		 Model m = ExampleDataset.getDataSetD1();
		 
		 Dataset datasetM = DatasetFactory.create(m);
		
		 //query.addResultVar("bla");//here we can add arbirtary new values :-)
		 //query.addResultVar("?test");
		 
         QueryExecution qe = QueryExecutionFactory.create(query, datasetM);//here also the additional variables are added (for *)//QueryExecutionFactory.create(query, infmodel);
         ResultSet response =  qe.execSelect();
         ResultSetFormatter.out(System.out, response) ;
         qe.close();

         //http://stackoverflow.com/questions/6981467/jena-arq-difference-between-model-graph-and-dataset
         //http://answers.semanticweb.com/questions/3186/model-vs-datasource-vs-dataset-vs-graph-vs-datasetgraph
         //https://cwiki.apache.org/confluence/display/JENA/SPI+Contract+Details
         //https://www.ibm.com/developerworks/community/blogs/nlp/entry/an_introduction_to_the_jena_api?lang=en
         //http://answers.semanticweb.com/questions/20372/how-to-integrate-a-triple-store-into-jena
         //https://groups.yahoo.com/neo/groups/jena-dev/conversations/topics/46674
	    
         //DatasetGraphTDB
	    
         //QueryExecutionBase qeb;
	    
         //QueryEngineMain main;
		
         //Query query = OpAsQuery.asQuery(op) ;
         //System.out.println(query.serialize()) ;
         
		
         
         
         
         //QueryExecution tdbqe = QueryExecutionFactory.create(newquery, tdb);//QueryExecutionFactory.create(query, infmodel);
         //ResultSet newRespose =  tdbqe.execSelect();
         //ResultSetFormatter.out(System.out, newRespose) ;
         //tdbqe.close();
		
	}

	private static void testTDB(){
		String newquery = "SELECT * WHERE {<http://km.dfki.de/eswc2016challenge/A> ?p ?o. }";
        
        Dataset tdb = makeTDB();
        
        
        tdb.begin(ReadWrite.READ) ;
        try {
       	 String qs1 = "SELECT * {?s ?p ?o} LIMIT 10" ;

       	 try(QueryExecution qExec = QueryExecutionFactory.create(qs1, tdb)) {
       	     ResultSet rs = qExec.execSelect() ;
       	     ResultSetFormatter.out(rs) ;
       	 }
        } finally { tdb.end() ; }
	}
	
	private static Dataset makeTDB(){
		//IMportant!
        //https://jena.apache.org/documentation/tdb/java_api.html
	     //http://stackoverflow.com/questions/11101757/jena-tdb-to-store-and-query-using-api
		
		//https://jena.apache.org/documentation/tdb/architecture.html
		
		//http://stackoverflow.com/questions/9200940/how-to-load-large-ntriple-data-into-jena-tdb
		
			//https://jena.apache.org/documentation/tdb/tdb_transactions.html
		
		 // Make a TDB-backed dataset
		  String directory = "challenge_training_dataset/tdb" ;
		  Dataset dataset = TDBFactory.createDataset(directory) ;

		  //TDBLoader a;
		  //tdbloader b;
		 // TDBLoader.load(dataset, "", true) ;

		  dataset.begin(ReadWrite.WRITE) ;
		  try {
			  Model model = dataset.getDefaultModel() ;
			  model.add(ExampleDataset.getDataSetD1());
			  
			  dataset.commit() ;//Have to call commit otherwise the information is not stored
		  } finally { 
		    dataset.end() ; 
		  };
		  
		  return dataset;
	}

}
