package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset;

import java.io.File;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.DatasetFactory;
import org.apache.jena.query.text.DatasetGraphText;
import org.apache.jena.query.text.TextDocProducer;
import org.apache.jena.query.text.TextDocProducerTriples;
import org.apache.jena.query.text.TextIndex;
import org.apache.jena.query.text.TextIndexConfig;
import org.apache.jena.query.text.TextQuery;
import org.apache.jena.sparql.core.DatasetGraph;
import org.apache.jena.sparql.util.Context;
import org.apache.lucene.store.Directory;

public class GraphDatasetFactory {
	
	/**
     * Create a graph-indexed dataset
     *
     * @param base the base Dataset
     * @param directory The directory for the index
     */
    public static Dataset createGraphIndex(Dataset base, File directory)
    {
        GraphIndex index = createGraphIndex(directory) ;
        return create(base, index, true) ;
    }

	public static GraphIndex createGraphIndex(File directory) {
		return new PathFinderGraphIndex(directory);
	}
	
	
	
	
	 /** Create a graph-indexed dataset, optionally allowing the graph index to be closed if the Dataset is */
    public static Dataset create(Dataset base, GraphIndex graphIndex, boolean closeIndexOnDSGClose)
    {
        DatasetGraph dsg = base.asDatasetGraph() ;
        dsg = create(dsg, graphIndex, closeIndexOnDSGClose) ;
        return DatasetFactory.wrap(dsg) ;
    }
    
    /** Create a text-indexed DatasetGraph, optionally allowing the text index to be closed if the DatasetGraph is */
    public static DatasetGraph create(DatasetGraph dsg, GraphIndex graphIndex, boolean closeIndexOnDSGClose)
    {
        return create(dsg, graphIndex, closeIndexOnDSGClose, null);
    }
    
    /** Create a text-indexed DatasetGraph, optionally allowing the text index to be closed if the DatasetGraph is */
    public static DatasetGraph create(DatasetGraph dsg, GraphIndex graphIndex, boolean closeIndexOnDSGClose, GraphSynchronizer producer) {
        if (producer == null) producer = new GraphSynchronizer(graphIndex) ;
        DatasetGraph dsgt = new DatasetGraphGraph(dsg, graphIndex, producer, closeIndexOnDSGClose) ;
        // Also set on dsg
        Context c = dsgt.getContext() ;
        c.set(GraphQuery.graphIndex, graphIndex) ;
        
        return dsgt ;
    }

}
