package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset;

import org.apache.jena.query.ARQ;
import org.apache.jena.query.text.TextQueryPF;
import org.apache.jena.query.text.assembler.TextAssembler;
import org.apache.jena.sparql.SystemARQ;
import org.apache.jena.sparql.engine.main.QC;
import org.apache.jena.sparql.lib.Metadata;
import org.apache.jena.sparql.mgt.SystemInfo;
import org.apache.jena.sparql.pfunction.PropertyFunction;
import org.apache.jena.sparql.pfunction.PropertyFunctionFactory;
import org.apache.jena.sparql.pfunction.PropertyFunctionRegistry;
import org.apache.jena.sparql.util.Symbol;
import org.apache.jena.system.JenaSystem;

import de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset.assembler.GraphAssembler;
import de.dfki.km.academicresearch.eswc2016challenge.sparqlext.opExecutor.HiconOpExecutorFactory;

public class GraphQuery {
	
	private static volatile boolean initialized = false ;
    private static Object lock = new Object() ;
    public static String NS = "http://jena.apache.org/graph#" ;
    public static String IRI = "http://jena.apache.org/#graph" ;
    public static final Symbol graphIndex = Symbol.create(NS+"index") ;
    public static final String PATH         = "de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset";
    
    static private String metadataLocation  = "org/apache/jena/query/text/properties.xml" ;
    static private Metadata metadata        = new Metadata(metadataLocation) ;
    public static final String NAME         = "ARQ Text Query";
   
    public static final String VERSION      = metadata.get(PATH+".version", "unknown") ;
    public static final String BUILD_DATE   = metadata.get(PATH+".build.datetime", "unset") ;
    
    static { JenaSystem.init(); }
    
    public static void init() 
    {
        if ( initialized ) 
            return ;
        synchronized(lock) {
            if ( initialized ) {
                if ( JenaSystem.DEBUG_INIT )
                    System.err.println("GraphQuery.init - skip") ;
                return ; 
            }
            initialized = true ;
            if ( JenaSystem.DEBUG_INIT )
                System.err.println("GraphQuery.init - start") ;
            
            GraphAssembler.init() ;
            
            SystemInfo sysInfo = new SystemInfo(IRI, PATH, VERSION, BUILD_DATE) ;
            SystemARQ.registerSubSystem(sysInfo) ;
            
            /*
            PropertyFunctionRegistry.get().put("http://jena.apache.org/text#query", new PropertyFunctionFactory() {
                @Override
                public PropertyFunction create(String uri) {
                    return new TextQueryPF() ;
                }
            });
            */
            
            //Own configurations:            
            QC.setFactory(ARQ.getContext(), new HiconOpExecutorFactory()) ;
            
            ARQ.getContext().set(ARQ.optFilterEquality, false);//for filter applications
            
            
            if ( JenaSystem.DEBUG_INIT )
                System.err.println("GraphQuery.init - finish") ;
        }
    }

}
