package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset;

import org.apache.jena.graph.Node;
import org.apache.jena.query.text.Entity;
import org.apache.jena.query.text.EntityDefinition;
import org.apache.jena.query.text.TextDocProducerTriples;
import org.apache.jena.query.text.TextIndex;
import org.apache.jena.query.text.TextQueryFuncs;
import org.apache.jena.sparql.core.DatasetChanges;
import org.apache.jena.sparql.core.QuadAction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Responsible for monitoring changes on a dataset and updating the GraphIndex appropriately.
 * <p>
 * <strong>Note:</strong> If implementation is to be instantiated dynamically via an Assembler, it must
 * contain a constructor that accepts a single parameter of type {@link de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset.GraphIndex}
 */
public class GraphSynchronizer implements DatasetChanges{

	private static Logger log     = LoggerFactory.getLogger(GraphSynchronizer.class) ;
    private final GraphIndex indexer ;
    
    // Have to have a ThreadLocal here to keep track of whether or not we are in a transaction,
    // therefore whether or not we have to do autocommit
    private final ThreadLocal<Boolean> inTransaction = new ThreadLocal<Boolean>() {
        @Override
        protected Boolean initialValue() {
            return Boolean.FALSE ;
        }
    } ;
	
	
	public GraphSynchronizer(GraphIndex indexer){
		this.indexer = indexer;
	}
	
	
	
	//Methods from DatasetChanges
	
	@Override
	public void start() {
		inTransaction.set(true) ;
	}
	
	@Override
	public void finish() {
		inTransaction.set(false) ;
	}
	
	@Override
	public void reset() { }

	@Override
	public void change(QuadAction qaction, Node g, Node s, Node p, Node o) {

		//Here the change happens
		
		if(qaction == QuadAction.ADD){
			indexer.add(g, s, p, o);
			autocommit();
		}
		else if(qaction == QuadAction.DELETE){
			indexer.remove(g, s, p, o);
			autocommit();
		}
	}
	
	/**
	 * Auto commit the entity if we aren't in a transaction
	 */
	private void autocommit(){
        if (!inTransaction.get()) {
            indexer.commit();
        }
	}

}
