package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.fuseki;

import java.io.File;

import org.apache.jena.atlas.lib.StrUtils;
import org.apache.jena.query.ARQ;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.DatasetFactory;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.query.text.EntityDefinition;
import org.apache.jena.query.text.TextDatasetFactory;
import org.apache.jena.query.text.TextIndexConfig;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.sparql.engine.main.OpExecutorFactory;
import org.apache.jena.sparql.engine.main.QC;
import org.apache.jena.sparql.util.QueryExecUtils;
import org.apache.jena.system.JenaSystem;
import org.apache.jena.vocabulary.RDFS;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;

import de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset.GraphDatasetFactory;
import de.dfki.km.academicresearch.eswc2016challenge.sparqlext.dataset.GraphIndex;

public class DataSetTest {

	public static void main(String[] args) {

		JenaSystem.DEBUG_INIT = true;
		
		//Dataset d = createCode();
		//Dataset d = createCode2();
		Dataset d = createAssembler() ;
		
		OpExecutorFactory f =  QC.getFactory(ARQ.getContext());
		
		loadData(d, "./subset/task1_q1_8.nt");
		queryData(d);
	}

	
	
	
	public static Dataset createCode() 
    {
		Dataset ds1 = DatasetFactory.create() ; 
		Dataset ds = GraphDatasetFactory.createGraphIndex(ds1, new File("/.testgraph/"));
		return ds;
    }
	
	
	public static Dataset createCode2(){
		
		GraphIndex graphIndex = GraphDatasetFactory.createGraphIndex(new File("./test"));
		Dataset ds = DatasetFactory.create() ;
		return GraphDatasetFactory.create(ds, graphIndex, true) ;
	}
	
	public static Dataset createAssembler() 
    {
		//Don't use tdb -> because different engine -> tdb engine -< has different executor which is given in the executioncontext
        Dataset ds = DatasetFactory.assemble("graph-config.ttl", "http://localhost/jena_example/#graph_dataset") ;
        return ds ;
    }
	
	
	public static void loadData(Dataset dataset, String file)
    {
        dataset.begin(ReadWrite.WRITE) ;
        try {
            Model m = dataset.getDefaultModel() ;
            RDFDataMgr.read(m, file) ;
            dataset.commit() ;
        } finally { dataset.end() ; }
    }
	
	
	public static void queryData(Dataset dataset)
    {
        String query = StrUtils.strjoinNL
            ( "PREFIX dbr: <http://dbpedia.org/resource/>"
            , "PREFIX : <http://doesnotexist.org>"
            , "SELECT *"
            , "WHERE  { dbr:Felipe_Massa !:* dbr:Red_Bull. }") ;
        
        dataset.begin(ReadWrite.READ) ;
        try {
            Query q = QueryFactory.create(query) ;
            
            q.addResultVar("length");
    		for(int count = 0; count < 20; count++){
    			String variableName = "r" + count;
    			q.addResultVar(variableName);			
    		}
            
            QueryExecution qexec = QueryExecutionFactory.create(q , dataset) ;
            QueryExecUtils.executeQuery(q, qexec) ;
        } finally { dataset.end() ; }

    }
}
