package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.propertyfunc;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.function.Function;

import org.apache.jena.atlas.iterator.Iter;
import org.apache.jena.datatypes.RDFDatatype;
import org.apache.jena.datatypes.xsd.XSDDatatype;
import org.apache.jena.graph.Node;
import org.apache.jena.graph.NodeFactory;
import org.apache.jena.query.QueryBuildException;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.RDFList;
import org.apache.jena.rdf.model.RDFNode;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.apache.jena.sparql.core.Substitute;
import org.apache.jena.sparql.core.Var;
import org.apache.jena.sparql.engine.ExecutionContext;
import org.apache.jena.sparql.engine.QueryIterator;
import org.apache.jena.sparql.engine.binding.Binding;
import org.apache.jena.sparql.engine.binding.BindingFactory;
import org.apache.jena.sparql.engine.binding.BindingMap;
import org.apache.jena.sparql.engine.iterator.QueryIterPlainWrapper;
import org.apache.jena.sparql.pfunction.PFuncSimpleAndList;
import org.apache.jena.sparql.pfunction.PropFuncArg;
import org.apache.jena.sparql.pfunction.PropertyFunctionBase;
import org.apache.jena.sparql.util.NodeFactoryExtra;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.EppsteinV2;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.MarkusV2;
import de.dfki.km.academicresearch.eswc2016challenge.sparqlext.JenaTest;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;

//or PropertyFunctionBase
public class PathSearch extends PFuncSimpleAndList {

	private static final Logger logger = LoggerFactory.getLogger(PathSearch.class);
	
	private Resource start;
	private Resource end;
		
	
	private PathFinder p;
	public PathSearch() {
		p = new MarkusV2();// EppsteinV2();
		
		logger.info("PathSearchConstructor");
		
		p.setIndex("challenge_training_dataset", "datasetD1");
		try{
	        if(p.hasIndex()){        	
	        	p.loadIndex(); 
	        }
	        else{
	            StreamRDF streamRDF = StreamRDFLib.sinkTriples(p);  
	            p.startStreaming();
	            StreamOps.graphToStream(ExampleDataset.getDataSetD1().getGraph(), streamRDF);   
	        }
		}
		catch(Exception e){
			logger.error("Exception with streaming", e);
		}
		
	}
	
	
	@Override
    public void build(PropFuncArg argSubject, Node predicate, PropFuncArg argObject, ExecutionContext execCxt) {
        super.build(argSubject, predicate, argObject, execCxt) ;
      

            List<Node> list = argObject.getArgList() ;

            if (list.size() != 4)
                throw new QueryBuildException("Not exact 4 arguments in list (start end numberOfPathes propertyPathExpression)") ;

            if ( list.get(0).isURI() == false)
            	throw new QueryBuildException("First argument (start) is not a URI Resource") ;
            
            
            if ( list.get(1).isURI() == false)
            	throw new QueryBuildException("Second argument (end) is not a URI Resource") ;
            
            if ( list.get(2).isLiteral() == false)
            	throw new QueryBuildException("Third argument (numberOfPathes) is not literal") ;
            
            if ( list.get(2).getLiteralDatatype() != XSDDatatype.XSDinteger && list.get(2).getLiteralDatatype() != XSDDatatype.XSDint)
            	throw new QueryBuildException("Third argument (numberOfPathes) is not integer") ;
            
            if ( list.get(3).isLiteral() == false)
            	throw new QueryBuildException("Fourth argument (propertyPathExpression) is not literal") ;
            
            if ( list.get(3).getLiteralDatatype() != XSDDatatype.XSDstring)
            	throw new QueryBuildException("Fourth argument (propertyPathExpression) is not string") ;
            
    }
	
	
	
	
	@Override
	public QueryIterator execEvaluated(Binding binding, Node subject, Node predicate, PropFuncArg object, ExecutionContext execCxt) {

		List<Node> list = object.getArgList() ;
		
		String start = list.get(0).getURI();
		String end = list.get(1).getURI();
		
		int numberOfPathes = (int) list.get(2).getLiteralValue();
		String propertyPathExpression = (String) list.get(3).getLiteralValue();
		
		
		
		Var sVar = Var.isVar(subject) ? Var.alloc(subject) : null ;
		
		List<Path> pathes = new LinkedList<>();
		try {
			pathes = p.find(start, end, numberOfPathes, "");
		} catch (Exception e) {
			logger.error("Pathfinder problem", e);
		}
		
		
		Function<Path,Binding> converter = (Path hit) -> {
						
			BindingMap bmap = BindingFactory.create(binding);
			
			
			bmap.add(Var.alloc("length"), NodeFactoryExtra.intToNode(hit.resourceCount()));
			int count = 0;
			for(Resource r : hit.getPath()){				
				String variableName = "r" + count;
				bmap.add(Var.alloc(variableName), NodeFactory.createURI(r.getURI()));	
				count++;
			}
			//Does not work!
			/*
			Model mymodel = ModelFactory.createDefaultModel();
			
			List<RDFNode> blaList = new LinkedList();
			blaList.add(mymodel.createLiteral("Test1"));
			blaList.add(mymodel.createLiteral("Test2"));
			RDFList test = mymodel.createList(blaList.iterator());

			bmap.add(Var.alloc("r19"), test.asNode());
			//bmap.add(sVar, NodeFactory.createLiteral("Test"));
			*/
			/*
			BindingMap bmap = BindingFactory.create(binding);
			bmap.add(sVar, NodeFactory.createLiteral("Test"));
			
			Var test = Var.alloc("bla");
			bmap.add(test, NodeFactory.createLiteral("Blub"));
			*/
			return bmap;
			
		};
		
		Iterator<Binding> bIter = Iter.map(pathes.iterator(), converter);
        QueryIterator qIter = new QueryIterPlainWrapper(bIter, execCxt);
        return qIter ;
		
		
		/*
		Function<TextHit,Binding> converter = (TextHit hit) -> {
            if (score == null && literal == null)
                return sVar != null ? BindingFactory.binding(binding, sVar, hit.getNode()) : BindingFactory.binding(binding);
            BindingMap bmap = BindingFactory.create(binding);
            if (sVar != null)
                bmap.add(sVar, hit.getNode());
            if (scoreVar != null)
                bmap.add(scoreVar, NodeFactoryExtra.floatToNode(hit.getScore()));
            if (literalVar != null)
                bmap.add(literalVar, hit.getLiteral());
            return bmap;
        } ;
		
		
		Iterator<Binding> bIter = Iter.map(results.iterator(), converter);
        QueryIterator qIter = new QueryIterPlainWrapper(bIter, execCxt);
        return qIter ;
		*/		
		
		//return null;
	} 

	//https://github.com/apache/jena/blob/master/jena-text/src/main/java/org/apache/jena/query/text/TextQueryPF.java
	/*
	public PathSearch() {
		super();
	}
	
	
	@Override
	public QueryIterator exec(Binding binding, PropFuncArg argSubject, Node predicate, PropFuncArg argObject,
			ExecutionContext execCxt) {

		//From PropertyFunctionEval
		argSubject = Substitute.substitute(argSubject, binding) ;
        argObject =  Substitute.substitute(argObject, binding) ;
		
        
        
        
                
        
		return null;
	}
*/
	
	
	
	
	
}
