package de.dfki.km.academicresearch.eswc2016challenge.sparqlext.propertyfunc.test;

import org.apache.jena.query.ARQ;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.sparql.pfunction.PropertyFunctionRegistry;
import org.apache.jena.vocabulary.RDFS;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PropertyFunctionTest {
	
private static final Logger logger = LoggerFactory.getLogger(PropertyFunctionTest.class);
	
	public static void main(String[] args) 
	{
		//https://jena.apache.org/documentation/query/writing_propfuncs.html
		//examples:
		//https://github.com/apache/jena/tree/master/jena-arq/src-examples/arq/examples/propertyfunction
		
		//special:
		//https://github.com/apache/jena/blob/master/jena-arq/src-examples/arq/examples/propertyfunction/labelSearch.java
		//The approach here could be used to access an external index (e.g. Lucene) although here we just show looking for RDFS labels.
		
		//path as property function:
		//https://jena.apache.org/documentation/query/property_paths.html
		
		//final PropertyFunctionRegistry reg = PropertyFunctionRegistry.chooseRegistry(ARQ.getContext());
		//reg.put("urn:ex:fn#example", new ExamplePropertyFunctionFactory());
		//PropertyFunctionRegistry.set(ARQ.getContext(), reg);
		
		
		
		//We use register instead of dynamically linking:
		
		Model model = make() ;
		
		PropertyFunctionRegistry.get().put("http://example/f#search", labelSearch.class) ;
        String prologue = "PREFIX ext: <http://example/f#>\n" ;
        String  qs = prologue+"SELECT * { ?x ext:search 'EF' }" ;
        Query query = QueryFactory.create(qs) ;
        try ( QueryExecution qExec = QueryExecutionFactory.create(query, model) ) {
            ResultSet rs = qExec.execSelect() ;
            ResultSetFormatter.out(rs) ;
        }
	
	}
	
	private static Model make()
    {
        String BASE = "http://example/" ;
        Model model = ModelFactory.createDefaultModel() ;
        model.setNsPrefix("", BASE) ;
        Resource r1 = model.createResource(BASE+"r1") ;
        Resource r2 = model.createResource(BASE+"r2") ;

        r1.addProperty(RDFS.label, "abc") ;
        r2.addProperty(RDFS.label, "def") ;

        return model  ;
    }

}