package de.dfki.km.academicresearch.eswc2016challenge.util;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import org.apache.jena.rdf.model.Resource;
import org.junit.Assert;


public class Util {
	
	
	public static int smallestPath(List<Path> p){		
		int min = 0;		
		for(Path path : p){
			if(path.getPath().size() > min){
				min = path.getPath().size();
			}
		}
		return min;
	}
	
	
	
	
	
	
	
	public static boolean equalsSlow(PathList expected, PathList actual){
		
		TreeMap<Integer, Set<Path>> expectedMap = expected.getMap();
		TreeMap<Integer, Set<Path>> actualMap = actual.getMap();
		
		
		//check expectedMap in actualMap -> too few pathes
		for(Entry<Integer, Set<Path>> entry : expectedMap.entrySet()){
			
			if(actualMap.containsKey(entry.getKey()) == false){
				throw new AssertionError(String.format("No pathes with length %d are contained but it should be %d, for example: %s.", entry.getKey(), entry.getValue().size(), entry.getValue().iterator().next().toString()));
			}
			
			Set<Path> expectedPathSet = entry.getValue();
			Set<Path> actualPathSet = actualMap.get(entry.getKey());
			
			equalsSlow(expectedPathSet, actualPathSet, entry.getKey());
		}
			
		
		//check actualMap in expectedMap -> too many pathes
		for(Entry<Integer, Set<Path>> entry : actualMap.entrySet()){
			
			if(expectedMap.containsKey(entry.getKey()) == false){
				throw new AssertionError(String.format("Too much. We have at least one path with length %d which is not in expected: %s", entry.getKey(), entry.getValue().iterator().next().toString()));
			}
			
			Set<Path> actualPathSet = entry.getValue();
			Set<Path> expectedPathSet = expectedMap.get(entry.getKey());
			
			equalsSlow(expectedPathSet, actualPathSet, entry.getKey());
		}
        
		return true;
		
	}
	
	
	public static boolean equalsSlow(Set<Path> expected, Set<Path> actual, int length){
	
		for (Path p : expected)
            if (!actual.contains(p))
            	throw new AssertionError(String.format("Too few! Following Path with length %d is expected but not in actual: %s", length, p.toString()));
        
        for (Path p : actual)
            if (!expected.contains(p))
            	throw new AssertionError(String.format("Too many! Following Path with length %d is in actual but it should not be there: %s",length, p.toString()));
        return true;
	
	}
	
	//=================================================
	
	

    public static List<Path> getPaths(File f) throws Exception {

        List<Path> list = new LinkedList<>();
        try (BufferedReader br = new BufferedReader(new FileReader(f))) {
            String line;
            while ((line = br.readLine()) != null) {
                Path p = new Path(line);
                list.add(p);
            }
        }
        return list;

    }

    public static void assertEqualSet(List<Path> expected, List<Path> actual) {
        Assert.assertEquals("list size difference", expected.size(), actual.size());

        Set<Path> expectedSet = new HashSet<>(expected);
        Set<Path> actualSet = new HashSet<>(actual);
        
        Assert.assertTrue("expected contains not all of actual", expectedSet.containsAll(actualSet));
        Assert.assertTrue("actual contains not all of expected", actualSet.containsAll(expectedSet));
    }
    
    @Deprecated
    public static void assertEquals(List<Path> expected, List<Path> actual) {
        Assert.assertEquals("list size difference", expected.size(), actual.size());

        for (int i = 0; i < expected.size(); i++) {
            Path e = expected.get(i);
            Path a = actual.get(i);

            assertEquals(e, a, i);
        }
    }
    
    public static void assertEquals(Path expected, Path actual, int j) {
        Assert.assertEquals("path size difference at index " + j, expected.getPath().size(), actual.getPath().size());
        
        for (int i = 0; i < expected.getPath().size(); i++) {
            Resource e = expected.getPath().get(i);
            Resource a = actual.getPath().get(i);

            Assert.assertEquals("different resource at index " + i, e, a);
        }
    }
    
    //==========================================================================
    @Deprecated
    public static boolean equals(List<Path> pl1, List<Path> pl2) {
        if(pl1.size() != pl2.size())
            return false;

        for (int i = 0; i < pl2.size(); i++) {
            Path p1 = pl1.get(i);
            Path p2 = pl2.get(i);

            if(!equals(p1, p2))
                return false;
        }
        
        return true;
    }
    
    public static boolean equals(Path p1, Path p2) {
        if(p1.getPath().size() != p2.getPath().size())
            return false;

        for (int i = 0; i < p2.getPath().size(); i++) {
            Resource r1 = p1.getPath().get(i);
            Resource r2 = p2.getPath().get(i);

            if(!r1.equals(r2))
                return false;
        }
        
        return true;
    }
    
}
