package de.dfki.km.jade.graph;



import java.util.List;




/**
 * This is the general interface for BiAdjacencyLists, which defines the methods for edge relationship/neighbourhood lookups
 * 
 * @param <V> the vertex type
 * @param <E> the edge type
 */
public interface BiAdjacencyList<V extends Vertex, E extends Edge<V>>
{

    void addEdge(E newEdge);



    public int getBiAdjacencyListSize();



    public List<E> getEdges();



    /**
     * Gets all edges with a given node as source entry. These are also called the 'target edges' of this node.
     * 
     * @param vertexIndex the node
     * 
     * @return all edges with a given node as target entry
     */
    public List<E> getEdgesWithSource(int vertexIndex);



    /**
     * Gets all edges with a given node as source entry. These are also called the 'target edges' of this node.
     * 
     * @param vertex the node
     * 
     * @return all edges with a given node as target entry
     */
    public List<E> getEdgesWithSource(V vertex);



    /**
     * Gets all edges with a given node as target entry. These are also called the 'source edges' of this node.
     * 
     * @param vertexIndex the node
     * 
     * @return all edges with a given node as target entry
     */
    public List<E> getEdgesWithTarget(int vertexIndex);



    /**
     * Gets all edges with a given node as target entry. These are also called the 'source edges' of this node.
     * 
     * @param vertex the node
     * 
     * @return all edges with a given node as target entry
     */
    public List<E> getEdgesWithTarget(V vertex);



    public List<E> getNeighbourEdges(int vertexIndex);



    public List<E> getNeighbourEdges(V vertex);
}
