package de.dfki.km.jade.graph;



import java.util.List;

import de.dfki.km.jade.graph.core.CoreBiGraph;
import de.dfki.km.jade.graph.core.CoreSubGraph;
import de.dfki.km.jade.graph.core.GraphEntitiesContainer;
import de.dfki.km.jade.graph.search.SubGraph;





/**
 * The general, minimum interface for a graph. Graphs can maintain their vertex indices, such as {@link CoreBiGraph} and
 * {@link GraphEntitiesContainer} or not, such as {@link CoreSubGraph} or all other {@link SubGraph}s. In the case a graph maintains its vertex
 * indices, the internal index value of added vertex objects will be modified.
 * 
 * @author reuschling
 * 
 * @param <V> the vertex type
 * @param <E> the edge type
 */
public interface Graph<V extends Vertex, E extends Edge<V>> extends Entity
{


    /**
     * Adds an edge to the graph. The method assumes all edge vertices are already added to the graph.<br>
     * 
     * @param edge the edge that should be added to the graph.
     */
    public void addEdge(E edge);



    /**
     * Adds a vertex inside the graph.<br>
     * <b>Caution</b>: The internal index of the vertex object will be modified for graphs that maintain their vertex indices.
     * 
     * @param vertex the vertex that should be created/added to the graph. Its internal index attribute will be modified for graphs that maintain
     *            their vertex indices.
     * 
     * @return the vertex index inside the graph, possibly set as index inside the given vertex object for graphs that maintain their vertex indices.
     */
    public int addVertex(V vertex);



    public boolean contains(E edge);



    public boolean contains(V vertex);



    public List<E> getEdges();



    public V getVertex(int index);



    public List<V> getVertices();




}
