package de.dfki.km.jade.graph.core;



import de.dfki.km.jade.graph.Edge;
import de.dfki.km.jade.graph.Vertex;


/**
 * A basic edge implementation
 * 
 * @author reuschling
 *
 * @param <V> the vertex type
 */
public class CoreEdge<V extends Vertex> implements Edge<V>
{

    protected V m_source;

    protected V m_target;



    protected CoreEdge()
    {
    }



    public CoreEdge(V source, V target)
    {
        m_source = source;
        m_target = target;
    }



    @Override
    public boolean equals(Object obj)
    {
        if(this == obj) return true;
        if(obj == null) return false;


        @SuppressWarnings("rawtypes")
        Edge casted = (Edge) obj;


        return m_source.equals(casted.getSource()) && m_target.equals(casted.getTarget());
    }



    @Override
    public int hashCode()
    {
        int hashCode = 31 + (m_source == null ? 0 : m_source.hashCode());
        hashCode = 31 * hashCode + (m_target == null ? 0 : m_target.hashCode());

        return hashCode;
    }



    @Override
    public V getSource()
    {
        return m_source;
    }



    @Override
    public V getTarget()
    {
        return m_target;
    }



    @Override
    public boolean hasSource(V vertex)
    {
        if(vertex.getIndex() == m_source.getIndex()) return true;

        return false;
    }



    @Override
    public boolean hasTarget(V vertex)
    {
        if(vertex.getIndex() == m_target.getIndex()) return true;

        return false;
    }



    @Override
    public String toString()
    {
        return m_source.toString() + ">>" + m_target.toString();
    }



    @Override
    public V getOtherVertex(V vertex)
    {
        if(hasSource(vertex)) return getTarget();

        if(hasTarget(vertex)) return getSource();

        return null;
    }

}
