package de.dfki.km.jade.graph.core;



import de.dfki.km.jade.graph.Vertex;
import de.dfki.km.jade.graph.WeightedEdge;


/**
 * A basic {@link WeightedEdge} implementation..
 * 
 * @author reuschling
 *
 * @param <V> the vertex type
 */
public class CoreWeightedEdge<V extends Vertex> implements WeightedEdge<V>
{


    protected V m_source;

    protected V m_target;

    protected float m_weight = 0f;



    protected CoreWeightedEdge()
    {
    }



    public CoreWeightedEdge(V source, V target)
    {
        m_source = source;
        m_target = target;
    }



    public CoreWeightedEdge(V source, V target, float weight)
    {
        m_source = source;
        m_target = target;
        m_weight = weight;
    }



    @Override
    public void addWeight(float weight2add)
    {
        m_weight += weight2add;
    }



    @Override
    public boolean equals(Object obj)
    {
        if(this == obj) return true;
        if(obj == null) return false;


        @SuppressWarnings("rawtypes")
        WeightedEdge casted = (WeightedEdge) obj;


        return m_source.equals(casted.getSource()) && m_target.equals(casted.getTarget()) && (m_weight == casted.getWeight());
    }



    @Override
    public V getOtherVertex(V vertex)
    {
        if(hasSource(vertex)) return getTarget();

        if(hasTarget(vertex)) return getSource();

        return null;
    }



    @Override
    public V getSource()
    {
        return m_source;
    }



    @Override
    public V getTarget()
    {
        return m_target;
    }



    @Override
    public float getWeight()
    {
        return m_weight;
    }



    @Override
    public int hashCode()
    {
        int hashCode = 31 + (m_source == null ? 0 : m_source.hashCode());
        hashCode = 31 * hashCode + (m_target == null ? 0 : m_target.hashCode());
        hashCode = 31 * hashCode + Float.floatToIntBits(m_weight);

        return hashCode;
    }



    @Override
    public boolean hasSource(V vertex)
    {
        if(vertex.getIndex() == m_source.getIndex()) return true;

        return false;
    }



    @Override
    public boolean hasTarget(V vertex)
    {
        if(vertex.getIndex() == m_target.getIndex()) return true;

        return false;
    }



    @Override
    public void setWeight(float weight)
    {
        m_weight = weight;
    }



    @Override
    public void subtractWeight(float weight2subtract)
    {
        m_weight -= weight2subtract;
    }



    @Override
    public String toString()
    {
        return new StringBuilder().append(m_source).append(">w").append(m_weight).append(">").append(m_target).toString();
    }

}
