package de.dfki.km.jade.graph.search;



import java.util.Set;

import de.dfki.km.jade.graph.Edge;
import de.dfki.km.jade.graph.Entity;
import de.dfki.km.jade.graph.Vertex;
import de.dfki.km.jade.graph.WeightedEdge;
import de.dfki.km.jade.graph.core.CorePath;



/**
 * Simply sums up all edge weights of a path. Filters nothing. In the case an edge is not of type WeightedEdge, the method adds a default weight,
 * which is configurable.
 * 
 * @author reuschling
 * 
 * @param <V> the vertex type
 * @param <E> the edge type
 * @param <R> the result type
 */
public class EdgeSumWeightCalculator<V extends Vertex, E extends Edge<V>, R extends Entity> extends WeightCalculator<V, E, R>
{

    protected float m_fDefaultWeight = 0;



    @SuppressWarnings("rawtypes")
    @Override
    public float calculateWeight(V newEndVertex, E newEndEdge, CorePath<V, E> parentPath)
    {

        return parentPath.getWeight() + (newEndEdge instanceof WeightedEdge ? ((WeightedEdge) newEndEdge).getWeight() : m_fDefaultWeight);
    }



    /**
     * Specifies the weight that is added to the sum in the case the current edge is not of type {@link WeightedEdge}
     * 
     * @param fDefaultWeight the default weight. default is 0
     * 
     * @return this, as sugar
     */
    public EdgeSumWeightCalculator<V, E, R> setDefaultWeight(float fDefaultWeight)
    {
        m_fDefaultWeight = fDefaultWeight;

        return this;
    }



    /**
     * Get the weight that is added to the sum in the case the current edge is not of type {@link WeightedEdge}
     * 
     * @return the weight that is added to the sum in the case the current edge is not of type {@link WeightedEdge}
     */
    public float getDefaultWeight()
    {
        return m_fDefaultWeight;
    }



    @Override
    public boolean filterResult(R potentialResult, Set<R> currentResults)
    {
        // we allow all results
        return false;
    }

}
