package de.dfki.km.jade.graph.search;



import java.util.HashSet;
import java.util.Set;

import de.dfki.km.jade.graph.Edge;
import de.dfki.km.jade.graph.Vertex;
import de.dfki.km.jade.graph.WeightedEdge;
import de.dfki.km.jade.graph.core.CorePath;



/**
 * Like {@link ShrinkingWeightCalculator}, but gives a static penalty for longer path lenghts, and filters out unwanted vertices
 * 
 * @author reuschling
 * @param <V>
 * @param <E>
 */
public class PathLengthShrinkingWeightCalculator<V extends Vertex, E extends Edge<V>> extends ShrinkingWeightCalculator<V, E>
{

    protected Set<Vertex> m_vertices2filter;

    protected float m_fPenalty = 0.3f;



    /**
     * Get the static penatly for longer path lenghts. This penalty will be added to each new hop weight.
     * 
     * @return the penalty
     */
    public float getPenalty()
    {
        return m_fPenalty;
    }



    /**
     * Set the static penatly for longer path lenghts. This penalty will be added to each new hop weight. Default is 0.3.
     * 
     * @param penalty
     * 
     * @return this as sugar
     */
    public PathLengthShrinkingWeightCalculator<V, E> setPenalty(float penalty)
    {
        m_fPenalty = penalty;

        return this;
    }



    public PathLengthShrinkingWeightCalculator()
    {
        m_vertices2filter = new HashSet<>();
    }



    public PathLengthShrinkingWeightCalculator(Set<Vertex> vertices2filter)
    {
        m_vertices2filter = vertices2filter;
    }



    @Override
    public float calculateWeight(V newEndVertex, E newEndEdge, CorePath<V, E> parentPath)
    {
        if(m_vertices2filter.contains(newEndVertex)) return Float.MAX_VALUE;
        if(newEndEdge instanceof WeightedEdge)
        {
            float fWeight4Edge = ((WeightedEdge<?>) newEndEdge).getWeight();

            if(fWeight4Edge == Float.MAX_VALUE) return Float.MAX_VALUE;

            return parentPath.getWeight() + fWeight4Edge + m_fPenalty;
        }

        return parentPath.getWeight() + m_fDefaultWeight + m_fPenalty;
    }







    public Set<Vertex> getVertices2filter()
    {
        return m_vertices2filter;
    }




    public void setVertices2filter(Set<Vertex> vertices2filter)
    {
        m_vertices2filter = vertices2filter;
    }


}
