package de.dfki.km.jade.graph.search;



import de.dfki.km.jade.graph.Edge;
import de.dfki.km.jade.graph.Entity;
import de.dfki.km.jade.graph.Vertex;



/**
 * The generalized interface for search interrupts, interrupting {@link EntitySearch}es. What exactly an interruption effects is on the
 * {@link EntitySearch} implementation. Instances of {@link SearchInterruptConfig} deals with the interrupt criterias such as maximum number of
 * results, walked path count, etc., making the decision if a search instance should be interrupted or not. The current status of these criterias are
 * again available through the {@link EntitySearch} instance that should be potentially interrupted.
 * 
 * @author reuschling
 * 
 * @param <V> the vertex type
 * @param <E> the entity type
 * @param <R> the result type
 */
public interface SearchInterruptConfig<V extends Vertex, E extends Edge<V>, R extends Entity>
{

    /**
     * An {@link EntitySearch} invokes this method asking if it should be interrupted or not. The method makes this decision according to the
     * criterias/stati that are available through the {@link EntitySearch} instance itself.
     * 
     * @param entitySearch the {@link EntitySearch} instance that asks for interruption
     * 
     * @return true in the case the search should be interrupted, false otherwise
     */
    public boolean interrupt(EntitySearch<V, E, R> entitySearch);



    /**
     * Can be invoked by an {@link EntitySearch}, indicating to the {@link SearchInterruptConfig} that the search has started.
     */
    public void startSearch();
}
