package de.dfki.km.jade.util;



import java.io.File;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.Map;

import org.mapdb.DBMaker;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.ReferenceResolver;
import com.esotericsoftware.kryo.util.Util;



@SuppressWarnings({ "rawtypes", "unchecked" })
public class MapDBReferenceResolver implements ReferenceResolver
{


    // diese Klasse kann man sich sparen, wenn man komplett mit identityHHashCode arbeitet - die Objekte selbst werden in der serialize-map für den schreibvorgang nicht
    // benötigt
    // private static class Value
    // {
    // private int hash;
    //
    // private Object val;
    //
    //
    //
    // public Value(Object val)
    // {
    // this.val = val;
    // this.hash = System.identityHashCode(val);
    // }
    //
    //
    //
    // @Override
    // public boolean equals(Object obj)
    // {
    // return val == ((Value) obj).val;
    // }
    //
    //
    //
    // @Override
    // public int hashCode()
    // {
    // return hash;
    // }
    //
    // }



    protected final ArrayList m_alReadObjects = new ArrayList();

    // protected Map map = new HashMap();
    // wenn man noch mehr memory sparen muß, können wir auch eine MapDB nehmen - wir probieren es erst mal so
    // protected TIntIntHashMap m_hsIdentityHashCode2ObjId = new TIntIntHashMap(Constants.DEFAULT_CAPACITY, Constants.DEFAULT_LOAD_FACTOR, Integer.MIN_VALUE,
    // Integer.MIN_VALUE);
    // wir müssen noch mehr sparen
    protected Map<Integer, Integer> m_hsIdentityHashCode2ObjId = DBMaker.newFileDB(new File("./tmpMapDbRemoveMeIfYouSeeMe")).transactionDisable()
            .mmapFileEnableIfSupported().asyncWriteEnable().cacheWeakRefEnable().closeOnJvmShutdown().deleteFilesAfterClose().make()
            .getTreeMap("hsIdentityHashCode2ObjId");

    protected Kryo m_kryo;



    //
    //
    // public void addReadObject(int id, Object object)
    // {
    // while (id >= readObjects.size())
    // readObjects.add(null);
    //
    // readObjects.set(id, object);
    // }




    public int addWrittenObject(Object object)
    {

        // das Teil kann noch nicht drin sein

        // Versuch mit identityHashCode
        // das wird mit der size gemacht, damit die id mit der deserialisierungs-arrayIndex-id aus den read-Methoden harmoniert. Wenn man einfach den identityHashCode
        // nehmen würde, wäre sie nicht mehr sequentiell - ein abspeichern unter einer entsprechenden array-position wäre schwierig.
        int id = m_hsIdentityHashCode2ObjId.size();

        m_hsIdentityHashCode2ObjId.put(System.identityHashCode(object), id);

        return id;


        // erste implementierung von mir, etwas besser als das Original. Die Value-Klasse kann man sich sparen, wenn man gleich mit identityHashCode arbeitet
        // Value v = new Value(object);
        //
        // int id = map.size();
        // map.put(v, id);
        //
        // return id;


        // original vom der kryo-issue seite, von der ich das template her hab
        // Value v = new Value(object);
        // Integer i = (Integer) map.get(v);
        // if(i == null)
        // {
        // int ret = map.size();
        // map.put(v, ret);
        //
        // return ret;
        // }
        // else
        // {
        // return i;
        // }
    }



    public Object getReadObject(Class type, int id)
    {
        return m_alReadObjects.get(id);
    }



    public int getWrittenId(Object object)
    {

        // Versuch mit identityHashCode
        Integer id = m_hsIdentityHashCode2ObjId.get(System.identityHashCode(object));

        // if(id == m_hsIdentityHashCode2ObjId.getNoEntryValue())
        if(id == null)
            return -1;
        else
            return id;

        // original vom der kryo-issue seite, von der ich das template her hab
        // Value v = new Value(object);
        // Integer i = (Integer) map.get(v);
        // if(i == null)
        // {
        // return -1;
        // }
        // else
        // {
        // return i;
        // }
    }



    public int nextReadId(Class type)
    {
        int id = m_alReadObjects.size();

        m_alReadObjects.add(null);

        return id;
    }



    public void reset()
    {
        m_alReadObjects.clear();

        m_hsIdentityHashCode2ObjId.clear();

        // original vom der kryo-issue seite, von der ich das template her hab
        // map.clear();
    }



    public void setKryo(Kryo kryo)
    {
        this.m_kryo = kryo;
    }



    public void setReadObject(int id, Object object)
    {
        m_alReadObjects.set(id, object);
    }



    /** Returns false for all primitive wrappers. */
    public boolean useReferences(Class type)
    {
        return !Util.isWrapperClass(type) && !type.equals(String.class) && !type.equals(Date.class) && !type.equals(BigDecimal.class) && !type.equals(BigInteger.class);
    }

}
