package de.dfki.km.jade.util;



import java.util.Comparator;
import java.util.Set;



/**
 * A {@link Comparator} implementation that uses the natural ordering of {@link Comparable}s, but ensures that objects that are NOT equal will not
 * 'considered as equal' and lost in {@link Set}s through the {@link Comparable#compareTo(Object)} results. For this, the class makes use of
 * {@link Object#equals(Object)} and {@link System#identityHashCode(Object)} of the Comparables.
 * 
 * @author reuschling
 * @param <T> 
 */
public class SaveNonEqualSetObjectsComparator<T> implements Comparator<T>
{

    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    public int compare(T o1, T o2)
    {
        
        int iOrder = ((Comparable)o1).compareTo((T) o2);

        if(iOrder != 0) return iOrder;
        
        if(o1.equals(o2)) return 0;
        
        
        return Integer.compare(System.identityHashCode(o1), System.identityHashCode(o2));
    }



}
