package de.dfki.km.academicresearch.eswc2016challenge.unittests.TaskTest;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.data.InputParam;
import de.dfki.km.academicresearch.eswc2016challenge.util.ConfusionMatrix;
import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Parameterized.class)
public class TaskOne {

    private static final Logger logger = LoggerFactory.getLogger(TaskOne.class);
    
    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        //pathFinder = new GraphML();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        
        pathFinder.setIndex("challenge_training_dataset", "training");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);   
            pathFinder.startStreaming();
            RDFDataMgr.parse(streamRDF, "training_dataset_filtered.nt.gz"); 
        }
        logger.debug("Finished Importing");
    }
    //========================================================================== 

    @Parameters(name = "Test {index}:  task1_q{0}_{1}.txt")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
            
            //{1, 8},             
            //{1, 344},            
            //{1, 1068},
            //{1, 20152},
            
            
            //{2, 3},
            //{2, 4},
            //{2, 79},
            //{2, 154},
            
            //{3, 36},
            //{3, 336},
            //{3, 4866},
            
            
            //{4, 2},
            //{4, 16},
            {4, 250},
            //{4, 1906},
            //{4, 20224}, 
            //{4, 175560}
            
        });
    }

    /*
    
    Wir sollen entweder:
    * die reihenfolgenbestimmung der challenge implementieren und sortieren
      (wenn es die denn gibt)
    * oder eine mengen vergleich machen
    damit wir das problem der richtigen reihenfolge bei 
    gleichlangen pfaden nicht mehr haben.
    
    JGraphT own algo output:
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/ontology/firstWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/property/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/ontology/firstWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/ontology/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/ontology/lastWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/property/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/ontology/lastWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/ontology/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/property/firstWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/property/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/property/firstWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/ontology/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/property/lastWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/property/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    (3) [http://dbpedia.org/resource/Felipe_Massa, http://dbpedia.org/property/after, http://dbpedia.org/resource/Robert_Kubica][http://dbpedia.org/resource/Robert_Kubica, http://dbpedia.org/property/lastWin, http://dbpedia.org/resource/2008_Canadian_Grand_Prix][http://dbpedia.org/resource/2008_Canadian_Grand_Prix, http://dbpedia.org/ontology/thirdTeam, http://dbpedia.org/resource/Red_Bull]
    */
    
    //onto
        //fistwin
        //lastwin
        //thirdteam
    //prop
        //firstwin
        //lastwin
        //thirdteam
    
    
    

    private int query;
    private int k;    
    
    public TaskOne(int query, int k) {
        this.query = query;
        this.k = k;
    }

    @Test
    public void test() throws FileNotFoundException, IOException, Exception {

        //load file
        File f = new File("./challenge_training_result_sets/task1_q" + query + "_" + k + ".txt");
        Assert.assertTrue("File with expected results should be existing", f.exists());

        //parameters
        InputParam params = queryToParam.get(query);
        String start = params.getStart();
        String end = params.getEnd();

        //actual and expected
        PrintHeapUsage.gcAndprintGB();
        List<Path> actual = pathFinder.find(start, end, k, null);
        PrintHeapUsage.gcAndprintGB();
        
        PathList listExpected = new PathList(f);
        PathList listActual = new PathList(actual);
        
        String task = "task1_q" + query + "_" + k;
        
        //WriteComparableFile.writeSingleFile(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        //WriteComparableFile.writeFileToAll(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        
        //Util.equalsSlow(listExpected, listActual);
        
        ConfusionMatrix m = new ConfusionMatrix();
        m.compare(listExpected, listActual);
        //m.writeSingleFile(new File("./challenge_training_dataset/"), task);
        //m.printConsole(task);
        m.assertIt();
    }
    
    
    
    //PARAM
    public static Map<Integer, InputParam> queryToParam = generateParam();
    private static Map<Integer, InputParam> generateParam() {
        Map<Integer, InputParam> map = new HashMap<>();
        map.put(1, new InputParam("http://dbpedia.org/resource/Felipe_Massa", "http://dbpedia.org/resource/Red_Bull", null));
        map.put(2, new InputParam("http://dbpedia.org/resource/1952_Winter_Olympics", "http://dbpedia.org/resource/Elliot_Richardson", null));
        map.put(3, new InputParam("http://dbpedia.org/resource/Karl_W._Hofmann", "http://dbpedia.org/resource/Elliot_Richardson", null));
        map.put(4, new InputParam("http://dbpedia.org/resource/James_K._Polk", "http://dbpedia.org/resource/Felix_Grundy", null));
        return map;
    }
}
