package de.dfki.km.academicresearch.eswc2016challenge.unittests.TaskTest;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.data.InputParam;
import de.dfki.km.academicresearch.eswc2016challenge.util.ConfusionMatrix;
import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Parameterized.class)
public class TaskTwo {

    private static final Logger logger = LoggerFactory.getLogger(TaskTwo.class);

    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;

    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");

        //pathFinder = new DefaultPathFinder();
        //pathFinder = new JadePathFinder();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "training");
        if (pathFinder.hasIndex()) {
            pathFinder.loadIndex();
        } else {
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);
            pathFinder.startStreaming();
            RDFDataMgr.parse(streamRDF, "training_dataset_filtered.nt.gz");
        }
    }
    //========================================================================== 

    @Parameters(name = "Test {index}:  task2_q{0}_{1}.txt")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
            //{1, 32},
            //{1, 98},
            //{1, 1914},
            //{1, 16632},
            //{1, 212988},
            
            //{2, 3},
            //{2, 4},
            //{2, 76},
            //{2, 151},
            //{2, 2311},
            
            //{3, 12},
            //{3, 76},
            //{3, 1440},
            //{3, 8088},
            
            //{4, 1},
            //{4, 6},
            //{4, 72},
            //{4, 614},
            //{4, 5483},
            //{4, 52649},
            {4, 471199}
        });
    }

    

    private int query;
    private int k;

    public TaskTwo(int query, int k) {
        this.query = query;
        this.k = k;
    }

    @Test
    public void test() throws Exception {

        //load file
        File f = new File("./challenge_training_result_sets/task2_q" + query + "_" + k + ".txt");
        Assert.assertTrue("File with expected results should be existing", f.exists());

        //parameters
        InputParam params = queryToParam.get(query);
        String start = params.getStart();
        String end = params.getEnd();
        String P = params.getP();

        //actual and expected
        PrintHeapUsage.gcAndprintGB();
        List<Path> actual = pathFinder.find(start, end, k, P);
        PrintHeapUsage.gcAndprintGB();
        
        PathList listActual = new PathList(actual);
        PathList listExpected = new PathList(f);

        String task = "task1_q" + query + "_" + k;
        
        //WriteComparableFile.writeSingleFile(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        //WriteComparableFile.writeFileToAll(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        
        //Util.equalsSlow(listExpected, listActual);
        
        ConfusionMatrix m = new ConfusionMatrix();
        m.compare(listExpected, listActual);
        //m.writeSingleFile(new File("./challenge_training_dataset/"), task);
        //m.printConsole(task);
        m.assertIt();

    }
    
    
    
    
    
    public static Map<Integer, InputParam> queryToParam = generateParam();
    private static Map<Integer, InputParam> generateParam() {
        Map<Integer, InputParam> map = new HashMap<>();
        map.put(1, new InputParam("http://dbpedia.org/resource/Felipe_Massa", "http://dbpedia.org/resource/Red_Bull", "http://dbpedia.org/property/firstWin"));
        map.put(2, new InputParam("http://dbpedia.org/resource/1952_Winter_Olympics", "http://dbpedia.org/resource/Elliot_Richardson", "http://dbpedia.org/property/after"));
        map.put(3, new InputParam("http://dbpedia.org/resource/Karl_W._Hofmann", "http://dbpedia.org/resource/Elliot_Richardson", "	http://dbpedia.org/property/predecessor"));
        map.put(4, new InputParam("http://dbpedia.org/resource/James_K._Polk", "http://dbpedia.org/resource/Felix_Grundy", "http://dbpedia.org/ontology/president"));
        return map;
    }
}
