package de.dfki.km.academicresearch.eswc2016challenge.unittests.data;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import org.junit.Assert;
import org.junit.Test;

public class PathTests {

    @Test
    public void pathSerialization() throws Exception {

        Path p1 = new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B);
        
        Assert.assertEquals(5, p1.resourceCount());
        Assert.assertEquals(2, p1.length());
        
        String s = p1.toString();

        Path p2 = new Path(s);

        Assert.assertTrue(Util.equals(p1, p2));
    }

    @Test
    public void pathFile() throws Exception {

        List<Path> pathes = Util.getPaths(new File("./challenge_training_result_sets/task1_q1_8.txt"));
        
        Assert.assertEquals(8, pathes.size());
    }
    
    @Test
    public void pathListTestOne() throws Exception {

        PathList l = new PathList();
        l.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        l.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.B));
        l.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        
    }
    
    @Test(expected=Exception.class)
    public void pathlistTest() throws Exception {

        PathList l = new PathList();
        l.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        l.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
    }
    
    
    //============Pathlist equals
    
    @Test
    public void pathlistShouldBeEqualExactSame() throws Exception {

        PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));        
        
        Assert.assertTrue(Util.equalsSlow(expected, actualOne)); 
    }
    
    @Test
    public void pathlistShouldBeEqualButDifferentPath() throws Exception {

        PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        
        Assert.assertTrue(Util.equalsSlow(expected, actualOne)); 
    }
    
    @Test(expected=AssertionError.class)
    public void pathlistTooFew() throws Exception {

        PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        
        Util.equalsSlow(expected, actualOne); 
    }
    
    @Test(expected=AssertionError.class)
    public void pathlistTooMuch() throws Exception {

        PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        
        Util.equalsSlow(expected, actualOne); 
    }
    
    @Test(expected=AssertionError.class)
    public void pathlistMissingPathLength() throws Exception {

    	PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));       
        
        Assert.assertTrue(Util.equalsSlow(expected, actualOne)); 
    }
    
    @Test(expected=AssertionError.class)
    public void pathlistTooManyPathLength() throws Exception {

    	PathList expected = new PathList();
        expected.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        
        PathList actualOne = new PathList();
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        actualOne.addWithOrderCheck(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p6, ExampleDataset.B));        
        
        Assert.assertTrue(Util.equalsSlow(expected, actualOne)); 
    }
    
    
}
