package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.EppsteinDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EppteinTest {

    private static final Logger logger = LoggerFactory.getLogger(EppteinTest.class);

    private static Model datasetD1 = EppsteinDataset.getWeightedDataSet();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "eppsteinDataset");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);  
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD1.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 
    
    //============= task one ===================================================

    @Test
    public void TaskOneTestOne() throws Exception {
    	
        List<Path> actual;
    	   	
        List<Path> expected = new LinkedList<>();
        //55 -no sideedge
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_e,  EppsteinDataset.e,  EppsteinDataset.e_f, EppsteinDataset.f,  EppsteinDataset.f_g, EppsteinDataset.g, EppsteinDataset.g_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //58 - side edge 3
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_c, EppsteinDataset.c,  EppsteinDataset.c_d, EppsteinDataset.d, EppsteinDataset.d_h, EppsteinDataset.h, EppsteinDataset.h_l, EppsteinDataset.l));
        //59 - side edge 3,1
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_c, EppsteinDataset.c,  EppsteinDataset.c_g, EppsteinDataset.g, EppsteinDataset.g_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //61 - side edge 6
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_e,  EppsteinDataset.e,  EppsteinDataset.e_f, EppsteinDataset.f,  EppsteinDataset.f_j, EppsteinDataset.j, EppsteinDataset.j_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //62 - side edge 3,4
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_f, EppsteinDataset.f,  EppsteinDataset.f_g, EppsteinDataset.g, EppsteinDataset.g_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //64 - side edge 9
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_e,  EppsteinDataset.e,  EppsteinDataset.e_f, EppsteinDataset.f,  EppsteinDataset.f_g, EppsteinDataset.g, EppsteinDataset.g_h, EppsteinDataset.h, EppsteinDataset.h_l, EppsteinDataset.l));
        //65 - side edge 10
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_e,  EppsteinDataset.e,  EppsteinDataset.e_i, EppsteinDataset.i,  EppsteinDataset.i_j, EppsteinDataset.j, EppsteinDataset.j_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //68 - side edge 3,4,6
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_f, EppsteinDataset.f,  EppsteinDataset.f_j, EppsteinDataset.j, EppsteinDataset.j_k, EppsteinDataset.k, EppsteinDataset.k_l, EppsteinDataset.l));
        //68 - side edge 3,1,9
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_c, EppsteinDataset.c,  EppsteinDataset.c_g, EppsteinDataset.g, EppsteinDataset.g_h, EppsteinDataset.h, EppsteinDataset.h_l, EppsteinDataset.l));
        //71 - side edge 3,4,9
        expected.add(new Path(EppsteinDataset.a, EppsteinDataset.a_b,  EppsteinDataset.b,  EppsteinDataset.b_f, EppsteinDataset.f,  EppsteinDataset.f_g, EppsteinDataset.g, EppsteinDataset.g_h, EppsteinDataset.h, EppsteinDataset.h_l, EppsteinDataset.l));
        
        
        actual = pathFinder.find(EppsteinDataset.a.getURI(), EppsteinDataset.l.getURI(), 10, null);
        Util.equalsSlow(new PathList(expected), new PathList(actual));
        
        
        
        actual = pathFinder.find(EppsteinDataset.a.getURI(), EppsteinDataset.l.getURI(), 20, null);//more paths are not possible, so return only 10
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }



}
