package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.EppsteinDataset2;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EppteinTest2 {

    private static final Logger logger = LoggerFactory.getLogger(EppteinTest2.class);

    private static Model eppsteinDataset2 = EppsteinDataset2.getWeightedDataSet();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "eppsteinDataset2");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);  
            pathFinder.startStreaming();
            StreamOps.graphToStream(eppsteinDataset2.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 
    
    //============= task one ===================================================

    @Test
    public void TaskOneTestOne() throws Exception {
    	
    	   	
        List<Path> expected = new LinkedList<>();
        //SHortest
        expected.add(new Path(EppsteinDataset2.a, EppsteinDataset2.a_p,  EppsteinDataset2.p,  EppsteinDataset2.p_q, EppsteinDataset2.q,  EppsteinDataset2.q_r, EppsteinDataset2.r, EppsteinDataset2.r_t, EppsteinDataset2.t, EppsteinDataset2.t_z, EppsteinDataset2.z));
        
        //choose first p
        expected.add(new Path(EppsteinDataset2.a, EppsteinDataset2.a_p,  EppsteinDataset2.p,  EppsteinDataset2.p_j, EppsteinDataset2.j,  EppsteinDataset2.j_q, EppsteinDataset2.q, EppsteinDataset2.q_r, EppsteinDataset2.r, EppsteinDataset2.r_t, EppsteinDataset2.t, EppsteinDataset2.t_z, EppsteinDataset2.z));
        
        //choose 4 from t
        expected.add(new Path(EppsteinDataset2.a, EppsteinDataset2.a_p,  EppsteinDataset2.p,  EppsteinDataset2.p_q, EppsteinDataset2.q,  EppsteinDataset2.q_r, EppsteinDataset2.r, EppsteinDataset2.r_t, EppsteinDataset2.t, EppsteinDataset2.t_b, EppsteinDataset2.b, EppsteinDataset2.b_z, EppsteinDataset2.z));
      
        //choose 1, 4
        expected.add(new Path(EppsteinDataset2.a, EppsteinDataset2.a_p,  EppsteinDataset2.p,  EppsteinDataset2.p_j, EppsteinDataset2.j,  EppsteinDataset2.j_q, EppsteinDataset2.q,  EppsteinDataset2.q_r, EppsteinDataset2.r, EppsteinDataset2.r_t, EppsteinDataset2.t, EppsteinDataset2.t_b, EppsteinDataset2.b, EppsteinDataset2.b_z, EppsteinDataset2.z));
        
        
        List<Path> actual = pathFinder.find(EppsteinDataset2.a.getURI(), EppsteinDataset2.z.getURI(), 4, null);
        
        
        PathList listExpected = new PathList(expected);
        PathList listActual = new PathList(actual);
        
        Util.equalsSlow(listExpected, listActual);  
    }



}
