package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.jgraphturiedge.EppsteinV2;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/***
 * This is a check for visting the edges correctly
 * @author Sven
 *
 */
public class ExampleTests2 {
	
	private static final Logger logger = LoggerFactory.getLogger(ExampleTests2.class);

    private static Model datasetD2 = ExampleDataset.getDataSetD2();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        //pathFinder = new DefaultPathFinder();
        //pathFinder = new JadePathFinder();
        //pathFinder = new JenaPathFinder();
        //pathFinder = new JGraphTFinder();
        //pathFinder = new YqiPathFinder();
        //pathFinder = new EppsteinV2();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "datasetD2");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);  
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD2.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 
    
    
    //Dont use only edge for checking if triple is visited

    @Test
    public void TaskOneTestOne() throws Exception { 

    	//k = 4
        //A nach B    	
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));

        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 4, null);
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

    @Test
    public void TaskOneTestTwo() throws Exception {

    	//k = 3
        //A nach B    	
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u10, ExampleDataset.p4, ExampleDataset.B));

        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 3, null);
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

}
