package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * This is a multigraph example
 * @author Sven
 *
 */
public class ExampleTests3 {
	
    private static final Logger logger = LoggerFactory.getLogger(ExampleTests3.class);

    private static Model datasetD3 = ExampleDataset.getDataSetD3();
    
    
    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "datasetD3");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder); 
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD3.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 
    
    
    //Also multigraph!

    //@Test
    public void First2Pathes() throws Exception {
    	 	
    	testPathes(2);
    	
    }
    
    //@Test
    public void First3Pathes() throws Exception {
    	
    	testPathes(3);
    	
    }
    
    //@Test
    public void First11Pathes() throws Exception {
    	testPathes(11);
    }
    
    
    //@Test
    public void First19Pathes() throws Exception {
    	testPathes(19);
    }
    
    
    @Test
    public void First20Pathes() throws Exception {    	
    	List<Path> expected = fullExpected().subList(0, 19);        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 20, null);
        
        PathList listExpected = new PathList(expected);
        PathList listActual = new PathList(actual);
        
        Util.equalsSlow(listExpected, listActual);    	  	
    }
    
    
    private void testPathes(int count) throws Exception{
    	
    	List<Path> expected = fullExpected().subList(0, count);        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), count, null);
        
        PathList listExpected = new PathList(expected);
        PathList listActual = new PathList(actual);
        
        Util.equalsSlow(listExpected, listActual);    	
    }
    
    private List<Path> fullExpected(){
    	
    	List<Path> expected = new LinkedList<>();
    	
    	//5 lang
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.B));
    	//2 pfade    	
    	
    	//7 lang
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.B));
    	//3pfade
    	
    	//11 lang
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	//11pfade
    	
    	//17 lang
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p10, ExampleDataset.u4, ExampleDataset.p11, ExampleDataset.u5, ExampleDataset.p12, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
    	//19
    	
    	return expected;
    	
    }

}
