package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * This is a multigraph example
 *
 * @author Sven
 *
 */
public class ExampleTests5 {

    private static final Logger logger = LoggerFactory.getLogger(ExampleTests5.class);

    private static Model datasetD5 = ExampleDataset.getDataSetD5();

    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;

    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");

        pathFinder = PathChooser.getCurrentPathFinder();

        pathFinder.setIndex("challenge_training_dataset", "datasetD5");
        if (pathFinder.hasIndex()) {
            pathFinder.loadIndex();
        } else {
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD5.getGraph(), streamRDF);
        }
    }
    //========================================================================== 

    @Test
    public void TaskOneTestOne() throws Exception {

        //          u2
        //         | ^
        //      p3 | | p4
        //         v |
        // A -p1->  u1  -p2-> B
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));

        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 2, null);
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

    @Test
    public void TaskOneTestTwo() throws Exception {

        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));

        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 3, null);    //one more which is not possible    
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

}
