package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.data.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.util.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import de.dfki.km.academicresearch.eswc2016challenge.util.WriteComparableFile;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * This is a multigraph example
 * @author Sven
 *
 */
public class ExampleTests8 {
	
	private static final Logger logger = LoggerFactory.getLogger(ExampleTests8.class);

    private static Model datasetD8 = ExampleDataset.getDataSetD8();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "datasetD8");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder); 
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD8.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 

//  u2
//        | | | ^ ^ ^     
//  6 7 8 | | | | | |  3 4 5
//        v v v | | |   
// A -p1->    u1     -p2-> B
    
    
    
    @Test
    public void First1Pathes() throws Exception {
    	 	
    	testPathes(1);
    	
    }
    
    @Test
    public void First10Pathes() throws Exception {
    	
    	testPathes(10);
    	
    }
    
    @Test
    public void First46Pathes() throws Exception {
    	testPathes(46);
    }
    
    
    @Test
    public void First82Pathes() throws Exception {
    	testPathes(82);
    }
    
    
    @Test
    public void First83Pathes() throws Exception {    	
    	List<Path> expected = fullExpected().subList(0, 82);        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 83, null);
        
        PathList listExpected = new PathList(expected);
        PathList listActual = new PathList(actual);
        
        Util.equalsSlow(listExpected, listActual);    	  	
    }
    
    
    private void testPathes(int count) throws Exception{
    	
    	List<Path> expected = fullExpected().subList(0, count);        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), count, null);
        
        PathList listExpected = new PathList(expected);
        PathList listActual = new PathList(actual);
        
        Util.equalsSlow(listExpected, listActual);    	
    }
    
    /*
    public void TaskOneTestOne() throws Exception {
    	
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 200, null);
        
        PathList listActual = new PathList(actual);
        
        //Util.equalsSlow(listExpected, listActual);   
    	    	
        
        String s = listActual.toSourceCode("ExampleDataset");        
        Files.write(Paths.get("./sourceCode.txt"), s.getBytes());
        
        logger.debug("test");
        //Util.equalsSlow(new PathList(expected), new PathList(actual));
    }
    */
    
    
    
    
    
    private List<Path> fullExpected(){
    	
    	List<Path> expected = new LinkedList<>();
    	
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	//1
    	
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	//10
    	
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	//46
    	
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p7, ExampleDataset.u1, ExampleDataset.p3, ExampleDataset.u2, ExampleDataset.p6, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
    	//82

    	
    	return expected;
    	
    }

}
