package de.dfki.km.academicresearch.eswc2016challenge.unittests.onePath;

import de.dfki.km.academicresearch.eswc2016challenge.data.Path;
import de.dfki.km.academicresearch.eswc2016challenge.pathfinder.basis.PathFinder;

import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.TaskTest.TaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.util.PrintHeapUsage;
import de.dfki.km.academicresearch.eswc2016challenge.util.Util;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public class OneShortestPathTest {
	
private static final Logger logger = LoggerFactory.getLogger(TaskOne.class);
    
    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        PrintHeapUsage.gcAndprintGB();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "training");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);   
            pathFinder.startStreaming();
            RDFDataMgr.parse(streamRDF, "training_dataset_filtered.nt.gz"); 
        }
        PrintHeapUsage.gcAndprintGB();
        logger.debug("Finished Importing");
    }
    //========================================================================== 

    
    
    
    @Test
    public void q1() throws FileNotFoundException, IOException, Exception {

    	File f = new File("./challenge_training_result_sets/task1_q1_344.txt");
    	List<Path> expected = Util.getPaths(f);
    	List<Path> allequalWeight = expected.subList(0, 8);
    	
    	List<Path> actual = pathFinder.find("http://dbpedia.org/resource/Felipe_Massa", "http://dbpedia.org/resource/Red_Bull", 1, null);
    	
    	Assert.assertTrue("Not contained", allequalWeight.containsAll(actual));
    }
    
    @Test
    public void q2() throws FileNotFoundException, IOException, Exception {

    	File f = new File("./challenge_training_result_sets/task1_q2_79.txt");
    	List<Path> expected = Util.getPaths(f);
    	List<Path> allequalWeight = expected.subList(0, 3);

    	List<Path> actual = pathFinder.find("http://dbpedia.org/resource/1952_Winter_Olympics", "http://dbpedia.org/resource/Elliot_Richardson", 1, null);
    	
    	Assert.assertTrue("Not contained", allequalWeight.containsAll(actual));
    }
    
    @Test
    public void q3() throws FileNotFoundException, IOException, Exception {

    	File f = new File("./challenge_training_result_sets/task1_q3_336.txt");
    	List<Path> expected = Util.getPaths(f);
    	List<Path> allequalWeight = expected.subList(0, 36);
    	
    	List<Path> actual = pathFinder.find("http://dbpedia.org/resource/Karl_W._Hofmann", "http://dbpedia.org/resource/Elliot_Richardson", 1, null);
    	
    	Assert.assertTrue("Not contained", allequalWeight.containsAll(actual));
    }
    
    @Test
    public void q4() throws FileNotFoundException, IOException, Exception {

    	File f = new File("./challenge_training_result_sets/task1_q4_250.txt");
    	List<Path> expected = Util.getPaths(f);
    	List<Path> allequalWeight = expected.subList(0, 2);
    	
    	List<Path> actual = pathFinder.find("http://dbpedia.org/resource/James_K._Polk", "http://dbpedia.org/resource/Felix_Grundy", 1, null);
    	
    	Assert.assertTrue("Not contained", allequalWeight.containsAll(actual));
    }
    
}
